package vennDIS;

// Copyright (C) 2014 Vladimir Ignatchenko (vladimirsign@gmail.com)
// Dr. Thomas Kislinger laboratory (http://kislingerlab.uhnres.utoronto.ca/)
//
// This file is part of VennDIS software.
// VennDIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VennDIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VennDIS. If not, see <http://www.gnu.org/licenses/>.

import javafx.application.Platform;
import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.GridPane;
import javafx.scene.control.Button;
import javafx.scene.control.Toggle;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.control.Tooltip;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.control.Label;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Separator;
import javafx.scene.control.CheckBox;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.paint.Color;
import javafx.geometry.Insets;
import javafx.beans.property.SimpleStringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.beans.value.ObservableValue;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellEditEvent;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.scene.control.ProgressBar;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.control.Separator;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.scene.input.Clipboard;
import javafx.event.EventHandler;
import javafx.util.Callback;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import javafx.stage.FileChooser;


public class DataInputWindow {
	private Stage dataInputStage;
	private BorderPane dataInputMainBP;
	private BorderPane bottomBP;
	private BorderPane dbox;
	private VennCalc vennCalc;
	private Image imOK, imLists, imTable, imValues, imFasta;
	private Image imVenn2, imVenn3, imVenn4, imVenn5;
	private Image imEuler001, imEuler003, imEuler005, imEuler006, imEuler011, imEuler015, imEuler021, imEuler023, imEuler027, imEuler029, imEuler031, imEuler038, imEuler039, imEuler043, imEuler046, imEuler047, imEuler056, imEuler057, imEuler058, imEuler059, imEuler063, imEuler069, imEuler070, imEuler071, imEuler075, imEuler077, imEuler078, imEuler079, imEuler081, imEuler083, imEuler085, imEuler087, imEuler088, imEuler089, imEuler090, imEuler091, imEuler093, imEuler094, imEuler095, imEuler098, imEuler099, imEuler102, imEuler103, imEuler104, imEuler105, imEuler106, imEuler107, imEuler109, imEuler110, imEuler111, imEuler115, imEuler119, imEuler120, imEuler121, imEuler122, imEuler123, imEuler125, imEuler126;
	public ToggleGroup vennGroup; 
	public Button btnOK;
	private ToggleButton btnLists, btnTable, btnValues, btnFasta;
	private final Clipboard clipboard = Clipboard.getSystemClipboard();
	private TableView<String[]> table;
	protected ComboBox<String> cbxID, cbxA, cbxB, cbxC, cbxD, cbxE;
	private GridPane dataInputGridPane;
	private ArrayList<ArrayList> dataArray;
	private CheckBox headerCB, idCB;
	private Fasta fastaFileA, fastaFileB, fastaFileC, fastaFileD, fastaFileE;
	private TextArea textAreaA, textAreaB, textAreaC, textAreaD, textAreaE;
	private CheckBox cbImportListA, cbImportListB, cbImportListC, cbImportListD, cbImportListE; 
	protected ProgressBar pbFastaSearch;
	protected Text pbText;
	private	Thread peptideSearchThread;
	private String currentDirectory = "./";
	private CheckBox cbLI;

	public DataInputWindow(VennCalc vCalc) {
		vennCalc = vCalc;
		try {
			imOK = new Image(getClass().getResourceAsStream("icons/OK.png"));
			imLists = new Image(getClass().getResourceAsStream("icons/lists.png"));
			imTable = new Image(getClass().getResourceAsStream("icons/table.png"));
			imValues = new Image(getClass().getResourceAsStream("icons/values.png"));
			imFasta = new Image(getClass().getResourceAsStream("icons/fasta.png"));
			imVenn2 = new Image(getClass().getResourceAsStream("icons/venn2x_icon.png"));
			imVenn3 = new Image(getClass().getResourceAsStream("icons/venn3x_icon.png"));
			imVenn4 = new Image(getClass().getResourceAsStream("icons/venn4x_icon.png"));
			imVenn5 = new Image(getClass().getResourceAsStream("icons/venn5x_icon.png"));
			imEuler001 = new Image(getClass().getResourceAsStream("icons/euler001_icon.png"));
			imEuler003 = new Image(getClass().getResourceAsStream("icons/euler003_icon.png"));
			imEuler005 = new Image(getClass().getResourceAsStream("icons/euler005_icon.png"));
			imEuler006 = new Image(getClass().getResourceAsStream("icons/euler006_icon.png"));
			imEuler011 = new Image(getClass().getResourceAsStream("icons/euler011_icon.png"));
			imEuler015 = new Image(getClass().getResourceAsStream("icons/euler015_icon.png"));
			imEuler021 = new Image(getClass().getResourceAsStream("icons/euler021_icon.png"));
			imEuler023 = new Image(getClass().getResourceAsStream("icons/euler023_icon.png"));
			imEuler027 = new Image(getClass().getResourceAsStream("icons/euler027_icon.png"));
			imEuler029 = new Image(getClass().getResourceAsStream("icons/euler029_icon.png"));
			imEuler031 = new Image(getClass().getResourceAsStream("icons/euler031_icon.png"));
			imEuler038 = new Image(getClass().getResourceAsStream("icons/euler038_icon.png"));
			imEuler039 = new Image(getClass().getResourceAsStream("icons/euler039_icon.png"));
			imEuler043 = new Image(getClass().getResourceAsStream("icons/euler043_icon.png"));
			imEuler046 = new Image(getClass().getResourceAsStream("icons/euler046_icon.png"));
			imEuler047 = new Image(getClass().getResourceAsStream("icons/euler047_icon.png"));
			imEuler056 = new Image(getClass().getResourceAsStream("icons/euler056_icon.png"));
			imEuler057 = new Image(getClass().getResourceAsStream("icons/euler057_icon.png"));
			imEuler058 = new Image(getClass().getResourceAsStream("icons/euler058_icon.png"));
			imEuler059 = new Image(getClass().getResourceAsStream("icons/euler059_icon.png"));
			imEuler063 = new Image(getClass().getResourceAsStream("icons/euler063_icon.png"));
			imEuler069 = new Image(getClass().getResourceAsStream("icons/euler069_icon.png"));
			imEuler070 = new Image(getClass().getResourceAsStream("icons/euler070_icon.png"));
			imEuler071 = new Image(getClass().getResourceAsStream("icons/euler071_icon.png"));
			imEuler075 = new Image(getClass().getResourceAsStream("icons/euler075_icon.png"));
			imEuler077 = new Image(getClass().getResourceAsStream("icons/euler077_icon.png"));
			imEuler078 = new Image(getClass().getResourceAsStream("icons/euler078_icon.png"));
			imEuler079 = new Image(getClass().getResourceAsStream("icons/euler079_icon.png"));
			imEuler081 = new Image(getClass().getResourceAsStream("icons/euler081_icon.png"));
			imEuler083 = new Image(getClass().getResourceAsStream("icons/euler083_icon.png"));
			imEuler085 = new Image(getClass().getResourceAsStream("icons/euler085_icon.png"));
			imEuler087 = new Image(getClass().getResourceAsStream("icons/euler087_icon.png"));
			imEuler088 = new Image(getClass().getResourceAsStream("icons/euler088_icon.png"));
			imEuler089 = new Image(getClass().getResourceAsStream("icons/euler089_icon.png"));
			imEuler090 = new Image(getClass().getResourceAsStream("icons/euler090_icon.png"));
			imEuler091 = new Image(getClass().getResourceAsStream("icons/euler091_icon.png"));
			imEuler093 = new Image(getClass().getResourceAsStream("icons/euler093_icon.png"));
			imEuler094 = new Image(getClass().getResourceAsStream("icons/euler094_icon.png"));
			imEuler095 = new Image(getClass().getResourceAsStream("icons/euler095_icon.png"));
			imEuler098 = new Image(getClass().getResourceAsStream("icons/euler098_icon.png"));
			imEuler099 = new Image(getClass().getResourceAsStream("icons/euler099_icon.png"));
			imEuler102 = new Image(getClass().getResourceAsStream("icons/euler102_icon.png"));
			imEuler103 = new Image(getClass().getResourceAsStream("icons/euler103_icon.png"));
			imEuler104 = new Image(getClass().getResourceAsStream("icons/euler104_icon.png"));
			imEuler105 = new Image(getClass().getResourceAsStream("icons/euler105_icon.png"));
			imEuler106 = new Image(getClass().getResourceAsStream("icons/euler106_icon.png"));
			imEuler107 = new Image(getClass().getResourceAsStream("icons/euler107_icon.png"));
			imEuler109 = new Image(getClass().getResourceAsStream("icons/euler109_icon.png"));
			imEuler110 = new Image(getClass().getResourceAsStream("icons/euler110_icon.png"));
			imEuler111 = new Image(getClass().getResourceAsStream("icons/euler111_icon.png"));
			imEuler115 = new Image(getClass().getResourceAsStream("icons/euler115_icon.png"));
			imEuler119 = new Image(getClass().getResourceAsStream("icons/euler119_icon.png"));
			imEuler120 = new Image(getClass().getResourceAsStream("icons/euler120_icon.png"));
			imEuler121 = new Image(getClass().getResourceAsStream("icons/euler121_icon.png"));
			imEuler122 = new Image(getClass().getResourceAsStream("icons/euler122_icon.png"));
			imEuler123 = new Image(getClass().getResourceAsStream("icons/euler123_icon.png"));
			imEuler125 = new Image(getClass().getResourceAsStream("icons/euler125_icon.png"));
			imEuler126 = new Image(getClass().getResourceAsStream("icons/euler126_icon.png"));
		} catch(Exception e) {
			e.printStackTrace();
		}
		dataInputStage = new Stage();
		dataInputStage.setTitle("Input data...");
		dataInputMainBP = new BorderPane();
		dataInputMainBP.setStyle("-fx-background-color: linear-gradient(to bottom, derive(-fx-base,-30%), derive(-fx-base,-60%)), linear-gradient(to bottom, black 0%, gray 100%);");
		Scene dataInputMainScene = new Scene(dataInputMainBP, 1280, 600);
		dataInputMainBP.prefHeightProperty().bind(dataInputMainScene.heightProperty());
		dataInputMainBP.prefWidthProperty().bind(dataInputMainScene.widthProperty());

		bottomBP = new BorderPane();
		HBox selectInputModeHBox = new HBox();
		selectInputModeHBox.setSpacing(5);
		btnOK = new Button("OK");
		btnOK.setGraphic(new ImageView(imOK));
		btnOK.setTooltip(new Tooltip("OK"));
		ToggleGroup inputGroup = new ToggleGroup();
		btnLists = new ToggleButton("Lists");
		btnLists.setGraphic(new ImageView(imLists));
		btnLists.setToggleGroup(inputGroup);
		btnLists.setUserData("lists");
		btnLists.setSelected(true);
		btnTable = new ToggleButton("Table");
		btnTable.setGraphic(new ImageView(imTable));
		btnTable.setToggleGroup(inputGroup);
		btnTable.setUserData("table");
		btnValues = new ToggleButton("Values");
		btnValues.setGraphic(new ImageView(imValues));
		btnValues.setToggleGroup(inputGroup);
		btnValues.setUserData("values");
		btnFasta = new ToggleButton("Fasta");
		btnFasta.setGraphic(new ImageView(imFasta));
		btnFasta.setToggleGroup(inputGroup);
		btnFasta.setUserData("fasta");
		inputGroup.selectedToggleProperty().addListener(e->{ 
			if (inputGroup.getSelectedToggle() != null) {
				String inputType = (String)inputGroup.getSelectedToggle().getUserData();
				switch (inputType) {
					case "table":
						vennCalc.clearVennCalc();
						setTableInput();
						break;
					case "lists":
						vennCalc.clearVennCalc();
						setListsInput();
						break;
					case "values":
						vennCalc.clearVennCalc();
						setValuesInput();
						break;
					case "fasta":
						vennCalc.clearVennCalc();
						setFastaInput();
						break;
				}
			}
		});
		selectInputModeHBox.getChildren().addAll(btnLists, btnTable, btnValues, btnFasta, btnOK);
		bottomBP.setRight(selectInputModeHBox);
		dataInputMainBP.setBottom(bottomBP);
		setListsInput();
		dataInputStage.setScene(dataInputMainScene);
		dataInputStage.show();
	}

	private void setFastaInput() {
		GridPane dbox = new GridPane();
		dbox.setHgap(5);
		dbox.setVgap(5);
		Text peptidesText = new Text("Peptide sequences: ");
		peptidesText.setFill(Color.WHITE);
		peptidesText.setFont(Font.font(14));
		dbox.add(peptidesText, 1, 1);
		textAreaA = new TextArea() {
	            @Override
	            public void paste() {
	                Clipboard clipboard = Clipboard.getSystemClipboard();
	                if (clipboard.hasString()) {
	                   replaceSelection(clipboard.getString().replaceAll("\r\n|\r|\n", "\r\n"));
	                }
	            }
	        };
		textAreaA.setWrapText(false);
		textAreaA.setPrefHeight(1000);
		textAreaA.setPrefWidth(400);

		dbox.add(textAreaA, 1, 2, 1, 18);
		Text fastaText = new Text("Select fasta files: ");
		fastaText.setFill(Color.WHITE);
		fastaText.setFont(Font.font(14));
		dbox.add(fastaText, 2, 1, 2, 1);

		TextField datasetHeaderA = new TextField();
		datasetHeaderA.setPromptText("Fasta file A");
		datasetHeaderA.textProperty().addListener(e->{ vennCalc.headerA = datasetHeaderA.getText(); });
		dbox.add(datasetHeaderA, 2, 2, 2, 1);
		Button btnFastaA = new Button("Browse");
		dbox.add(btnFastaA, 2, 3);
		TextField pathFastaA = new TextField();
		pathFastaA.setText("..");
		pathFastaA.setPrefWidth(750);
		pathFastaA.setEditable(false);
		dbox.add(pathFastaA, 3, 3);
		Separator fastaSeparatorA = new Separator();
		dbox.add(fastaSeparatorA, 2, 4, 2, 1);

		TextField datasetHeaderB = new TextField();
		datasetHeaderB.setPromptText("Fasta file B");
		datasetHeaderB.textProperty().addListener(e->{ vennCalc.headerB = datasetHeaderB.getText(); });
		dbox.add(datasetHeaderB, 2, 5, 2, 1);
		Button btnFastaB = new Button("Browse");
		dbox.add(btnFastaB, 2, 6);
		TextField pathFastaB = new TextField();
		pathFastaB.setText("..");
		pathFastaB.setPrefWidth(750);
		pathFastaB.setEditable(false);
		dbox.add(pathFastaB, 3, 6);
		Separator fastaSeparatorB = new Separator();
		dbox.add(fastaSeparatorB, 2, 7, 2, 1);

		TextField datasetHeaderC = new TextField();
		datasetHeaderC.setPromptText("Fasta file C");
		datasetHeaderC.textProperty().addListener(e->{ vennCalc.headerC = datasetHeaderC.getText(); });
		dbox.add(datasetHeaderC, 2, 8, 2, 1);
		Button btnFastaC = new Button("Browse");
		dbox.add(btnFastaC, 2, 9);
		TextField pathFastaC = new TextField();
		pathFastaC.setText("..");
		pathFastaC.setPrefWidth(750);
		pathFastaC.setEditable(false);
		dbox.add(pathFastaC, 3, 9);
		Separator fastaSeparatorC = new Separator();
		dbox.add(fastaSeparatorC, 2, 10, 2, 1);

		TextField datasetHeaderD = new TextField();
		datasetHeaderD.setPromptText("Fasta file D");
		datasetHeaderD.textProperty().addListener(e->{ vennCalc.headerD = datasetHeaderD.getText(); });
		dbox.add(datasetHeaderD, 2, 11, 2, 1);
		Button btnFastaD = new Button("Browse");
		dbox.add(btnFastaD, 2, 12);
		TextField pathFastaD = new TextField();
		pathFastaD.setText("..");
		pathFastaD.setPrefWidth(750);
		pathFastaD.setEditable(false);
		dbox.add(pathFastaD, 3, 12);
		Separator fastaSeparatorD = new Separator();
		dbox.add(fastaSeparatorD, 2, 13, 2, 1);

		TextField datasetHeaderE = new TextField();
		datasetHeaderE.setPromptText("Fasta file E");
		datasetHeaderE.textProperty().addListener(e->{ vennCalc.headerE = datasetHeaderE.getText(); });
		dbox.add(datasetHeaderE, 2, 14, 2, 1);
		Button btnFastaE = new Button("Browse");
		dbox.add(btnFastaE, 2, 15);
		TextField pathFastaE = new TextField();
		pathFastaE.setText("..");
		pathFastaE.setPrefWidth(750);
		pathFastaE.setEditable(false);
		dbox.add(pathFastaE, 3, 15);
		dataInputMainBP.setCenter(dbox);
		Separator fastaSeparatorE = new Separator();
		dbox.add(fastaSeparatorE, 2, 16, 2, 1);

		Text optionsText = new Text("Options: ");
		optionsText.setFill(Color.WHITE);
		optionsText.setFont(Font.font(14));
		dbox.add(optionsText, 2, 17, 2, 1);

		cbLI = new CheckBox(" L = I ");
		cbLI.setStyle("-fx-font-size: 14;" + "-fx-text-fill: #FFFFFF;");
		dbox.add(cbLI, 2, 18, 2, 1);

		StackPane stack = new StackPane();
		pbFastaSearch = new ProgressBar(0);
		pbFastaSearch.setStyle("-fx-accent: #00FF00;");
		pbFastaSearch.setPrefHeight(30);
		pbFastaSearch.setPrefWidth(400);
		pbText = new Text(" ");
		pbText.setFont(Font.font(14));
		stack.getChildren().addAll(pbFastaSearch, pbText);
		dbox.add(stack, 1, 20);

		Button btnSearchFasta = new Button("Search");
		btnSearchFasta.setTooltip(new Tooltip("Match petides to fasta files"));
		dbox.add(btnSearchFasta, 2, 20);
		Button btnCancelSearchFasta = new Button("Cancel");
		btnCancelSearchFasta.setTooltip(new Tooltip("Cancel"));
		btnCancelSearchFasta.setDisable(true);
		dbox.add(btnCancelSearchFasta, 3, 20);

		btnSearchFasta.setOnAction(e->{
			pbFastaSearch.setProgress(0);
			btnOK.setDisable(true);
			btnLists.setDisable(true);
			btnTable.setDisable(true);
			btnValues.setDisable(true);
			btnFasta.setDisable(true);
			btnFastaA.setDisable(true);
			btnFastaB.setDisable(true);
			btnFastaC.setDisable(true);
			btnFastaD.setDisable(true);
			btnFastaE.setDisable(true);
			btnSearchFasta.setDisable(true);
			cbLI.setDisable(true);
			btnCancelSearchFasta.setDisable(false);
			ArrayList<String> dataArray = new ArrayList<String>(Arrays.asList(textAreaA.getText().split("\n")));
			for (int i=(dataArray.size()-1); i>=0; i--){
				dataArray.set(i, dataArray.get(i).trim());
				if (dataArray.get(i).equals("")) {
					dataArray.remove(i);
				}
			}
			peptideSearchThread = new Thread(){
				public void run() {
					Platform.runLater(() -> pbFastaSearch.setProgress(-1));
					Platform.runLater(() -> pbText.setText("Reading fasta files ..."));
					fastaFileA = new Fasta();
					fastaFileB = new Fasta();
					fastaFileC = new Fasta();
					fastaFileD = new Fasta();
					fastaFileE = new Fasta();
					if (!(pathFastaA.getText().equals(".."))) { fastaFileA.readFasta(pathFastaA.getText()); }
					if (!(pathFastaB.getText().equals(".."))) { fastaFileB.readFasta(pathFastaB.getText()); }
					if (!(pathFastaC.getText().equals(".."))) { fastaFileC.readFasta(pathFastaC.getText()); }
					if (!(pathFastaD.getText().equals(".."))) { fastaFileD.readFasta(pathFastaD.getText()); }
					if (!(pathFastaE.getText().equals(".."))) { fastaFileE.readFasta(pathFastaE.getText()); }
					ArrayList<ArrayList> dArray = new ArrayList<ArrayList>();
					if (cbLI.isSelected()) {
						fastaFileA.replaceLItoJ(); 
						fastaFileB.replaceLItoJ(); 
						fastaFileC.replaceLItoJ(); 
						fastaFileD.replaceLItoJ(); 
						fastaFileE.replaceLItoJ(); 
						for (int i = 0; i < dataArray.size(); i++) {
							if (peptideSearchThread.interrupted()) {
								Platform.runLater(() -> pbFastaSearch.setProgress(0));
								Platform.runLater(() -> pbText.setText("0")); 
								return; 
							}
							final double step = i;
							Platform.runLater(() -> pbFastaSearch.setProgress(step/dataArray.size()));
							Platform.runLater(() -> pbText.setText(Math.round((step/dataArray.size())*100)+"%"));
							ArrayList<String> rArray = new ArrayList<String>();
							rArray.add(dataArray.get(i).toString());
							rArray.add("");
							rArray.add("");
							rArray.add("");
							rArray.add("");
							rArray.add("");
							if ((fastaFileA.findPeptideJ(dataArray.get(i).toString())).size()>0) { rArray.set(1, "1"); }
							if ((fastaFileB.findPeptideJ(dataArray.get(i).toString())).size()>0) { rArray.set(2, "1"); }
							if ((fastaFileC.findPeptideJ(dataArray.get(i).toString())).size()>0) { rArray.set(3, "1"); }
							if ((fastaFileD.findPeptideJ(dataArray.get(i).toString())).size()>0) { rArray.set(4, "1"); }
							if ((fastaFileE.findPeptideJ(dataArray.get(i).toString())).size()>0) { rArray.set(5, "1"); }
							dArray.add(rArray);
						}
					} else {
						for (int i = 0; i < dataArray.size(); i++) {
							if (peptideSearchThread.interrupted()) {
								Platform.runLater(() -> pbFastaSearch.setProgress(0));
								Platform.runLater(() -> pbText.setText(""));
								return; 
							}
							final double step = i;
							Platform.runLater(() -> pbFastaSearch.setProgress(step/dataArray.size()));
							Platform.runLater(() -> pbText.setText(Math.round((step/dataArray.size())*100)+"%"));
							ArrayList<String> rArray = new ArrayList<String>();
							rArray.add(dataArray.get(i).toString());
							rArray.add("");
							rArray.add("");
							rArray.add("");
							rArray.add("");
							rArray.add("");
							if ((fastaFileA.findPeptide(dataArray.get(i).toString())).size()>0) { rArray.set(1, "1"); }
							if ((fastaFileB.findPeptide(dataArray.get(i).toString())).size()>0) { rArray.set(2, "1"); }
							if ((fastaFileC.findPeptide(dataArray.get(i).toString())).size()>0) { rArray.set(3, "1"); }
							if ((fastaFileD.findPeptide(dataArray.get(i).toString())).size()>0) { rArray.set(4, "1"); }
							if ((fastaFileE.findPeptide(dataArray.get(i).toString())).size()>0) { rArray.set(5, "1"); }
							dArray.add(rArray);
						}
					}
					vennCalc.setTable(dArray);
					Platform.runLater(() -> selectVennType());
					Platform.runLater(() -> pbFastaSearch.setProgress(1));
					Platform.runLater(() -> pbText.setText(100+"%"));
					Platform.runLater(() -> btnOK.setDisable(false));
					Platform.runLater(() -> btnLists.setDisable(false));
					Platform.runLater(() -> btnTable.setDisable(false));
					Platform.runLater(() -> btnValues.setDisable(false));
					Platform.runLater(() -> btnFasta.setDisable(false));
					Platform.runLater(() -> btnFastaA.setDisable(false));
					Platform.runLater(() -> btnFastaB.setDisable(false));
					Platform.runLater(() -> btnFastaC.setDisable(false));
					Platform.runLater(() -> btnFastaD.setDisable(false));
					Platform.runLater(() -> btnFastaE.setDisable(false));
					Platform.runLater(() -> btnSearchFasta.setDisable(false));
					Platform.runLater(() -> cbLI.setDisable(false));
					Platform.runLater(() -> btnCancelSearchFasta.setDisable(true));
				}
			};
			peptideSearchThread.start();
		});

		btnCancelSearchFasta.setOnAction(e->{
			//peptideSearchThread.stop();
			peptideSearchThread.interrupt();
			peptideSearchThread = null;
			pbFastaSearch.setProgress(0);
			btnOK.setDisable(false);
			btnLists.setDisable(false);
			btnTable.setDisable(false);
			btnValues.setDisable(false);
			btnFasta.setDisable(false);
			btnFastaA.setDisable(false);
			btnFastaB.setDisable(false);
			btnFastaC.setDisable(false);
			btnFastaD.setDisable(false);
			btnFastaE.setDisable(false);
			btnSearchFasta.setDisable(false);
			cbLI.setDisable(false);
			btnCancelSearchFasta.setDisable(true);
		});

		btnFastaA.setOnAction(e->{
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open Fasta file ...");
			fileChooser.setInitialDirectory(new File(currentDirectory));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Fasta file *.fasta", "*.fasta"),
				new FileChooser.ExtensionFilter("Any file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					currentDirectory = file.getParent().toString();
					pathFastaA.setText(file.toString());
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		btnFastaB.setOnAction(e->{
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open Fasta file ...");
			fileChooser.setInitialDirectory(new File(currentDirectory));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Fasta file *.fasta", "*.fasta"),
				new FileChooser.ExtensionFilter("Any file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					currentDirectory = file.getParent().toString();
					pathFastaB.setText(file.toString());
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		btnFastaC.setOnAction(e->{
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open Fasta file ...");
			fileChooser.setInitialDirectory(new File(currentDirectory));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Fasta file *.fasta", "*.fasta"),
				new FileChooser.ExtensionFilter("Any file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					currentDirectory = file.getParent().toString();
					pathFastaC.setText(file.toString());
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		btnFastaD.setOnAction(e->{
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open Fasta file ...");
			fileChooser.setInitialDirectory(new File(currentDirectory));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Fasta file *.fasta", "*.fasta"),
				new FileChooser.ExtensionFilter("Any file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					currentDirectory = file.getParent().toString();
					pathFastaD.setText(file.toString());
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		btnFastaE.setOnAction(e->{
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open Fasta file ...");
			fileChooser.setInitialDirectory(new File(currentDirectory));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Fasta file *.fasta", "*.fasta"),
				new FileChooser.ExtensionFilter("Any file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					currentDirectory = file.getParent().toString();
					pathFastaE.setText(file.toString());
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		dataInputMainBP.setCenter(dbox);
	}

	private void setTableInput() {
		dbox = new BorderPane();
		BorderPane importDataBP = new BorderPane();
		dataInputGridPane = new GridPane();
			dataInputGridPane.setStyle("-fx-background-color: DAE6F3;");
			dataInputGridPane.setPrefWidth(220);
			dataInputGridPane.setHgap(10);
			dataInputGridPane.setVgap(10);
			HBox loadDataHB = new HBox();
			loadDataHB.setSpacing(5);
			Button btnPaste = new Button("Paste");
			btnPaste.setTooltip(new Tooltip("Paste from Clipbord"));
			Button btnImport = new Button("Import");
			btnImport.setTooltip(new Tooltip("Import from File"));
			loadDataHB.getChildren().addAll(btnPaste, btnImport);
			dataInputGridPane.add(loadDataHB, 1, 1, 2, 1);
			Separator importSeparator = new Separator();
			dataInputGridPane.add(importSeparator, 1, 2, 2, 1);
			Label lblID = new Label("IDs ");
			dataInputGridPane.add(lblID, 1, 3);
			cbxID = new ComboBox<String>();
			cbxID.setPrefWidth(120);
			dataInputGridPane.add(cbxID, 2, 3);
			Label lblA = new Label("DatasetA ");
			dataInputGridPane.add(lblA, 1, 4);
			cbxA = new ComboBox<String>();
			cbxA.setPrefWidth(120);
			dataInputGridPane.add(cbxA, 2, 4);
			Label lblB = new Label("DatasetB ");
			dataInputGridPane.add(lblB, 1, 5);
			cbxB = new ComboBox<String>();
			cbxB.setPrefWidth(120);
			dataInputGridPane.add(cbxB, 2, 5);
			Label lblC = new Label("DatasetC ");
			dataInputGridPane.add(lblC, 1, 6);
			cbxC = new ComboBox<String>();
			cbxC.setPrefWidth(120);
			dataInputGridPane.add(cbxC, 2, 6);
			Label lblD = new Label("DatasetD ");
			dataInputGridPane.add(lblD, 1, 7);
			cbxD = new ComboBox<String>();
			cbxD.setPrefWidth(120);
			dataInputGridPane.add(cbxD, 2, 7);
			Label lblE = new Label("DatasetE ");
			dataInputGridPane.add(lblE, 1, 8);
			cbxE = new ComboBox<String>();
			cbxE.setPrefWidth(120);
			dataInputGridPane.add(cbxE, 2, 8);
		dbox.setLeft(dataInputGridPane);
		btnPaste.setOnAction(e->{
			dataArray = new ArrayList<ArrayList>();
			ArrayList<String> rawArray = new ArrayList<String>();
			try {
				String stringContent = clipboard.getString();
				String[] lines = stringContent.split("\r\n|\n|\r");
				for (int i=0; i<lines.length; i++) {
					rawArray = new ArrayList<String>();
					String[] cell = lines[i].split("\t");
					for (int j=0; j<cell.length; j++) {
						rawArray.add(cell[j]);
					}
					dataArray.add(rawArray);
				}
				dataArray = RWFile.checkForHeaderDuplication(dataArray);
				setDataInTable();
				setCBX();
			} catch(Exception ex) {
				ex.printStackTrace();
			}
		});
		btnImport.setOnAction(e->{
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open file ...");
			fileChooser.setInitialDirectory(new File("./"));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Tab delimited text file *.txt", "*.txt"),
				new FileChooser.ExtensionFilter("Tab-separated values file *.tsv", "*.tsv"),
				new FileChooser.ExtensionFilter("Comma-separated values file *.csv", "*.csv")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					dataArray = new ArrayList<ArrayList>();
					String[] tmp = file.toString().split("\\.");
					String ext = tmp[tmp.length-1];
					if (ext.toLowerCase().equals("txt")) { dataArray = RWFile.readTabFile(file.toString()); }
					if (ext.toLowerCase().equals("tsv")) { dataArray = RWFile.readTabFile(file.toString()); }
					if (ext.toLowerCase().equals("csv")) { dataArray = RWFile.readCSV(file.toString(), ",", "\""); };
					dataArray = RWFile.checkForHeaderDuplication(dataArray);
					setDataInTable();
					setCBX();
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		dataInputMainBP.setCenter(dbox);
	}

	private void setCBX() {
		cbxID = new ComboBox<String>();
		cbxID.setPrefWidth(120);
		cbxID.getItems().add("N/A");
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			cbxID.getItems().add(dataArray.get(0).get(k).toString());
		}
		cbxID.setValue("N/A");
		cbxID.valueProperty().addListener(e->{ setDataInCalc(); cbxID.setTooltip(new Tooltip(cbxID.getValue())); });
		dataInputGridPane.add(cbxID, 2, 3);

		cbxA = new ComboBox<String>();
		cbxA.setPrefWidth(120);
		cbxA.getItems().add("N/A");
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			cbxA.getItems().add(dataArray.get(0).get(k).toString());
		}
		cbxA.setValue("N/A");
		cbxA.valueProperty().addListener(e->{ setDataInCalc();  cbxA.setTooltip(new Tooltip(cbxA.getValue())); });
		dataInputGridPane.add(cbxA, 2, 4);

		cbxB = new ComboBox<String>();
		cbxB.setPrefWidth(120);
		cbxB.getItems().add("N/A");
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			cbxB.getItems().add(dataArray.get(0).get(k).toString());
		}
		cbxB.setValue("N/A");
		cbxB.valueProperty().addListener(e->{ setDataInCalc(); cbxB.setTooltip(new Tooltip(cbxB.getValue())); });
		dataInputGridPane.add(cbxB, 2, 5);

		cbxC = new ComboBox<String>();
		cbxC.setPrefWidth(120);
		cbxC.getItems().add("N/A");
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			cbxC.getItems().add(dataArray.get(0).get(k).toString());
		}
		cbxC.setValue("N/A");
		cbxC.valueProperty().addListener(e->{ setDataInCalc(); cbxC.setTooltip(new Tooltip(cbxC.getValue())); });
		dataInputGridPane.add(cbxC, 2, 6);


		cbxD = new ComboBox<String>();
		cbxD.setPrefWidth(120);
		cbxD.getItems().add("N/A");
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			cbxD.getItems().add(dataArray.get(0).get(k).toString());
		}
		cbxD.setValue("N/A");
		cbxD.valueProperty().addListener(e->{ setDataInCalc();  cbxD.setTooltip(new Tooltip(cbxD.getValue())); });
		dataInputGridPane.add(cbxD, 2, 7);

		cbxE = new ComboBox<String>();
		cbxE.setPrefWidth(120);
		cbxE.getItems().add("N/A");
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			cbxE.getItems().add(dataArray.get(0).get(k).toString());
		}
		cbxE.setValue("N/A");
		cbxE.valueProperty().addListener(e->{ setDataInCalc(); cbxE.setTooltip(new Tooltip(cbxE.getValue())); });
		dataInputGridPane.add(cbxE, 2, 8);
	}

	private void setDataInTable() {
		String[][] staffArray = new String[dataArray.size()-1][dataArray.get(0).size()];
		String[] staffArrayHeader = new String[dataArray.get(0).size()];
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			try {
				staffArrayHeader[k] = dataArray.get(0).get(k).toString();
			} catch (Exception e) {
				staffArrayHeader[k] = "";
			}	
		}
		for (int i = 1; i < dataArray.size(); i++) {
			for (int j = 0; j < dataArray.get(0).size(); j++) {
				try {
					staffArray[i-1][j] = dataArray.get(i).get(j).toString();
				} catch (Exception e) {
					staffArray[i-1][j] = "";
				}	
			}
		}
		ObservableList<String[]> data = FXCollections.observableArrayList();
		data.addAll(Arrays.asList(staffArray));

		table = new TableView<>();
		for (int i = 0; i < staffArray[0].length; i++) {
			TableColumn<String[], String> tc = new TableColumn<String[], String>(staffArrayHeader[i]);
			final int colNo = i;
			tc.setCellValueFactory(new Callback<CellDataFeatures<String[], String>, ObservableValue<String>>() {
				@Override
				public ObservableValue<String> call(CellDataFeatures<String[], String> p) {
					return new SimpleStringProperty((p.getValue()[colNo]));
				}
			});
			tc.setSortable(false);
			if (staffArrayHeader[i].length()<10) { 
				tc.setPrefWidth(100); 
			} else if (staffArrayHeader[i].length()>100) {
				tc.setPrefWidth(1000); 
			} else { 
				tc.setPrefWidth(staffArrayHeader[i].length()*10); 
			};
			table.getColumns().add(tc);
		}
	        table.setItems(data);
		dbox.setCenter(table);
		dataInputMainBP.setCenter(dbox);
	}

	private void setDataInCalc() {
		vennCalc.headerA = cbxA.getValue().toString();
		vennCalc.headerB = cbxB.getValue().toString();
		vennCalc.headerC = cbxC.getValue().toString();
		vennCalc.headerD = cbxD.getValue().toString();
		vennCalc.headerE = cbxE.getValue().toString();
		int idCol = -1; int aCol = -1; int bCol = -1; int cCol = -1; int dCol = -1; int eCol = -1;
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			if (dataArray.get(0).get(k).toString().equals(cbxID.getValue().toString())) { idCol = k; }
			if (dataArray.get(0).get(k).toString().equals(cbxA.getValue().toString())) { aCol = k; }
			if (dataArray.get(0).get(k).toString().equals(cbxB.getValue().toString())) { bCol = k; }
			if (dataArray.get(0).get(k).toString().equals(cbxC.getValue().toString())) { cCol = k; }
			if (dataArray.get(0).get(k).toString().equals(cbxD.getValue().toString())) { dCol = k; }
			if (dataArray.get(0).get(k).toString().equals(cbxE.getValue().toString())) { eCol = k; }
		}
		ArrayList<ArrayList> dArray = new ArrayList<ArrayList>();
		for (int i = 1; i < dataArray.size(); i++) {
			ArrayList<String> rArray = new ArrayList<String>();
			if (idCol > -1) { rArray.add(dataArray.get(i).get(idCol).toString()); } else { rArray.add(""); }
			if (aCol > -1) { rArray.add(dataArray.get(i).get(aCol).toString()); } else { rArray.add(""); }
			if (bCol > -1) { rArray.add(dataArray.get(i).get(bCol).toString()); } else { rArray.add(""); }
			if (cCol > -1) { rArray.add(dataArray.get(i).get(cCol).toString()); } else { rArray.add(""); }
			if (dCol > -1) { rArray.add(dataArray.get(i).get(dCol).toString()); } else { rArray.add(""); }
			if (eCol > -1) { rArray.add(dataArray.get(i).get(eCol).toString()); } else { rArray.add(""); }
			dArray.add(rArray);
		}
		vennCalc.setTable(dArray);
		selectVennType();
	}

	private void setListsInput() {
		GridPane dbox = new GridPane();
		dbox.setHgap(3);
		dbox.setVgap(3);
		TextField datasetHeaderA = new TextField();
		datasetHeaderA.setPromptText("DatasetA");
		datasetHeaderA.textProperty().addListener(e->{ vennCalc.headerA = datasetHeaderA.getText(); });
		dbox.add(datasetHeaderA, 1, 2, 2, 1);
		textAreaA = new TextArea() {
	            @Override
	            public void paste() {
	                Clipboard clipboard = Clipboard.getSystemClipboard();
	                if (clipboard.hasString()) {
	                   replaceSelection(clipboard.getString().replaceAll("\r\n|\r|\n", "\r\n"));
	                }
	            }
	        };
		textAreaA.setWrapText(false);
		textAreaA.setPrefHeight(1000);
		textAreaA.textProperty().addListener(e->{
			vennCalc.setA(textAreaA.getText());
			vennCalc.countVenn();
			selectVennType();
        	});
		dbox.add(textAreaA, 1, 3, 2, 1);
		TextField datasetHeaderB = new TextField();
		datasetHeaderB.setPromptText("DatasetB");
		datasetHeaderB.textProperty().addListener(e->{ vennCalc.headerB = datasetHeaderB.getText(); });
		dbox.add(datasetHeaderB, 3, 2, 2, 1);
		textAreaB = new TextArea() {
	            @Override
	            public void paste() {
	                Clipboard clipboard = Clipboard.getSystemClipboard();
	                if (clipboard.hasString()) {
	                   replaceSelection(clipboard.getString().replaceAll("\r\n|\r|\n", "\r\n"));
	                }
	            }
	        };
		textAreaB.setWrapText(false);
		textAreaB.setPrefHeight(1000);
		textAreaB.textProperty().addListener(e->{
			vennCalc.setB(textAreaB.getText());
			vennCalc.countVenn();
			selectVennType();
        	});
		dbox.add(textAreaB, 3, 3, 2, 1);
		TextField datasetHeaderC = new TextField();
		datasetHeaderC.setPromptText("DatasetC");
		datasetHeaderC.textProperty().addListener(e->{ vennCalc.headerC = datasetHeaderC.getText(); });
		dbox.add(datasetHeaderC, 5, 2, 2, 1);
		textAreaC = new TextArea() {
	            @Override
	            public void paste() {
	                Clipboard clipboard = Clipboard.getSystemClipboard();
	                if (clipboard.hasString()) {
	                   replaceSelection(clipboard.getString().replaceAll("\r\n|\r|\n", "\r\n"));
	                }
	            }
	        };
		textAreaC.setWrapText(false);
		textAreaC.setPrefHeight(1000);
		textAreaC.textProperty().addListener(e->{
			vennCalc.setC(textAreaC.getText());
			vennCalc.countVenn();
			selectVennType();
        	});
		dbox.add(textAreaC, 5, 3, 2, 1);
		TextField datasetHeaderD = new TextField();
		datasetHeaderD.setPromptText("DatasetD");
		datasetHeaderD.textProperty().addListener(e->{ vennCalc.headerD = datasetHeaderD.getText(); });
		dbox.add(datasetHeaderD, 7, 2, 2, 1);
		textAreaD = new TextArea() {
	            @Override
	            public void paste() {
	                Clipboard clipboard = Clipboard.getSystemClipboard();
	                if (clipboard.hasString()) {
	                   replaceSelection(clipboard.getString().replaceAll("\r\n|\r|\n", "\r\n"));
	                }
	            }
	        };
		textAreaD.setWrapText(false);
		textAreaD.setPrefHeight(1000);
		textAreaD.textProperty().addListener(e->{
			vennCalc.setD(textAreaD.getText());
			vennCalc.countVenn();
			selectVennType();
        	});
		dbox.add(textAreaD, 7, 3, 2, 1);
		TextField datasetHeaderE = new TextField();
		datasetHeaderE.setPromptText("DatasetE");
		datasetHeaderE.textProperty().addListener(e->{ vennCalc.headerE = datasetHeaderE.getText(); });
		dbox.add(datasetHeaderE, 9, 2, 2, 1);
		textAreaE = new TextArea() {
	            @Override
	            public void paste() {
	                Clipboard clipboard = Clipboard.getSystemClipboard();
	                if (clipboard.hasString()) {
	                   replaceSelection(clipboard.getString().replaceAll("\r\n|\r|\n", "\r\n"));
	                }
	            }
	        };
		textAreaE.setWrapText(false);
		textAreaE.setPrefHeight(1000);
		textAreaE.textProperty().addListener(e->{
			vennCalc.setE(textAreaE.getText());
			vennCalc.countVenn();
			selectVennType();
        	});
		dbox.add(textAreaE, 9, 3, 2, 1);

		Button btnImportListA = new Button("Import");
		btnImportListA.setTooltip(new Tooltip("Import from File"));
		dbox.add(btnImportListA, 1, 1);
		cbImportListA = new CheckBox(" Header ");
		cbImportListA.setTooltip(new Tooltip("First line in file is header"));
		cbImportListA.setStyle("-fx-font-size: 14;" + "-fx-text-fill: #FFFFFF;");
		dbox.add(cbImportListA, 2, 1);

		btnImportListA.setOnAction(e->{
			ArrayList<String> data = new ArrayList<String>();
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open file ...");
			fileChooser.setInitialDirectory(new File("./"));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Text file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					data = RWFile.readList(file.toString());
					String dataStr = "";
					if(cbImportListA.isSelected()) {
						datasetHeaderA.setText(data.get(0));
						for (int k = 1; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					} else {
						for (int k = 0; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					}
					textAreaA.setText(dataStr);
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		Button btnImportListB = new Button("Import");
		btnImportListB.setTooltip(new Tooltip("Import from File"));
		dbox.add(btnImportListB, 3, 1);
		cbImportListB = new CheckBox(" Header ");
		cbImportListB.setStyle("-fx-font-size: 14;" + "-fx-text-fill: #FFFFFF;");
		dbox.add(cbImportListB, 4, 1);

		btnImportListB.setOnAction(e->{
			ArrayList<String> data = new ArrayList<String>();
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open file ...");
			fileChooser.setInitialDirectory(new File("./"));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Text file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					data = RWFile.readList(file.toString());
					String dataStr = "";
					if(cbImportListB.isSelected()) {
						datasetHeaderB.setText(data.get(0));
						for (int k = 1; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					} else {
						for (int k = 0; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					}
					textAreaB.setText(dataStr);
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		Button btnImportListC = new Button("Import");
		btnImportListC.setTooltip(new Tooltip("Import from File"));
		dbox.add(btnImportListC, 5, 1);
		cbImportListC = new CheckBox(" Header ");
		cbImportListC.setStyle("-fx-font-size: 14;" + "-fx-text-fill: #FFFFFF;");
		dbox.add(cbImportListC, 6, 1);

		btnImportListC.setOnAction(e->{
			ArrayList<String> data = new ArrayList<String>();
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open file ...");
			fileChooser.setInitialDirectory(new File("./"));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Text file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					data = RWFile.readList(file.toString());
					String dataStr = "";
					if(cbImportListC.isSelected()) {
						datasetHeaderC.setText(data.get(0));
						for (int k = 1; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					} else {
						for (int k = 0; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					}
					textAreaC.setText(dataStr);
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		Button btnImportListD = new Button("Import");
		btnImportListD.setTooltip(new Tooltip("Import from File"));
		dbox.add(btnImportListD, 7, 1);
		cbImportListD = new CheckBox(" Header ");
		cbImportListD.setStyle("-fx-font-size: 14;" + "-fx-text-fill: #FFFFFF;");
		dbox.add(cbImportListD, 8, 1);
		btnImportListD.setOnAction(e->{
			ArrayList<String> data = new ArrayList<String>();
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open file ...");
			fileChooser.setInitialDirectory(new File("./"));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Text file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					data = RWFile.readList(file.toString());
					String dataStr = "";
					if(cbImportListD.isSelected()) {
						datasetHeaderD.setText(data.get(0));
						for (int k = 1; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					} else {
						for (int k = 0; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					}
					textAreaD.setText(dataStr);
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		Button btnImportListE = new Button("Import");
		btnImportListE.setTooltip(new Tooltip("Import from File"));
		dbox.add(btnImportListE, 9, 1);
		cbImportListE = new CheckBox(" Header ");
		cbImportListE.setStyle("-fx-font-size: 14;" + "-fx-text-fill: #FFFFFF;");
		dbox.add(cbImportListE, 10, 1);
		btnImportListE.setOnAction(e->{
			ArrayList<String> data = new ArrayList<String>();
			FileChooser fileChooser = new FileChooser();
			fileChooser.setTitle("Open file ...");
			fileChooser.setInitialDirectory(new File("./"));
			fileChooser.getExtensionFilters().addAll(
				new FileChooser.ExtensionFilter("Text file *.*", "*.*")
			);
			File file = fileChooser.showOpenDialog(null);
			if (file!=null) {
				try {
					data = RWFile.readList(file.toString());
					String dataStr = "";
					if(cbImportListE.isSelected()) {
						datasetHeaderE.setText(data.get(0));
						for (int k = 1; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					} else {
						for (int k = 0; k < data.size(); k++) {
							dataStr += data.get(k)+"\r\n";
						}
					}
					textAreaE.setText(dataStr);
				} catch (Exception ex) {
					System.err.println("Error: " + ex.getMessage());
				}
			}
		});
		dataInputMainBP.setCenter(dbox);
		textAreaA.requestFocus();
	}

	private void setDataInList() {
		System.out.println("setDataInList....");
		System.out.println(dataArray.size());
		String a = dataArray.get(0).get(0).toString();
		textAreaA.setText(a);
		for (int k = 0; k < dataArray.get(0).size(); k++) {
			System.out.println(dataArray.get(0).get(k));
		}
	}


	private void setValuesInput() {
		GridPane dbox = new GridPane();
		dbox.setHgap(10);
		dbox.setVgap(5);
		TextField datasetHeaderA = new TextField();
		datasetHeaderA.setPromptText("DatasetA");
		datasetHeaderA.setPrefWidth(400);
		datasetHeaderA.textProperty().addListener(e->{ vennCalc.headerA = datasetHeaderA.getText(); });
		dbox.add(datasetHeaderA, 1, 0, 2, 1);
		Label labelA = new Label("A");
		labelA.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelA, 1, 1);
		TextField valueA = new TextField("");
		valueA.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[1] = Integer.parseInt(valueA.getText()); } catch(Exception ex) { vennCalc.vennOverlap[1] = 0; }
			selectVennType();
		});
		dbox.add(valueA, 2, 1);
		TextField datasetHeaderB = new TextField();
		datasetHeaderB.setPromptText("DatasetB");
		datasetHeaderB.setPrefWidth(400);
		datasetHeaderB.textProperty().addListener(e->{ vennCalc.headerB = datasetHeaderB.getText(); });
		dbox.add(datasetHeaderB, 3, 0, 2, 1);
		Label labelB = new Label("B");
		labelB.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelB, 3, 1);
		TextField valueB = new TextField("");
		valueB.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[2] = Integer.parseInt(valueB.getText()); } catch(Exception ex) { vennCalc.vennOverlap[2] = 0; }
			selectVennType();
		});
		dbox.add(valueB, 4, 1);
		Label labelAB = new Label("AB");
		labelAB.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelAB, 3, 2);
		TextField valueAB = new TextField("");
		valueAB.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[3] = Integer.parseInt(valueAB.getText()); } catch(Exception ex) { vennCalc.vennOverlap[3] = 0; }
			selectVennType();
		});
		dbox.add(valueAB, 4, 2);
		TextField datasetHeaderC = new TextField();
		datasetHeaderC.setPromptText("DatasetC");
		datasetHeaderC.setPrefWidth(400);
		datasetHeaderC.textProperty().addListener(e->{ vennCalc.headerC = datasetHeaderC.getText(); });
		dbox.add(datasetHeaderC, 5, 0, 2, 1);
		Label labelC = new Label("C");
		labelC.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelC, 5, 1);
		TextField valueC = new TextField("");
		valueC.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[4] = Integer.parseInt(valueC.getText()); } catch(Exception ex) { vennCalc.vennOverlap[4] = 0; }
			selectVennType();
		});
		dbox.add(valueC, 6, 1);
		Label labelAC = new Label("AC");
		labelAC.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelAC, 5, 2);
		TextField valueAC = new TextField("");
		valueAC.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[5] = Integer.parseInt(valueAC.getText()); } catch(Exception ex) { vennCalc.vennOverlap[5] = 0; }
			selectVennType();
		});
		dbox.add(valueAC, 6, 2);
		Label labelBC = new Label("BC");
		labelBC.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelBC, 5, 3);
		TextField valueBC = new TextField("");
		valueBC.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[6] = Integer.parseInt(valueBC.getText()); } catch(Exception ex) { vennCalc.vennOverlap[6] = 0; }
			selectVennType();
		});
		dbox.add(valueBC, 6, 3);
		Label labelABC = new Label("ABC");
		labelABC.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelABC, 5, 4);
		TextField valueABC = new TextField("");
		valueABC.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[7] = Integer.parseInt(valueABC.getText()); } catch(Exception ex) { vennCalc.vennOverlap[7] = 0; }
			selectVennType();
		});
		dbox.add(valueABC, 6, 4);
		TextField datasetHeaderD = new TextField();
		datasetHeaderD.setPromptText("DatasetD");
		datasetHeaderD.setPrefWidth(400);
		datasetHeaderD.textProperty().addListener(e->{ vennCalc.headerD = datasetHeaderD.getText(); });
		dbox.add(datasetHeaderD, 7, 0, 2, 1);
		Label labelD = new Label("D");
		labelD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelD, 7, 1);
		TextField valueD = new TextField("");
		valueD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[8] = Integer.parseInt(valueD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[8] = 0; }
			selectVennType();
		});
		dbox.add(valueD, 8, 1);
		Label labelAD = new Label("AD");
		labelAD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelAD, 7, 2);
		TextField valueAD = new TextField("");
		valueAD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[9] = Integer.parseInt(valueAD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[9] = 0; }
			selectVennType();
		});
		dbox.add(valueAD, 8, 2);

		Label labelBD = new Label("BD");
		labelBD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelBD, 7, 3);
		TextField valueBD = new TextField("");
		valueBD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[10] = Integer.parseInt(valueBD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[10] = 0; }
			selectVennType();
		});
		dbox.add(valueBD, 8, 3);
		Label labelABD = new Label("ABD");
		labelABD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelABD, 7, 4);
		TextField valueABD = new TextField("");
		valueABD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[11] = Integer.parseInt(valueABD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[11] = 0; }
			selectVennType();
		});
		dbox.add(valueABD, 8, 4);
		Label labelCD = new Label("CD");
		labelCD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelCD, 7, 5);
		TextField valueCD = new TextField("");
		valueCD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[12] = Integer.parseInt(valueCD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[12] = 0; }
			selectVennType();
		});
		dbox.add(valueCD, 8, 5);
		Label labelACD = new Label("ACD");
		labelACD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelACD, 7, 6);
		TextField valueACD = new TextField("");
		valueACD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[13] = Integer.parseInt(valueACD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[13] = 0; }
			selectVennType();
		});
		dbox.add(valueACD, 8, 6);
		Label labelBCD = new Label("BCD");
		labelBCD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelBCD, 7, 7);
		TextField valueBCD = new TextField("");
		valueBCD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[14] = Integer.parseInt(valueBCD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[14] = 0; }
			selectVennType();
		});
		dbox.add(valueBCD, 8, 7);
		Label labelABCD = new Label("ABCD");
		labelABCD.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelABCD, 7, 8);
		TextField valueABCD = new TextField("");
		valueABCD.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[15] = Integer.parseInt(valueABCD.getText()); } catch(Exception ex) { vennCalc.vennOverlap[15] = 0; }
			selectVennType();
		});
		dbox.add(valueABCD, 8, 8);
		TextField datasetHeaderE = new TextField();
		datasetHeaderE.setPromptText("DatasetE");
		datasetHeaderE.setPrefWidth(400);
		datasetHeaderE.textProperty().addListener(e->{ vennCalc.headerE = datasetHeaderE.getText(); });
		dbox.add(datasetHeaderE, 9, 0, 2, 1);
		Label labelE = new Label("E");
		labelE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelE, 9, 1);
		TextField valueE = new TextField("");
		valueE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[16] = Integer.parseInt(valueE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[16] = 0; }
			selectVennType();
		});
		dbox.add(valueE, 10, 1);
		Label labelAE = new Label("AE");
		labelAE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelAE, 9, 2);
		TextField valueAE = new TextField("");
		valueAE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[17] = Integer.parseInt(valueAE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[17] = 0; }
			selectVennType();
		});
		dbox.add(valueAE, 10, 2);
		Label labelBE = new Label("BE");
		labelBE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelBE, 9, 3);
		TextField valueBE = new TextField("");
		valueBE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[18] = Integer.parseInt(valueBE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[18] = 0; }
			selectVennType();
		});
		dbox.add(valueBE, 10, 3);
		Label labelABE = new Label("ABE");
		labelABE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelABE, 9, 4);
		TextField valueABE = new TextField("");
		valueABE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[19] = Integer.parseInt(valueABE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[19] = 0; }
			selectVennType();
		});
		dbox.add(valueABE, 10, 4);
		Label labelCE = new Label("CE");
		labelCE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelCE, 9, 5);
		TextField valueCE = new TextField("");
		valueCE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[20] = Integer.parseInt(valueCE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[20] = 0; }
			selectVennType();
		});
		dbox.add(valueCE, 10, 5);
		Label labelACE = new Label("ACE");
		labelACE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelACE, 9, 6);
		TextField valueACE = new TextField("");
		valueACE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[21] = Integer.parseInt(valueACE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[21] = 0; }
			selectVennType();
		});
		dbox.add(valueACE, 10, 6);
		Label labelBCE = new Label("BCE");
		labelBCE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelBCE, 9, 7);
		TextField valueBCE = new TextField("");
		valueBCE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[22] = Integer.parseInt(valueBCE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[22] = 0; }
			selectVennType();
		});
		dbox.add(valueBCE, 10, 7);
		Label labelABCE = new Label("ABCE");
		labelABCE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelABCE, 9, 8);
		TextField valueABCE = new TextField("");
		valueABCE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[23] = Integer.parseInt(valueABCE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[23] = 0; }
			selectVennType();
		});
		dbox.add(valueABCE, 10, 8);
		Label labelDE = new Label("DE");
		labelDE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelDE, 9, 9);
		TextField valueDE = new TextField("");
		valueDE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[24] = Integer.parseInt(valueDE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[24] = 0; }
			selectVennType();
		});
		dbox.add(valueDE, 10, 9);
		Label labelADE = new Label("ADE");
		labelADE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelADE, 9, 10);
		TextField valueADE = new TextField("");
		valueADE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[25] = Integer.parseInt(valueADE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[25] = 0; }
			selectVennType();
		});
		dbox.add(valueADE, 10, 10);

		Label labelBDE = new Label("BDE");
		labelBDE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelBDE, 9, 11);
		TextField valueBDE = new TextField("");
		valueBDE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[26] = Integer.parseInt(valueBDE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[26] = 0; }
			selectVennType();
		});
		dbox.add(valueBDE, 10, 11);
		Label labelABDE = new Label("ABDE");
		labelABDE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelABDE, 9, 12);
		TextField valueABDE = new TextField("");
		valueABDE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[27] = Integer.parseInt(valueABDE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[27] = 0; }
			selectVennType();
		});
		dbox.add(valueABDE, 10, 12);
		Label labelCDE = new Label("CDE");
		labelCDE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelCDE, 9, 13);
		TextField valueCDE = new TextField("");
		valueCDE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[28] = Integer.parseInt(valueCDE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[28] = 0; }
			selectVennType();
		});
		dbox.add(valueCDE, 10, 13);
		Label labelACDE = new Label("ACDE");
		labelACDE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelACDE, 9, 14);
		TextField valueACDE = new TextField("");
		valueACDE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[29] = Integer.parseInt(valueACDE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[29] = 0; }
			selectVennType();
		});
		dbox.add(valueACDE, 10, 14);
		Label labelBCDE = new Label("BCDE");
		labelBCDE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelBCDE, 9, 15);
		TextField valueBCDE = new TextField("");
		valueBCDE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[30] = Integer.parseInt(valueBCDE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[30] = 0; }
			selectVennType();
		});
		dbox.add(valueBCDE, 10, 15);
		Label labelABCDE = new Label("ABCDE");
		labelABCDE.setTextFill(Color.web("#FFFFFF"));
		dbox.add(labelABCDE, 9, 16);
		TextField valueABCDE = new TextField("");
		valueABCDE.textProperty().addListener(e->{
			try { vennCalc.vennOverlap[31] = Integer.parseInt(valueABCDE.getText()); } catch(Exception ex) { vennCalc.vennOverlap[31] = 0; }
			selectVennType();
		});
		dbox.add(valueABCDE, 10, 16);
		dataInputMainBP.setCenter(dbox);
	}

	protected void selectVennType() {
		vennGroup = new ToggleGroup();
		HBox vPane = new HBox();
		vPane.setPadding(new Insets(0, 0, 0, 2));
		vPane.setSpacing(2);
		if (vennCalc.getSetsN()==1) {
			ToggleButton btnEuler001 = new ToggleButton();
			btnEuler001.setGraphic(new ImageView(imEuler001));
			btnEuler001.setToggleGroup(vennGroup);
			btnEuler001.setUserData("e001");
			vPane.getChildren().add(btnEuler001);
		}
		if (vennCalc.getSetsN()==2) {
			ToggleButton btnVenn2 = new ToggleButton();
			btnVenn2.setGraphic(new ImageView(imVenn2));
			btnVenn2.setToggleGroup(vennGroup);
			btnVenn2.setUserData("v2");
			vPane.getChildren().add(btnVenn2);
			switch (vennCalc.getEulerType()) {
				case 3:
					ToggleButton btnEuler003 = new ToggleButton();
					btnEuler003.setGraphic(new ImageView(imEuler003));
					btnEuler003.setToggleGroup(vennGroup);
					btnEuler003.setUserData("e003");
					vPane.getChildren().add(btnEuler003);
					break;
				case 5:
					ToggleButton btnEuler005 = new ToggleButton();
					btnEuler005.setGraphic(new ImageView(imEuler005));
					btnEuler005.setToggleGroup(vennGroup);
					btnEuler005.setUserData("e005");
					vPane.getChildren().add(btnEuler005);
					break;
				case 6:
					ToggleButton btnEuler006 = new ToggleButton();
					btnEuler006.setGraphic(new ImageView(imEuler006));
					btnEuler006.setToggleGroup(vennGroup);
					btnEuler006.setUserData("e006");
					vPane.getChildren().add(btnEuler006);
					break;
			}
		}
		if (vennCalc.getSetsN()==3) {
			ToggleButton btnVenn3 = new ToggleButton();
			btnVenn3.setGraphic(new ImageView(imVenn3));
			btnVenn3.setToggleGroup(vennGroup);
			btnVenn3.setUserData("v3");
			vPane.getChildren().add(btnVenn3);
			switch (vennCalc.getEulerType()) {
				case 11:
					ToggleButton btnEuler011 = new ToggleButton();
					btnEuler011.setGraphic(new ImageView(imEuler011));
					btnEuler011.setToggleGroup(vennGroup);
					btnEuler011.setUserData("e011");
					vPane.getChildren().add(btnEuler011);
					break;
				case 15:
					ToggleButton btnEuler015 = new ToggleButton();
					btnEuler015.setGraphic(new ImageView(imEuler015));
					btnEuler015.setToggleGroup(vennGroup);
					btnEuler015.setUserData("e015");
					vPane.getChildren().add(btnEuler015);
					break;
				case 21:
					ToggleButton btnEuler021 = new ToggleButton();
					btnEuler021.setGraphic(new ImageView(imEuler021));
					btnEuler021.setToggleGroup(vennGroup);
					btnEuler021.setUserData("e021");
					vPane.getChildren().add(btnEuler021);
					break;
				case 23:
					ToggleButton btnEuler023 = new ToggleButton();
					btnEuler023.setGraphic(new ImageView(imEuler023));
					btnEuler023.setToggleGroup(vennGroup);
					btnEuler023.setUserData("e023");
					vPane.getChildren().add(btnEuler023);
					break;
				case 27:
					ToggleButton btnEuler027 = new ToggleButton();
					btnEuler027.setGraphic(new ImageView(imEuler027));
					btnEuler027.setToggleGroup(vennGroup);
					btnEuler027.setUserData("e027");
					vPane.getChildren().add(btnEuler027);
					break;
				case 29:
					ToggleButton btnEuler029 = new ToggleButton();
					btnEuler029.setGraphic(new ImageView(imEuler029));
					btnEuler029.setToggleGroup(vennGroup);
					btnEuler029.setUserData("e029");
					vPane.getChildren().add(btnEuler029);
					break;
				case 31:
					ToggleButton btnEuler031 = new ToggleButton();
					btnEuler031.setGraphic(new ImageView(imEuler031));
					btnEuler031.setToggleGroup(vennGroup);
					btnEuler031.setUserData("e031");
					vPane.getChildren().add(btnEuler031);
					break;
				case 38:
					ToggleButton btnEuler038 = new ToggleButton();
					btnEuler038.setGraphic(new ImageView(imEuler038));
					btnEuler038.setToggleGroup(vennGroup);
					btnEuler038.setUserData("e038");
					vPane.getChildren().add(btnEuler038);
					break;
				case 39:
					ToggleButton btnEuler039 = new ToggleButton();
					btnEuler039.setGraphic(new ImageView(imEuler039));
					btnEuler039.setToggleGroup(vennGroup);
					btnEuler039.setUserData("e039");
					vPane.getChildren().add(btnEuler039);
					break;
				case 43:
					ToggleButton btnEuler043 = new ToggleButton();
					btnEuler043.setGraphic(new ImageView(imEuler043));
					btnEuler043.setToggleGroup(vennGroup);
					btnEuler043.setUserData("e043");
					vPane.getChildren().add(btnEuler043);
					break;
				case 46:
					ToggleButton btnEuler046 = new ToggleButton();
					btnEuler046.setGraphic(new ImageView(imEuler046));
					btnEuler046.setToggleGroup(vennGroup);
					btnEuler046.setUserData("e046");
					vPane.getChildren().add(btnEuler046);
					break;
				case 47:
					ToggleButton btnEuler047 = new ToggleButton();
					btnEuler047.setGraphic(new ImageView(imEuler047));
					btnEuler047.setToggleGroup(vennGroup);
					btnEuler047.setUserData("e047");
					vPane.getChildren().add(btnEuler047);
					break;
				case 56:
					ToggleButton btnEuler056 = new ToggleButton();
					btnEuler056.setGraphic(new ImageView(imEuler056));
					btnEuler056.setToggleGroup(vennGroup);
					btnEuler056.setUserData("e056");
					vPane.getChildren().add(btnEuler056);
					break;
				case 57:
					ToggleButton btnEuler057 = new ToggleButton();
					btnEuler057.setGraphic(new ImageView(imEuler057));
					btnEuler057.setToggleGroup(vennGroup);
					btnEuler057.setUserData("e057");
					vPane.getChildren().add(btnEuler057);
					break;
				case 58:
					ToggleButton btnEuler058 = new ToggleButton();
					btnEuler058.setGraphic(new ImageView(imEuler058));
					btnEuler058.setToggleGroup(vennGroup);
					btnEuler058.setUserData("e058");
					vPane.getChildren().add(btnEuler058);
					break;
				case 59:
					ToggleButton btnEuler059 = new ToggleButton();
					btnEuler059.setGraphic(new ImageView(imEuler059));
					btnEuler059.setToggleGroup(vennGroup);
					btnEuler059.setUserData("e059");
					vPane.getChildren().add(btnEuler059);
					break;
				case 63:
					ToggleButton btnEuler063 = new ToggleButton();
					btnEuler063.setGraphic(new ImageView(imEuler063));
					btnEuler063.setToggleGroup(vennGroup);
					btnEuler063.setUserData("e063");
					vPane.getChildren().add(btnEuler063);
					break;
				case 69:
					ToggleButton btnEuler069 = new ToggleButton();
					btnEuler069.setGraphic(new ImageView(imEuler069));
					btnEuler069.setToggleGroup(vennGroup);
					btnEuler069.setUserData("e069");
					vPane.getChildren().add(btnEuler069);
					break;
				case 70:
					ToggleButton btnEuler070 = new ToggleButton();
					btnEuler070.setGraphic(new ImageView(imEuler070));
					btnEuler070.setToggleGroup(vennGroup);
					btnEuler070.setUserData("e070");
					vPane.getChildren().add(btnEuler070);
					break;
				case 71:
					ToggleButton btnEuler071 = new ToggleButton();
					btnEuler071.setGraphic(new ImageView(imEuler071));
					btnEuler071.setToggleGroup(vennGroup);
					btnEuler071.setUserData("e071");
					vPane.getChildren().add(btnEuler071);
					break;
				case 75:
					ToggleButton btnEuler075 = new ToggleButton();
					btnEuler075.setGraphic(new ImageView(imEuler075));
					btnEuler075.setToggleGroup(vennGroup);
					btnEuler075.setUserData("e075");
					vPane.getChildren().add(btnEuler075);
					break;
				case 77:
					ToggleButton btnEuler077 = new ToggleButton();
					btnEuler077.setGraphic(new ImageView(imEuler077));
					btnEuler077.setToggleGroup(vennGroup);
					btnEuler077.setUserData("e077");
					vPane.getChildren().add(btnEuler077);
					break;
				case 78:
					ToggleButton btnEuler078 = new ToggleButton();
					btnEuler078.setGraphic(new ImageView(imEuler078));
					btnEuler078.setToggleGroup(vennGroup);
					btnEuler078.setUserData("e078");
					vPane.getChildren().add(btnEuler078);
					break;
				case 79:
					ToggleButton btnEuler079 = new ToggleButton();
					btnEuler079.setGraphic(new ImageView(imEuler079));
					btnEuler079.setToggleGroup(vennGroup);
					btnEuler079.setUserData("e079");
					vPane.getChildren().add(btnEuler079);
					break;
				case 81:
					ToggleButton btnEuler081 = new ToggleButton();
					btnEuler081.setGraphic(new ImageView(imEuler081));
					btnEuler081.setToggleGroup(vennGroup);
					btnEuler081.setUserData("e081");
					vPane.getChildren().add(btnEuler081);
					break;
				case 83:
					ToggleButton btnEuler083 = new ToggleButton();
					btnEuler083.setGraphic(new ImageView(imEuler083));
					btnEuler083.setToggleGroup(vennGroup);
					btnEuler083.setUserData("e083");
					vPane.getChildren().add(btnEuler083);
					break;
				case 85:
					ToggleButton btnEuler085 = new ToggleButton();
					btnEuler085.setGraphic(new ImageView(imEuler085));
					btnEuler085.setToggleGroup(vennGroup);
					btnEuler085.setUserData("e085");
					vPane.getChildren().add(btnEuler085);
					break;
				case 87:
					ToggleButton btnEuler087 = new ToggleButton();
					btnEuler087.setGraphic(new ImageView(imEuler087));
					btnEuler087.setToggleGroup(vennGroup);
					btnEuler087.setUserData("e087");
					vPane.getChildren().add(btnEuler087);
					break;
				case 88:
					ToggleButton btnEuler088 = new ToggleButton();
					btnEuler088.setGraphic(new ImageView(imEuler088));
					btnEuler088.setToggleGroup(vennGroup);
					btnEuler088.setUserData("e088");
					vPane.getChildren().add(btnEuler088);
					break;
				case 89:
					ToggleButton btnEuler089 = new ToggleButton();
					btnEuler089.setGraphic(new ImageView(imEuler089));
					btnEuler089.setToggleGroup(vennGroup);
					btnEuler089.setUserData("e089");
					vPane.getChildren().add(btnEuler089);
					break;
				case 90:
					ToggleButton btnEuler090 = new ToggleButton();
					btnEuler090.setGraphic(new ImageView(imEuler090));
					btnEuler090.setToggleGroup(vennGroup);
					btnEuler090.setUserData("e090");
					vPane.getChildren().add(btnEuler090);
					break;
				case 91:
					ToggleButton btnEuler091 = new ToggleButton();
					btnEuler091.setGraphic(new ImageView(imEuler091));
					btnEuler091.setToggleGroup(vennGroup);
					btnEuler091.setUserData("e091");
					vPane.getChildren().add(btnEuler091);
					break;
				case 93:
					ToggleButton btnEuler093 = new ToggleButton();
					btnEuler093.setGraphic(new ImageView(imEuler093));
					btnEuler093.setToggleGroup(vennGroup);
					btnEuler093.setUserData("e093");
					vPane.getChildren().add(btnEuler093);
					break;
				case 94:
					ToggleButton btnEuler094 = new ToggleButton();
					btnEuler094.setGraphic(new ImageView(imEuler094));
					btnEuler094.setToggleGroup(vennGroup);
					btnEuler094.setUserData("e094");
					vPane.getChildren().add(btnEuler094);
					break;
				case 95:
					ToggleButton btnEuler095 = new ToggleButton();
					btnEuler095.setGraphic(new ImageView(imEuler095));
					btnEuler095.setToggleGroup(vennGroup);
					btnEuler095.setUserData("e095");
					vPane.getChildren().add(btnEuler095);
					break;
				case 98:
					ToggleButton btnEuler098 = new ToggleButton();
					btnEuler098.setGraphic(new ImageView(imEuler098));
					btnEuler098.setToggleGroup(vennGroup);
					btnEuler098.setUserData("e098");
					vPane.getChildren().add(btnEuler098);
					break;
				case 99:
					ToggleButton btnEuler099 = new ToggleButton();
					btnEuler099.setGraphic(new ImageView(imEuler099));
					btnEuler099.setToggleGroup(vennGroup);
					btnEuler099.setUserData("e099");
					vPane.getChildren().add(btnEuler099);
					break;
				case 102:
					ToggleButton btnEuler102 = new ToggleButton();
					btnEuler102.setGraphic(new ImageView(imEuler102));
					btnEuler102.setToggleGroup(vennGroup);
					btnEuler102.setUserData("e102");
					vPane.getChildren().add(btnEuler102);
					break;
				case 103:
					ToggleButton btnEuler103 = new ToggleButton();
					btnEuler103.setGraphic(new ImageView(imEuler103));
					btnEuler103.setToggleGroup(vennGroup);
					btnEuler103.setUserData("e103");
					vPane.getChildren().add(btnEuler103);
					break;
				case 104:
					ToggleButton btnEuler104 = new ToggleButton();
					btnEuler104.setGraphic(new ImageView(imEuler104));
					btnEuler104.setToggleGroup(vennGroup);
					btnEuler104.setUserData("e104");
					vPane.getChildren().add(btnEuler104);
					break;
				case 105:
					ToggleButton btnEuler105 = new ToggleButton();
					btnEuler105.setGraphic(new ImageView(imEuler105));
					btnEuler105.setToggleGroup(vennGroup);
					btnEuler105.setUserData("e105");
					vPane.getChildren().add(btnEuler105);
					break;
				case 106:
					ToggleButton btnEuler106 = new ToggleButton();
					btnEuler106.setGraphic(new ImageView(imEuler106));
					btnEuler106.setToggleGroup(vennGroup);
					btnEuler106.setUserData("e106");
					vPane.getChildren().add(btnEuler106);
					break;
				case 107:
					ToggleButton btnEuler107 = new ToggleButton();
					btnEuler107.setGraphic(new ImageView(imEuler107));
					btnEuler107.setToggleGroup(vennGroup);
					btnEuler107.setUserData("e107");
					vPane.getChildren().add(btnEuler107);
					break;
				case 109:
					ToggleButton btnEuler109 = new ToggleButton();
					btnEuler109.setGraphic(new ImageView(imEuler109));
					btnEuler109.setToggleGroup(vennGroup);
					btnEuler109.setUserData("e109");
					vPane.getChildren().add(btnEuler109);
					break;
				case 110:
					ToggleButton btnEuler110 = new ToggleButton();
					btnEuler110.setGraphic(new ImageView(imEuler110));
					btnEuler110.setToggleGroup(vennGroup);
					btnEuler110.setUserData("e110");
					vPane.getChildren().add(btnEuler110);
					break;
				case 111:
					ToggleButton btnEuler111 = new ToggleButton();
					btnEuler111.setGraphic(new ImageView(imEuler111));
					btnEuler111.setToggleGroup(vennGroup);
					btnEuler111.setUserData("e111");
					vPane.getChildren().add(btnEuler111);
					break;
				case 115:
					ToggleButton btnEuler115 = new ToggleButton();
					btnEuler115.setGraphic(new ImageView(imEuler115));
					btnEuler115.setToggleGroup(vennGroup);
					btnEuler115.setUserData("e115");
					vPane.getChildren().add(btnEuler115);
					break;
				case 119:
					ToggleButton btnEuler119 = new ToggleButton();
					btnEuler119.setGraphic(new ImageView(imEuler119));
					btnEuler119.setToggleGroup(vennGroup);
					btnEuler119.setUserData("e119");
					vPane.getChildren().add(btnEuler119);
					break;
				case 120:
					ToggleButton btnEuler120 = new ToggleButton();
					btnEuler120.setGraphic(new ImageView(imEuler120));
					btnEuler120.setToggleGroup(vennGroup);
					btnEuler120.setUserData("e120");
					vPane.getChildren().add(btnEuler120);
					break;
				case 121:
					ToggleButton btnEuler121 = new ToggleButton();
					btnEuler121.setGraphic(new ImageView(imEuler121));
					btnEuler121.setToggleGroup(vennGroup);
					btnEuler121.setUserData("e121");
					vPane.getChildren().add(btnEuler121);
					break;
				case 122:
					ToggleButton btnEuler122 = new ToggleButton();
					btnEuler122.setGraphic(new ImageView(imEuler122));
					btnEuler122.setToggleGroup(vennGroup);
					btnEuler122.setUserData("e122");
					vPane.getChildren().add(btnEuler122);
					break;
				case 123:
					ToggleButton btnEuler123 = new ToggleButton();
					btnEuler123.setGraphic(new ImageView(imEuler123));
					btnEuler123.setToggleGroup(vennGroup);
					btnEuler123.setUserData("e123");
					vPane.getChildren().add(btnEuler123);
					break;
				case 125:
					ToggleButton btnEuler125 = new ToggleButton();
					btnEuler125.setGraphic(new ImageView(imEuler125));
					btnEuler125.setToggleGroup(vennGroup);
					btnEuler125.setUserData("e125");
					vPane.getChildren().add(btnEuler125);
					break;
				case 126:
					ToggleButton btnEuler126 = new ToggleButton();
					btnEuler126.setGraphic(new ImageView(imEuler126));
					btnEuler126.setToggleGroup(vennGroup);
					btnEuler126.setUserData("e126");
					vPane.getChildren().add(btnEuler126);
					break;
			}
		}
		if (vennCalc.getSetsN()==4) {
			ToggleButton btnVenn4 = new ToggleButton();
			btnVenn4.setGraphic(new ImageView(imVenn4));
			btnVenn4.setToggleGroup(vennGroup);
			btnVenn4.setUserData("v4");
			vPane.getChildren().add(btnVenn4);
		}
		if (vennCalc.getSetsN()==5) {
			ToggleButton btnVenn5 = new ToggleButton();
			btnVenn5.setGraphic(new ImageView(imVenn5));
			btnVenn5.setToggleGroup(vennGroup);
			btnVenn5.setUserData("v5");
			vPane.getChildren().add(btnVenn5);
		}
		bottomBP.setLeft(vPane);
	}

	public void closeInputWindow() {
		dataInputStage.close();
	}

	public void showInputWindow() {
		dataInputStage.show();
	}

	public void hideInputWindow() {
		dataInputStage.hide();
	}
}

