package vennDIS;

// Copyright (C) 2014 Vladimir Ignatchenko (vladimirsign@gmail.com)
// Dr. Thomas Kislinger laboratory (http://kislingerlab.uhnres.utoronto.ca/)
//
// This file is part of VennDIS software.
// VennDIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VennDIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VennDIS. If not, see <http://www.gnu.org/licenses/>.

import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.GridPane;
import javafx.scene.control.Button;
import javafx.scene.control.Slider;
import javafx.geometry.Orientation;

import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.Text;

import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.image.WritableImage;
import javafx.scene.SnapshotParameters;

public class ExportImageWindow {
	private Stage exportImageStage;
	private BorderPane exportImageMainBP;
	private BorderPane vennPlotBP;
	public Button btnExport;
	public double IR = 1.0;

	public ExportImageWindow(BorderPane vennPlotBP) {
		exportImageStage = new Stage();
		exportImageStage.setTitle("Export image...");
		exportImageMainBP = new BorderPane();
		exportImageMainBP.setStyle("-fx-background-color: linear-gradient(to bottom, derive(-fx-base,-30%), derive(-fx-base,-60%)), linear-gradient(to bottom, black 0%, gray 100%);");
		Scene exportImageMainScene = new Scene(exportImageMainBP, 900, 700);
		exportImageMainBP.prefHeightProperty().bind(exportImageMainScene.heightProperty());
		exportImageMainBP.prefWidthProperty().bind(exportImageMainScene.widthProperty());


		GridPane gridResolution = new GridPane();
		gridResolution.setHgap(10);
		gridResolution.setVgap(2);

		Slider sliderImageResolution = new Slider(1, 3, 1);
		sliderImageResolution.setShowTickLabels(false);
		sliderImageResolution.setShowTickMarks(true);
		sliderImageResolution.setMajorTickUnit(1);
		sliderImageResolution.setMinorTickCount(1);
		sliderImageResolution.setBlockIncrement(1);
		sliderImageResolution.setOrientation(Orientation.VERTICAL);
		sliderImageResolution.setPrefHeight(400);


		gridResolution.add(sliderImageResolution, 1, 1, 1, 4); 


		Text highResolution = new Text("High");
		highResolution.setFill(Color.rgb(255, 255, 255, 1.0));
		highResolution.setFont(new Font(20));
		gridResolution.add(highResolution, 1, 0);

		Text normalResolution = new Text("Normal");
		normalResolution.setFill(Color.rgb(255, 255, 255, 1.0));
		normalResolution.setFont(new Font(20));
		gridResolution.add(normalResolution, 1, 5);

		WritableImage wim = new WritableImage((int)vennPlotBP.getWidth(), (int)vennPlotBP.getHeight());
		final SnapshotParameters snapshotParameters = new SnapshotParameters();
		snapshotParameters.setFill(Color.TRANSPARENT);
		vennPlotBP.snapshot(snapshotParameters, wim);

		ImageView imagePreview = new ImageView();
		imagePreview.setImage(wim);
		imagePreview.setFitWidth((int)vennPlotBP.getWidth()*0.4);
		imagePreview.setFitHeight((int)vennPlotBP.getHeight()*0.4);
		gridResolution.add(imagePreview, 2, 1, 1, 4);

		Text imageSize = new Text("width:  "+(int)vennPlotBP.getWidth()+"\nheight: "+(int)vennPlotBP.getHeight());
		imageSize.setFill(Color.rgb(255, 255, 255, 1.0));
		imageSize.setFont(new Font(20));
		gridResolution.add(imageSize, 3, 1, 1, 4);

		btnExport = new Button("Export");
		btnExport.setFont(new Font(20));
		gridResolution.add(btnExport, 3, 5);

		exportImageMainBP.setLeft(gridResolution);

		sliderImageResolution.valueProperty().addListener(e->{
 			IR = sliderImageResolution.getValue();
			imageSize.setText("width:  "+Math.round((int)vennPlotBP.getWidth()*IR)+"\nheight: "+Math.round((int)vennPlotBP.getHeight()*IR));
		});

		exportImageStage.setScene(exportImageMainScene);
		exportImageStage.show();
	}

	public void closeWindow() {
		exportImageStage.close();
	}
}
