package vennDIS;

// Copyright (C) 2014 Vladimir Ignatchenko (vladimirsign@gmail.com)
// Dr. Thomas Kislinger laboratory (http://kislingerlab.uhnres.utoronto.ca/)
//
// This file is part of VennDIS software.
// VennDIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VennDIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VennDIS. If not, see <http://www.gnu.org/licenses/>.

import javafx.scene.Group;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Ellipse;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.io.FileWriter;
import java.io.IOException;


public class FX2SVG {

	public static String colorToHEX(Color c) {
		return String.format( "#%02X%02X%02X", (int)(c.getRed()*255+0.5), (int)(c.getGreen()*255+0.5), (int)(c.getBlue()*255+0.5));
	}

	public static double colorToOpacity(Color c) {
		return ((double)((int)(c.getOpacity()*100))/100);
	}

	public static String getColorFromStyle(String s) {
		Pattern p = Pattern.compile("-fx-background-color: rgba(.*);");
		Matcher m = p.matcher(s);
		String rgba = "";
		String rgbHex = "";
		if (m.find( )) { 
			rgba = m.group(1);
			String[] rgbaArray = (rgba.substring(1, rgba.length()-1)).split(",");
			Color c = Color.rgb((int)(Double.parseDouble(rgbaArray[0])),((int)Double.parseDouble(rgbaArray[1])),(int)(Double.parseDouble(rgbaArray[2])));
			rgbHex = colorToHEX(c);
		}
		return rgbHex;
	}

	public static double getOpacityFromStyle(String s) {
		Pattern p = Pattern.compile("-fx-background-color: (.*);");
		Matcher m = p.matcher(s);
		String rgba = "";
		double opacity = 0;
		if (m.find( )) { 
			rgba = m.group(1);
			String[] rgbaArray = (rgba.substring(1, rgba.length()-1)).split(",");
			opacity = ((double)((int)((Double.parseDouble(rgbaArray[3]))*100.0)))/100;
		}
		return opacity;
	}

	public static String getShadowFilter() {
		String sf = ""; 
		sf += "  <defs>\n";
		sf += "    <filter id=\"shadowFilter\" x=\"0\" y=\"0\" width=\"120%\" height=\"120%\">\n";
		sf += "      <feOffset result=\"offOut\" in=\"SourceAlpha\" dx=\"5\" dy=\"5\" />\n";
		sf += "      <feColorMatrix result=\"matrixOut\" in=\"offOut\" type=\"matrix\" values=\"0.2 0 0 0 0 0 0.2 0 0 0 0 0 0.2 0 0 0 0 0 0.7 0\" />\n";
		sf += "      <feGaussianBlur result=\"blurOut\" in=\"matrixOut\" stdDeviation=\"2 2\" />\n";
		sf += "      <feBlend in=\"SourceGraphic\" in2=\"blurOut\" mode=\"normal\" />\n";
		sf += "    </filter>\n";
		sf += "  </defs>\n";
		return sf;
	}

	public static String ellipseToSvg(Ellipse el) {
		String strokeColor = colorToHEX((Color)el.getStroke());
		double strokeOpacity = colorToOpacity((Color)el.getStroke());
		String fillColor = colorToHEX((Color)el.getFill());
		double fillOpacity = colorToOpacity((Color)el.getFill());
		double angle = el.getRotate();
		String fxStyle = el.getStyle();
		String svgStyle = "";
		if (fxStyle.toString().equals("-fx-stroke-dash-array: 10 20 10 20; -fx-stroke-line-cap: round;")) {
			svgStyle = "stroke-dasharray:10,20; stroke-linecap:round;";
		} else if (fxStyle.toString().equals("-fx-stroke-dash-array: 1 15 1 15; -fx-stroke-line-cap: round;")) {
			svgStyle = "stroke-dasharray:1,15; stroke-linecap:round;";
		} else {
			svgStyle = "";
		}		
		String svgString = "  <ellipse id=\""+el.getId()+"\" cx=\""+el.getCenterX()+"\" cy=\""+el.getCenterY()+"\" rx=\""+el.getRadiusX()+"\" ry=\""+el.getRadiusY()+"\" stroke-width=\""+el.getStrokeWidth()+"\" stroke=\""+strokeColor+"\" stroke-opacity=\""+strokeOpacity+"\" fill=\""+fillColor+"\" fill-opacity=\""+fillOpacity+"\" style=\""+svgStyle+"\" transform=\"rotate("+angle+" "+el.getCenterX()+","+el.getCenterY()+")\" />\n";
		return svgString;
	}

	public static String textToSvg(Text el) {
		double y = 0;
		if (el.getTextOrigin().toString().equals("CENTER")) { 
			y = el.getY()+(el.getLayoutBounds().getHeight()/4); 
		} else if (el.getTextOrigin().toString().equals("BOTTOM")){
			y = el.getY();
		} else if (el.getTextOrigin().toString().equals("TOP")){
			y = el.getY()+(el.getLayoutBounds().getHeight()); 
		} else if (el.getTextOrigin().toString().equals("BASELINE")){
			y = el.getY(); 
		}
		double x = el.getX()+(el.getLayoutBounds().getWidth()/2);
		if (el.getId().toString().equals("headerA")) { x = x + el.getLayoutX(); y = y + el.getLayoutY(); }
		if (el.getId().toString().equals("headerB")) { x = x + el.getLayoutX(); y = y + el.getLayoutY(); }
		if (el.getId().toString().equals("headerC")) { x = x + el.getLayoutX(); y = y + el.getLayoutY(); }
		if (el.getId().toString().equals("headerD")) { x = x + el.getLayoutX(); y = y + el.getLayoutY(); }
		if (el.getId().toString().equals("headerE")) { x = x + el.getLayoutX(); y = y + el.getLayoutY(); }
		String fillColor = colorToHEX((Color)el.getFill());
		double fillOpacity = colorToOpacity((Color)el.getFill());
		int fontSize = (int)el.getFont().getSize();
		String fontFamily = el.getFont().getFamily();
		String fontStyle = el.getFont().getStyle();
		Pattern p = Pattern.compile("Italic");
		Matcher m = p.matcher(fontStyle);
		String fontI, fontB;
		if (m.find( )) { fontI = "italic"; } else { fontI = "normal"; }
		p = Pattern.compile("Bold");
		m = p.matcher(fontStyle);
		if (m.find( )) { fontB = "bold"; } else { fontB = "normal"; }
		String underline = "";
		if (el.isUnderline()) { underline = "underline"; } else { underline = ""; }
		String shadow = "";
		if (el.getEffect()==null) { shadow = ""; } else { shadow = "url(#shadowFilter)"; }
		double angle = el.getRotate();
		double scale = el.getScaleX();
		String svgString = "  <text id=\""+el.getId()+"\" x=\""+x+"\" y=\""+y+"\" fill=\""+fillColor+"\" fill-opacity=\""+fillOpacity+"\" text-anchor=\"middle\" font-size=\""+((int)(fontSize*scale))+"\" font-family=\""+fontFamily+"\" text-decoration=\""+underline+"\" font-weight = \""+fontB+"\" font-style = \""+fontI+"\" filter=\""+shadow+"\" transform=\"rotate("+angle+" "+x+","+(y-(el.getLayoutBounds().getHeight()/4))+")\">"+el.getText()+"</text>\n";
		return svgString;
	}

	public static void saveSvg(String file, BorderPane el) {
		try {
			Group g = (Group)el.getChildren().get(0);
			int z = g.getChildren().size();
			double xShift = 0;
			double yShift = 0;
			for (int i=0; i<z; i++) {
				if (g.getChildren().get(i).getId().toString().equals("anchorPoint")) {
					xShift = ((Rectangle)g.getChildren().get(i)).getX();
					yShift = ((Rectangle)g.getChildren().get(i)).getY();
				}
			}
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" version=\"1.1\" width=\""+el.getWidth()+"\" height=\""+el.getHeight()+"\" viewBox=\""+xShift+" "+((int)(yShift-0.99))+" "+el.getWidth()+" "+el.getHeight()+"\">\n");
			fw.write(getShadowFilter());
			String bgfillColor = "";
			double bgfillOpacity = 0;
			for (int i=0; i<z; i++) {
				if (g.getChildren().get(i).getId().toString().equals("bg")) {
					bgfillColor = colorToHEX((Color)((Rectangle)g.getChildren().get(i)).getFill());
					bgfillOpacity = colorToOpacity((Color)((Rectangle)g.getChildren().get(i)).getFill());
				}
			}
			fw.write(" <rect x=\""+xShift+"\" y=\""+((int)(yShift-0.99))+"\" width=\""+el.getWidth()+"\" height=\""+el.getHeight()+"\" fill=\""+bgfillColor+"\" fill-opacity=\""+bgfillOpacity+"\"/>\n");
			for (int i=0; i<z; i++) {
				if (g.getChildren().get(i).getId().toString().equals("cBox")) {
					Group cbox = (Group)g.getChildren().get(i);
					for (int j=0; j<cbox.getChildren().size(); j++) {
						if (cbox.getChildren().get(j).getId().toString().equals("circleA")) {
							fw.write(ellipseToSvg((Ellipse)cbox.getChildren().get(j)));
						}
						if (cbox.getChildren().get(j).getId().toString().equals("circleB")) {
							fw.write(ellipseToSvg((Ellipse)cbox.getChildren().get(j)));
						}
						if (cbox.getChildren().get(j).getId().toString().equals("circleC")) {
							fw.write(ellipseToSvg((Ellipse)cbox.getChildren().get(j)));
						}
						if (cbox.getChildren().get(j).getId().toString().equals("circleD")) {
							fw.write(ellipseToSvg((Ellipse)cbox.getChildren().get(j)));
						}
						if (cbox.getChildren().get(j).getId().toString().equals("circleE")) {
							fw.write(ellipseToSvg((Ellipse)cbox.getChildren().get(j)));
						}
					}
				}
			}
			for (int i=0; i<z; i++) {
				for (int v=1; v<32; v++) {
					if (g.getChildren().get(i).getId().toString().equals(v+"")) {
						fw.write(textToSvg((Text)g.getChildren().get(i)));
					}
				}
			}
			for (int i=0; i<z; i++) {
				if (g.getChildren().get(i).getId().toString().equals("headerA")) {
					fw.write(textToSvg((Text)g.getChildren().get(i)));
				}
				if (g.getChildren().get(i).getId().toString().equals("headerB")) {
					fw.write(textToSvg((Text)g.getChildren().get(i)));
				}
				if (g.getChildren().get(i).getId().toString().equals("headerC")) {
					fw.write(textToSvg((Text)g.getChildren().get(i)));
				}
				if (g.getChildren().get(i).getId().toString().equals("headerD")) {
					fw.write(textToSvg((Text)g.getChildren().get(i)));
				}
				if (g.getChildren().get(i).getId().toString().equals("headerE")) {
					fw.write(textToSvg((Text)g.getChildren().get(i)));
				}
			}
			fw.write("</svg>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}



