package vennDIS;

// Copyright (C) 2014 Vladimir Ignatchenko (vladimirsign@gmail.com)
// Dr. Thomas Kislinger laboratory (http://kislingerlab.uhnres.utoronto.ca/)
//
// This file is part of VennDIS software.
// VennDIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VennDIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VennDIS. If not, see <http://www.gnu.org/licenses/>.

import javafx.application.Application;
import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.Group;
import javafx.scene.layout.BorderPane;
import java.io.File;
import javafx.stage.FileChooser;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.Clipboard;
import javafx.scene.text.Text;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.FontPosture;
import javafx.scene.effect.DropShadow;
import javafx.scene.shape.Rectangle;

public class VennDIS extends Application {

	private VennToolBar vennTB;
	private VennTabPane vennTP;
	private Group gBox;
	private VennCalc vennCalc;
	private BorderPane borderPane;
	private FileChooser fileChooser;
	private File file;
	private String currentDirectory = "./";
	private VennIO vio;
	private Venn2 v2;
	private Venn3 v3;
	private Venn4 v4;
	private Venn5 v5;
	private Euler1 e1;
	private Euler2 e2;
	private Euler3 e3;
	private ExportImageWindow EIW;
	private DataInputWindow dataInputWindow;

	public static void main(String[] args) {
		launch(args);
	}

	@Override
	public void start(Stage primaryStage) {
		try {
			primaryStage.getIcons().add(new Image(getClass().getResourceAsStream("icons/VennDIS.png")));
			primaryStage.setTitle("VennDIS v1.2");
			//primaryStage.setMaximized(true);
			borderPane = new BorderPane();
			Scene scene = new Scene(borderPane, 1600, 900);
			scene.getStylesheets().add("vennDIS/style.css");
			//borderPane.setId("mainBorderPaneStart");
			borderPane.setId("mainBorderPane");
			borderPane.prefHeightProperty().bind(scene.heightProperty());
			borderPane.prefWidthProperty().bind(scene.widthProperty());
			startBP();

			vennTB = new VennToolBar(borderPane);
			vennCalc = new VennCalc();
			vio = new VennIO();
		
			vennTB.btnNew.setOnAction(event->{ newVenn(); });
			vennTB.btnEdit.setOnAction(event->{ editVenn(); });
			vennTB.btnOpen.setOnAction(event->{ openVenn(); });
			vennTB.btnSave.setOnAction(event->{ saveVenn(); });
			vennTB.btnSaveImage.setOnAction(event->{ exportImage(); });
			vennTB.btnHelp.setOnAction(event->{ helpContent(); });
			vennTB.btnWFS.setOnAction(e->{ if (primaryStage.isFullScreen()) { primaryStage.setFullScreen(false); } else { primaryStage.setFullScreen(true); } });
			primaryStage.setScene(scene);
			primaryStage.show();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	private void startBP() {
		gBox = new Group();
		DropShadow dropShadow = new DropShadow();
		dropShadow.setRadius(10.0);
		dropShadow.setOffsetX(5.0);
		dropShadow.setOffsetY(5.0);
		dropShadow.setColor(Color.color(0.3, 0.3, 0.3));
		Rectangle bg = new Rectangle(0, 0, 1000, 600);
		bg.setFill(Color.rgb(255, 255, 255, 0));
		Text textBG = new Text(180, 350, "VennDIS");
		textBG.setFill(Color.rgb(255, 255, 255, 1.0));
		textBG.setFont(Font.font("System", FontPosture.ITALIC, 144));
		textBG.setEffect(dropShadow);
		textBG.setStyle("-fx-stroke: black; -fx-stroke-width: 1; -fx-fill: linear-gradient(from 0% 0% to 45% 100%, repeat, aqua 0%, blue 50%);");
		gBox.getChildren().addAll(bg, textBG);
		Image imV2 = new Image(getClass().getResourceAsStream("icons/venn2x_icon128.png"));
		ImageView ivV2 = new ImageView();
		ivV2.setImage(imV2);
		ivV2.setX(10);
		ivV2.setY(235);
		Image imV3 = new Image(getClass().getResourceAsStream("icons/venn3x_icon128.png"));
		ImageView ivV3 = new ImageView();
		ivV3.setImage(imV3);
		ivV3.setX(435);
		ivV3.setY(10);
		Image imV4 = new Image(getClass().getResourceAsStream("icons/venn4x_icon128.png"));
		ImageView ivV4 = new ImageView();
		ivV4.setImage(imV4);
		ivV4.setX(435);
		ivV4.setY(465);
		Image imV5 = new Image(getClass().getResourceAsStream("icons/venn5x_icon128.png"));
		ImageView ivV5 = new ImageView();
		ivV5.setImage(imV5);
		ivV5.setX(860);
		ivV5.setY(235);
		Image imE023 = new Image(getClass().getResourceAsStream("icons/euler023_icon128.png"));
		ImageView ivE023 = new ImageView();
		ivE023.setImage(imE023);
		ivE023.setX(250);
		ivE023.setY(20);
		Image imE063 = new Image(getClass().getResourceAsStream("icons/euler063_icon128.png"));
		ImageView ivE063 = new ImageView();
		ivE063.setImage(imE063);
		ivE063.setX(80);
		ivE063.setY(70);
		Image imE069 = new Image(getClass().getResourceAsStream("icons/euler069_icon128.png"));
		ImageView ivE069 = new ImageView();
		ivE069.setImage(imE069);
		ivE069.setX(790);
		ivE069.setY(70);
		Image imE071 = new Image(getClass().getResourceAsStream("icons/euler071_icon128.png"));
		ImageView ivE071 = new ImageView();
		ivE071.setImage(imE071);
		ivE071.setX(620);
		ivE071.setY(20);
		Image imE075 = new Image(getClass().getResourceAsStream("icons/euler075_icon128.png"));
		ImageView ivE075 = new ImageView();
		ivE075.setImage(imE075);
		ivE075.setX(620);
		ivE075.setY(450);
		Image imE085 = new Image(getClass().getResourceAsStream("icons/euler085_icon128.png"));
		ImageView ivE085 = new ImageView();
		ivE085.setImage(imE085);
		ivE085.setX(250);
		ivE085.setY(450);
		Image imE123 = new Image(getClass().getResourceAsStream("icons/euler123_icon128.png"));
		ImageView ivE123 = new ImageView();
		ivE123.setImage(imE123);
		ivE123.setX(80);
		ivE123.setY(400);
		Image imE125 = new Image(getClass().getResourceAsStream("icons/euler125_icon128.png"));
		ImageView ivE125 = new ImageView();
		ivE125.setImage(imE125);
		ivE125.setX(790);
		ivE125.setY(400);
		gBox.getChildren().addAll(ivV2, ivV3, ivV4, ivV5, ivE023, ivE063, ivE069, ivE071, ivE075, ivE085, ivE123, ivE125);
		borderPane.setCenter(gBox);
	}

	private void newVenn() {
		vennCalc = new VennCalc();
		borderPane.setLeft(null);
		borderPane.setCenter(gBox);
		dataInputWindow = new DataInputWindow(vennCalc);
		dataInputWindow.btnOK.setOnAction(e->{ 
			String vType;
			try {
				vType = (String)dataInputWindow.vennGroup.getSelectedToggle().getUserData();
			} catch(Exception ex) {
				vType = "";
			}
			createDiagram(vType);
			dataInputWindow.hideInputWindow();
			//dataInputWindow.closeInputWindow();
		});
	}

	private void editVenn() {
		dataInputWindow.showInputWindow();
	}

	private void helpContent() {
		HelpWebView helpWindow = new HelpWebView();
	}

	private void createDiagram(String vType) {
			vennCalc.vennType = vType;
			switch (vType) {
				case "v2":
					v2 = new Venn2(borderPane, vennCalc);
					break;
				case "v3":
					v3 = new Venn3(borderPane, vennCalc);
					break;
				case "v4":
					v4 = new Venn4(borderPane, vennCalc);
					break;
				case "v5":
					v5 = new Venn5(borderPane, vennCalc);
					break;
				case "e001":
					e1 = new Euler1(borderPane, vennCalc);
					break;
				case "e003":
				case "e005":
				case "e006":
					e2 = new Euler2(borderPane, vennCalc);
					break;
				case "e011":
				case "e015":
				case "e021":
				case "e023":
				case "e027":
				case "e029":
				case "e031":
				case "e038":
				case "e039":
				case "e043":
				case "e046":
				case "e047":
				case "e056":
				case "e057":
				case "e058":
				case "e059":
				case "e063":
				case "e069":
				case "e070":
				case "e071":
				case "e075":
				case "e077":
				case "e078":
				case "e079":
				case "e081":
				case "e083":
				case "e085":
				case "e087":
				case "e088":
				case "e089":
				case "e090":
				case "e091":
				case "e093":
				case "e094":
				case "e095":
				case "e098":
				case "e099":
				case "e102":
				case "e103":
				case "e104":
				case "e105":
				case "e106":
				case "e107":
				case "e109":
				case "e110":
				case "e111":
				case "e115":
				case "e119":
				case "e120":
				case "e121":
				case "e122":
				case "e123":
				case "e125":
				case "e126":
					e3 = new Euler3(borderPane, vennCalc);
					break;
				default:
					if (vennCalc.getSetsN()==1) {
						vennCalc.vennType = "e001";
						e1 = new Euler1(borderPane, vennCalc);
					}
					if (vennCalc.getSetsN()==2) {
						vennCalc.vennType = "v2";
						v2 = new Venn2(borderPane, vennCalc);
					}
					if (vennCalc.getSetsN()==3) {
						vennCalc.vennType = "v3";
						v3 = new Venn3(borderPane, vennCalc);
					}
					if (vennCalc.getSetsN()==4) {
						vennCalc.vennType = "v4";
						v4 = new Venn4(borderPane, vennCalc);
					}
					if (vennCalc.getSetsN()==5) {
						vennCalc.vennType = "v5";
						v5 = new Venn5(borderPane, vennCalc);
					}
					break;
			}
	}

	private void openVenn() {
		fileChooser = new FileChooser();
		fileChooser.setTitle("Open Venn/Euler diagram ...");
		fileChooser.setInitialDirectory(new File(currentDirectory)); 
		fileChooser.getExtensionFilters().addAll(
			new FileChooser.ExtensionFilter("XML", "*.xml")
		);
		file = fileChooser.showOpenDialog(null);
		if (file!=null) {
			currentDirectory = file.getParent().toString();
			vio.readVennXML(file);
			switch (vio.vennXML.vennType) {
				case "v2":
					vennCalc = new VennCalc();
					v2 = new Venn2(borderPane, vennCalc);
					vio.loadVennXML(v2);
					break;
				case "v3":
					vennCalc = new VennCalc();
					v3 = new Venn3(borderPane, vennCalc);
					vio.loadVennXML(v3);
					break;
				case "v4":
					vennCalc = new VennCalc();
					v4 = new Venn4(borderPane, vennCalc);
					vio.loadVennXML(v4);
					break;
				case "v5":
					vennCalc = new VennCalc();
					v5 = new Venn5(borderPane, vennCalc);
					vio.loadVennXML(v5);
					break;
				case "e001":
					vennCalc = new VennCalc();
					vennCalc.vennType = vio.vennXML.vennType;
					e1 = new Euler1(borderPane, vennCalc);
					vio.loadVennXML(e1);
					break;
				case "e003":
				case "e005":
				case "e006":
					vennCalc = new VennCalc();
					vennCalc.vennType = vio.vennXML.vennType;
					e2 = new Euler2(borderPane, vennCalc);
					vio.loadVennXML(e2);
					break;
				case "e011":
				case "e015":
				case "e021":
				case "e023":
				case "e027":
				case "e029":
				case "e031":
				case "e038":
				case "e039":
				case "e043":
				case "e046":
				case "e047":
				case "e056":
				case "e057":
				case "e058":
				case "e059":
				case "e063":
				case "e069":
				case "e070":
				case "e071":
				case "e075":
				case "e077":
				case "e078":
				case "e079":
				case "e081":
				case "e083":
				case "e085":
				case "e087":
				case "e088":
				case "e089":
				case "e090":
				case "e091":
				case "e093":
				case "e094":
				case "e095":
				case "e098":
				case "e099":
				case "e102":
				case "e103":
				case "e104":
				case "e106":
				case "e105":
				case "e107":
				case "e109":
				case "e110":
				case "e111":
				case "e115":
				case "e119":
				case "e120":
				case "e121":
				case "e122":
				case "e123":
				case "e125":
				case "e126":
					vennCalc = new VennCalc();
					vennCalc.vennType = vio.vennXML.vennType;
					e3 = new Euler3(borderPane, vennCalc);
					vio.loadVennXML(e3);
					break;
			}
		}
	}

	private void saveVenn() {
		fileChooser = new FileChooser();
		fileChooser.setTitle("Save Venn/Euler diagram ...");
		fileChooser.setInitialDirectory(new File(currentDirectory));
		fileChooser.getExtensionFilters().addAll(
			new FileChooser.ExtensionFilter("XML", "*.xml")
		);
		file = fileChooser.showSaveDialog(null);
		if (file!=null) {
			if (!(file.getName().endsWith(".xml"))) {
				file = new File(file.getAbsolutePath() + fileChooser.selectedExtensionFilterProperty().get().getExtensions().get(0).substring(1));
			}
			currentDirectory = file.getParent().toString();
			switch (vennCalc.vennType) {
				case "v2":
					vio.saveVennXML(v2, file);
					break;
				case "v3":
					vio.saveVennXML(v3, file);
					break;
				case "v4":
					vio.saveVennXML(v4, file);
					break;
				case "v5":
					vio.saveVennXML(v5, file);
					break;
				case "e001":
					vio.saveVennXML(e1, file);
					break;
				case "e003":
				case "e005":
				case "e006":
					vio.saveVennXML(e2, file);
					break;
				case "e011":
				case "e015":
				case "e021":
				case "e023":
				case "e027":
				case "e029":
				case "e031":
				case "e038":
				case "e039":
				case "e043":
				case "e046":
				case "e047":
				case "e056":
				case "e057":
				case "e058":
				case "e059":
				case "e063":
				case "e069":
				case "e070":
				case "e071":
				case "e075":
				case "e077":
				case "e078":
				case "e079":
				case "e081":
				case "e083":
				case "e085":
				case "e087":
				case "e088":
				case "e089":
				case "e090":
				case "e091":
				case "e093":
				case "e094":
				case "e095":
				case "e098":
				case "e099":
				case "e102":
				case "e103":
				case "e104":
				case "e105":
				case "e106":
				case "e107":
				case "e109":
				case "e110":
				case "e111":
				case "e115":
				case "e119":
				case "e120":
				case "e121":
				case "e122":
				case "e123":
				case "e125":
				case "e126":
					vio.saveVennXML(e3, file);
					break;
			}
		}
	}

	private void exportImage() {
		switch (vennCalc.vennType) {
			case "v2":
				EIW = new ExportImageWindow(v2.vennPlot);
				EIW.btnExport.setOnAction(event1->{ saveImage(); });
				break;
			case "v3":
				EIW = new ExportImageWindow(v3.vennPlot);
				EIW.btnExport.setOnAction(event->{ saveImage(); });
				break;
			case "v4":
				EIW = new ExportImageWindow(v4.vennPlot);
				EIW.btnExport.setOnAction(event->{ saveImage(); });
				break;
			case "v5":
				EIW = new ExportImageWindow(v5.vennPlot);
				EIW.btnExport.setOnAction(event->{ saveImage(); });
				break;
			case "e001":
				EIW = new ExportImageWindow(e1.vennPlot);
				EIW.btnExport.setOnAction(event->{ saveImage(); });
				break;
			case "e003":
			case "e005":
			case "e006":
				EIW = new ExportImageWindow(e2.vennPlot);
				EIW.btnExport.setOnAction(event->{ saveImage(); });
				break;
			case "e011":
			case "e015":
			case "e021":
			case "e023":
			case "e027":
			case "e029":
			case "e031":
			case "e038":
			case "e039":
			case "e043":
			case "e046":
			case "e047":
			case "e056":
			case "e057":
			case "e058":
			case "e059":
			case "e063":
			case "e069":
			case "e070":
			case "e071":
			case "e075":
			case "e077":
			case "e078":
			case "e079":
			case "e081":
			case "e083":
			case "e085":
			case "e087":
			case "e088":
			case "e089":
			case "e090":
			case "e091":
			case "e093":
			case "e094":
			case "e095":
			case "e098":
			case "e099":
			case "e102":
			case "e103":
			case "e104":
			case "e105":
			case "e106":
			case "e107":
			case "e109":
			case "e110":
			case "e111":
			case "e115":
			case "e119":
			case "e120":
			case "e121":
			case "e122":
			case "e123":
			case "e125":
			case "e126":
				EIW = new ExportImageWindow(e3.vennPlot);
				EIW.btnExport.setOnAction(event->{ saveImage(); });
				break;
			case "default":
				break;
		}
	}

	public void saveImage() {
		vio.res = EIW.IR;
		EIW.closeWindow();
		fileChooser = new FileChooser();
		fileChooser.setTitle("Save Image");
		fileChooser.setInitialDirectory(new File(currentDirectory));
		fileChooser.getExtensionFilters().addAll(
			new FileChooser.ExtensionFilter("PNG image (*.png)", "*.png"),
			new FileChooser.ExtensionFilter("JPEG image (*.jpg)", "*.jpg"),
			new FileChooser.ExtensionFilter("BMP image (*.bmp)", "*.bmp"),
			new FileChooser.ExtensionFilter("Scalable Vector Graphics (*.svg)", "*.svg")
		);
		file = fileChooser.showSaveDialog(null);
		if (file!=null) {
			currentDirectory = file.getParent().toString();
			if (!(file.getName().endsWith(".png")||file.getName().endsWith(".jpg")||file.getName().endsWith(".bmp")||file.getName().endsWith(".svg"))) {
				file = new File(file.getAbsolutePath() + fileChooser.selectedExtensionFilterProperty().get().getExtensions().get(0).substring(1));
			}
			switch (vennCalc.vennType) {
				case "v2":
					vio.saveImage(file, v2.vennPlot);
					break;
				case "v3":
					vio.saveImage(file, v3.vennPlot);
					break;
				case "v4":
					vio.saveImage(file, v4.vennPlot);
					break;
				case "v5":
					vio.saveImage(file, v5.vennPlot);
					break;
				case "e001":
					vio.saveImage(file, e1.vennPlot);
					break;
				case "e003":
				case "e005":
				case "e006":
					vio.saveImage(file, e2.vennPlot);
					break;
				case "e011":
				case "e015":
				case "e021":
				case "e023":
				case "e027":
				case "e029":
				case "e031":
				case "e038":
				case "e039":
				case "e043":
				case "e046":
				case "e047":
				case "e056":
				case "e057":
				case "e058":
				case "e059":
				case "e063":
				case "e069":
				case "e070":
				case "e071":
				case "e075":
				case "e077":
				case "e078":
				case "e079":
				case "e081":
				case "e083":
				case "e085":
				case "e087":
				case "e088":
				case "e089":
				case "e090":
				case "e091":
				case "e093":
				case "e094":
				case "e095":
				case "e098":
				case "e099":
				case "e102":
				case "e103":
				case "e104":
				case "e105":
				case "e106":
				case "e107":
				case "e109":
				case "e110":
				case "e111":
				case "e115":
				case "e119":
				case "e120":
				case "e121":
				case "e122":
				case "e123":
				case "e125":
				case "e126":
					vio.saveImage(file, e3.vennPlot);
					break;
				case "default":
					break;
			}
		}
	}
}



