package vennDIS;

// Copyright (C) 2014 Vladimir Ignatchenko (vladimirsign@gmail.com)
// Dr. Thomas Kislinger laboratory (http://kislingerlab.uhnres.utoronto.ca/)
//
// This file is part of VennDIS software.
// VennDIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VennDIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VennDIS. If not, see <http://www.gnu.org/licenses/>.

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import java.io.File;
import javafx.scene.image.WritableImage;
import javafx.scene.SnapshotParameters;
import javafx.embed.swing.SwingFXUtils;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import java.util.ArrayList;
import java.io.FileWriter;
import java.io.IOException;


public class VennIO {
	
	public class VennXML {
		public String vennType;
		public String bgcolor;
		public String valuecolor;
		public int fontSize;
		public String fontFamily;
		public String fontStyle;
		public String HeaderA, HeaderB, HeaderC, HeaderD, HeaderE;
		public double lxHeaderA, lxHeaderB, lxHeaderC, lxHeaderD, lxHeaderE;
		public double lyHeaderA, lyHeaderB, lyHeaderC, lyHeaderD, lyHeaderE;
		public double angleHeaderA, angleHeaderB, angleHeaderC, angleHeaderD, angleHeaderE;
		public String textcolorHeaderA, textcolorHeaderB, textcolorHeaderC, textcolorHeaderD, textcolorHeaderE;
		public int fontSizeHeaderA, fontSizeHeaderB, fontSizeHeaderC, fontSizeHeaderD, fontSizeHeaderE;
		public String fontFamilyHeaderA, fontFamilyHeaderB, fontFamilyHeaderC, fontFamilyHeaderD, fontFamilyHeaderE;
		public String fontStyleHeaderA, fontStyleHeaderB, fontStyleHeaderC, fontStyleHeaderD, fontStyleHeaderE;
		public String fillEllipseA, fillEllipseB, fillEllipseC, fillEllipseD, fillEllipseE; 
		public String strokeEllipseA, strokeEllipseB, strokeEllipseC, strokeEllipseD, strokeEllipseE; 
		public double widthEllipseA, widthEllipseB, widthEllipseC, widthEllipseD, widthEllipseE; 
		public double sizeEllipseA, sizeEllipseB, sizeEllipseC, sizeEllipseD, sizeEllipseE; 
		public String datasetA, datasetB, datasetC, datasetD, datasetE;
		public int[] overlaps = new int[32];
		public double overlapDist;
	}

	public double res;
	public VennXML vennXML;

	public VennIO() {
		res = 1.0;
	}

	public void readVennXML(File file) {
		try {
			vennXML = new VennXML();
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();

			DefaultHandler handler = new DefaultHandler() {
				boolean hA = false;
				boolean hB = false;
				boolean hC = false;
				boolean hD = false;
				boolean hE = false;
				boolean dsA = false;
				boolean dsB = false;
				boolean dsC = false;
				boolean dsD = false;
				boolean dsE = false;
				String data = "";
				public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
					if (qName=="venn") {
						vennXML.vennType = attributes.getValue("type");
					}
					if (qName=="main") {
						vennXML.bgcolor = attributes.getValue("bgcolor").toString();
						vennXML.valuecolor = attributes.getValue("valuecolor").toString();
						vennXML.fontSize = Integer.parseInt(attributes.getValue("fontSize").toString());
						vennXML.fontFamily = attributes.getValue("fontFamily").toString();
						vennXML.fontStyle = attributes.getValue("fontStyle").toString();
					}
					if (qName=="titleA") {
						vennXML.lxHeaderA = Double.parseDouble(attributes.getValue("lx").toString());
						vennXML.lyHeaderA = Double.parseDouble(attributes.getValue("ly").toString());
						vennXML.angleHeaderA = Double.parseDouble(attributes.getValue("angle").toString());
						vennXML.textcolorHeaderA = attributes.getValue("textcolor").toString();
						vennXML.fontSizeHeaderA = Integer.parseInt(attributes.getValue("fontSize").toString());
						vennXML.fontFamilyHeaderA = attributes.getValue("fontFamily").toString();
						vennXML.fontStyleHeaderA = attributes.getValue("fontStyle").toString();
						hA = true;
					}
					if (qName=="ellipseA") {
						vennXML.fillEllipseA = attributes.getValue("fill").toString();
						vennXML.strokeEllipseA = attributes.getValue("stroke").toString();
						vennXML.widthEllipseA = Double.parseDouble(attributes.getValue("stroke-width").toString());
						vennXML.sizeEllipseA = Double.parseDouble(attributes.getValue("size").toString());
					}
					if (qName=="datasetA") {
						dsA = true;
					}
					if (qName=="titleB") {
						vennXML.lxHeaderB = Double.parseDouble(attributes.getValue("lx").toString());
						vennXML.lyHeaderB = Double.parseDouble(attributes.getValue("ly").toString());
						vennXML.angleHeaderB = Double.parseDouble(attributes.getValue("angle").toString());
						vennXML.textcolorHeaderB = attributes.getValue("textcolor").toString();
						vennXML.fontSizeHeaderB = Integer.parseInt(attributes.getValue("fontSize").toString());
						vennXML.fontFamilyHeaderB = attributes.getValue("fontFamily").toString();
						vennXML.fontStyleHeaderB = attributes.getValue("fontStyle").toString();
						hB = true;
					}
					if (qName=="ellipseB") {
						vennXML.fillEllipseB = attributes.getValue("fill").toString();
						vennXML.strokeEllipseB = attributes.getValue("stroke").toString();
						vennXML.widthEllipseB = Double.parseDouble(attributes.getValue("stroke-width").toString());
						vennXML.sizeEllipseB = Double.parseDouble(attributes.getValue("size").toString());
					}
					if (qName=="datasetB") {
						dsB = true;
					}
					if (qName=="titleC") {
						vennXML.lxHeaderC = Double.parseDouble(attributes.getValue("lx").toString());
						vennXML.lyHeaderC = Double.parseDouble(attributes.getValue("ly").toString());
						vennXML.angleHeaderC = Double.parseDouble(attributes.getValue("angle").toString());
						vennXML.textcolorHeaderC = attributes.getValue("textcolor").toString();
						vennXML.fontSizeHeaderC = Integer.parseInt(attributes.getValue("fontSize").toString());
						vennXML.fontFamilyHeaderC = attributes.getValue("fontFamily").toString();
						vennXML.fontStyleHeaderC = attributes.getValue("fontStyle").toString();
						hC = true;
					}
					if (qName=="ellipseC") {
						vennXML.fillEllipseC = attributes.getValue("fill").toString();
						vennXML.strokeEllipseC = attributes.getValue("stroke").toString();
						vennXML.widthEllipseC = Double.parseDouble(attributes.getValue("stroke-width").toString());
						vennXML.sizeEllipseC = Double.parseDouble(attributes.getValue("size").toString());
					}
					if (qName=="datasetC") {
						dsC = true;
					}

					if (qName=="titleD") {
						vennXML.lxHeaderD = Double.parseDouble(attributes.getValue("lx").toString());
						vennXML.lyHeaderD = Double.parseDouble(attributes.getValue("ly").toString());
						vennXML.angleHeaderD = Double.parseDouble(attributes.getValue("angle").toString());
						vennXML.textcolorHeaderD = attributes.getValue("textcolor").toString();
						vennXML.fontSizeHeaderD = Integer.parseInt(attributes.getValue("fontSize").toString());
						vennXML.fontFamilyHeaderD = attributes.getValue("fontFamily").toString();
						vennXML.fontStyleHeaderD = attributes.getValue("fontStyle").toString();
						hD = true;
					}
					if (qName=="ellipseD") {
						vennXML.fillEllipseD = attributes.getValue("fill").toString();
						vennXML.strokeEllipseD = attributes.getValue("stroke").toString();
						vennXML.widthEllipseD = Double.parseDouble(attributes.getValue("stroke-width").toString());
						vennXML.sizeEllipseD = Double.parseDouble(attributes.getValue("size").toString());
					}
					if (qName=="datasetD") {
						dsD = true;
					}
					if (qName=="titleE") {
						vennXML.lxHeaderE = Double.parseDouble(attributes.getValue("lx").toString());
						vennXML.lyHeaderE = Double.parseDouble(attributes.getValue("ly").toString());
						vennXML.angleHeaderE = Double.parseDouble(attributes.getValue("angle").toString());
						vennXML.textcolorHeaderE = attributes.getValue("textcolor").toString();
						vennXML.fontSizeHeaderE = Integer.parseInt(attributes.getValue("fontSize").toString());
						vennXML.fontFamilyHeaderE = attributes.getValue("fontFamily").toString();
						vennXML.fontStyleHeaderE = attributes.getValue("fontStyle").toString();
						hE = true;
					}
					if (qName=="ellipseE") {
						vennXML.fillEllipseE = attributes.getValue("fill").toString();
						vennXML.strokeEllipseE = attributes.getValue("stroke").toString();
						vennXML.widthEllipseE = Double.parseDouble(attributes.getValue("stroke-width").toString());
						vennXML.sizeEllipseE = Double.parseDouble(attributes.getValue("size").toString());
					}
					if (qName=="datasetE") {
						dsE = true;
					}
					if (qName=="overlapEllipses") {
						vennXML.overlapDist = Double.parseDouble(attributes.getValue("distance").toString());
					}
					if (qName=="overlapValues") {
						vennXML.overlaps[1] = Integer.parseInt(attributes.getValue("A").toString());
						vennXML.overlaps[2] = Integer.parseInt(attributes.getValue("B").toString());
						vennXML.overlaps[3] = Integer.parseInt(attributes.getValue("AB").toString());
						vennXML.overlaps[4] = Integer.parseInt(attributes.getValue("C").toString());
						vennXML.overlaps[5] = Integer.parseInt(attributes.getValue("AC").toString());
						vennXML.overlaps[6] = Integer.parseInt(attributes.getValue("BC").toString());
						vennXML.overlaps[7] = Integer.parseInt(attributes.getValue("ABC").toString());
						vennXML.overlaps[8] = Integer.parseInt(attributes.getValue("D").toString());
						vennXML.overlaps[9] = Integer.parseInt(attributes.getValue("AD").toString());
						vennXML.overlaps[10] = Integer.parseInt(attributes.getValue("BD").toString());
						vennXML.overlaps[11] = Integer.parseInt(attributes.getValue("ABD").toString());
						vennXML.overlaps[12] = Integer.parseInt(attributes.getValue("CD").toString());
						vennXML.overlaps[13] = Integer.parseInt(attributes.getValue("ACD").toString());
						vennXML.overlaps[14] = Integer.parseInt(attributes.getValue("BCD").toString());
						vennXML.overlaps[15] = Integer.parseInt(attributes.getValue("ABCD").toString());
						vennXML.overlaps[16] = Integer.parseInt(attributes.getValue("E").toString());
						vennXML.overlaps[17] = Integer.parseInt(attributes.getValue("AE").toString());
						vennXML.overlaps[18] = Integer.parseInt(attributes.getValue("BE").toString());
						vennXML.overlaps[19] = Integer.parseInt(attributes.getValue("ABE").toString());
						vennXML.overlaps[20] = Integer.parseInt(attributes.getValue("CE").toString());
						vennXML.overlaps[21] = Integer.parseInt(attributes.getValue("ACE").toString());
						vennXML.overlaps[22] = Integer.parseInt(attributes.getValue("BCE").toString());
						vennXML.overlaps[23] = Integer.parseInt(attributes.getValue("ABCE").toString());
						vennXML.overlaps[24] = Integer.parseInt(attributes.getValue("DE").toString());
						vennXML.overlaps[25] = Integer.parseInt(attributes.getValue("ADE").toString());
						vennXML.overlaps[26] = Integer.parseInt(attributes.getValue("BDE").toString());
						vennXML.overlaps[27] = Integer.parseInt(attributes.getValue("ABDE").toString());
						vennXML.overlaps[28] = Integer.parseInt(attributes.getValue("CDE").toString());
						vennXML.overlaps[29] = Integer.parseInt(attributes.getValue("ACDE").toString());
						vennXML.overlaps[30] = Integer.parseInt(attributes.getValue("BCDE").toString());
						vennXML.overlaps[31] = Integer.parseInt(attributes.getValue("ABCDE").toString());
					}

				}
				public void endElement(String uri, String localName, String qName) throws SAXException {
					int a=0;
 				}
				public void characters(char ch[], int start, int length) throws SAXException {
					if (hA) {
						data =  new String(ch, start, length);
						vennXML.HeaderA = data;
						hA = false;
					}
					if (hB) {
						data =  new String(ch, start, length);
						vennXML.HeaderB = data;
						hB = false;
					}
					if (hC) {
						data =  new String(ch, start, length);
						vennXML.HeaderC = data;
						hC = false;
					}
					if (hD) {
						data =  new String(ch, start, length);
						vennXML.HeaderD = data;
						hD = false;
					}
					if (hE) {
						data =  new String(ch, start, length);
						vennXML.HeaderE = data;
						hE = false;
					}
					if (dsA) {
						data =  new String(ch, start, length);
						vennXML.datasetA = data;
						dsA = false;
					}
					if (dsB) {
						data =  new String(ch, start, length);
						vennXML.datasetB = data;
						dsB = false;
					}
					if (dsC) {
						data =  new String(ch, start, length);
						vennXML.datasetC = data;
						dsC = false;
					}
					if (dsD) {
						data =  new String(ch, start, length);
						vennXML.datasetD = data;
						dsD = false;
					}
					if (dsE) {
						data =  new String(ch, start, length);
						vennXML.datasetE = data;
						dsE = false;
					}
				}
			};
			saxParser.parse(file, handler);
		} catch (Exception e) {
    			e.printStackTrace();
   		}
	}

	public void loadVennXML(Venn2 v2) {
		v2.vennTP.tabPane.getSelectionModel().select(0);
		v2.vennTP.colorPickerBG.setValue(Color.valueOf(vennXML.bgcolor));
		v2.vennTP.valuesColorPicker.setValue(Color.valueOf(vennXML.valuecolor));
		v2.vennTP.fontSize.setValue(vennXML.fontSize);
		v2.vennTP.fontFamily.setValue(vennXML.fontFamily);
		String[] fs = vennXML.fontStyle.split(",");
		v2.vennTP.fontWeight.setSelected(Boolean.parseBoolean(fs[0]));
		v2.vennTP.fontPosture.setSelected(Boolean.parseBoolean(fs[1]));
		v2.vennTP.fontUnderline.setSelected(Boolean.parseBoolean(fs[2]));
		v2.vennTP.fontShadow.setSelected(Boolean.parseBoolean(fs[3]));

		v2.vennTP.tabPane.getSelectionModel().select(1);
		v2.headerA.setLayoutX(vennXML.lxHeaderA);
		v2.headerA.setLayoutY(vennXML.lyHeaderA);
		v2.vennTP.titleColorPickerA.setValue(Color.valueOf(vennXML.textcolorHeaderA));
		v2.headerA.setFill(Color.valueOf(vennXML.textcolorHeaderA));
		v2.vennTP.fontSizeA.setValue(vennXML.fontSizeHeaderA);
		v2.vennTP.fontFamilyA.setValue(vennXML.fontFamilyHeaderA);
		fs = vennXML.fontStyleHeaderA.split(",");
		v2.vennTP.fontWeightA.setSelected(Boolean.parseBoolean(fs[0]));
		v2.vennTP.fontPostureA.setSelected(Boolean.parseBoolean(fs[1]));
		v2.vennTP.fontUnderlineA.setSelected(Boolean.parseBoolean(fs[2]));
		v2.vennTP.fontShadowA.setSelected(Boolean.parseBoolean(fs[3]));
		v2.vennTP.sliderTitleAngleA.setValue(vennXML.angleHeaderA);
		v2.vennTP.fillColorPickerA.setValue(Color.valueOf(vennXML.fillEllipseA));
		v2.circleA.setFill(Color.valueOf(vennXML.fillEllipseA));
		v2.vennTP.strokeColorPickerA.setValue(Color.valueOf(vennXML.strokeEllipseA));
		v2.circleA.setStroke(Color.valueOf(vennXML.strokeEllipseA));
		v2.vennTP.sliderStrokeWidthA.setValue(vennXML.widthEllipseA);
		v2.circleA.setStrokeWidth(vennXML.widthEllipseA);
		v2.vennTP.sliderSizeCircleA.setValue(vennXML.sizeEllipseA);
		v2.vennTP.datasetHeaderA.setText(vennXML.HeaderA);
		v2.headerA.setText(vennXML.HeaderA);

		v2.vennTP.tabPane.getSelectionModel().select(2);
		v2.headerB.setLayoutX(vennXML.lxHeaderB);
		v2.headerB.setLayoutY(vennXML.lyHeaderB);
		v2.vennTP.titleColorPickerB.setValue(Color.valueOf(vennXML.textcolorHeaderB));
		v2.headerB.setFill(Color.valueOf(vennXML.textcolorHeaderB));
		v2.vennTP.fontSizeB.setValue(vennXML.fontSizeHeaderB);
		v2.vennTP.fontFamilyB.setValue(vennXML.fontFamilyHeaderB);
		fs = vennXML.fontStyleHeaderB.split(",");
		v2.vennTP.fontWeightB.setSelected(Boolean.parseBoolean(fs[0]));
		v2.vennTP.fontPostureB.setSelected(Boolean.parseBoolean(fs[1]));
		v2.vennTP.fontUnderlineB.setSelected(Boolean.parseBoolean(fs[2]));
		v2.vennTP.fontShadowB.setSelected(Boolean.parseBoolean(fs[3]));
		v2.vennTP.sliderTitleAngleB.setValue(vennXML.angleHeaderB);
		v2.vennTP.fillColorPickerB.setValue(Color.valueOf(vennXML.fillEllipseB));
		v2.circleB.setFill(Color.valueOf(vennXML.fillEllipseB));
		v2.vennTP.strokeColorPickerB.setValue(Color.valueOf(vennXML.strokeEllipseB));
		v2.circleB.setStroke(Color.valueOf(vennXML.strokeEllipseB));
		v2.vennTP.sliderStrokeWidthB.setValue(vennXML.widthEllipseB);
		v2.circleB.setStrokeWidth(vennXML.widthEllipseB);
		v2.vennTP.sliderSizeCircleB.setValue(vennXML.sizeEllipseB);
		v2.vennTP.datasetHeaderB.setText(vennXML.HeaderB);
		v2.headerB.setText(vennXML.HeaderB);
		v2.vennTP.sliderOverlapCirclesAB.setValue(vennXML.overlapDist);

		v2.setBGColor();
		v2.setValuesColor();
		v2.setFont();
		v2.setFontUnderline();
		v2.setFontShadow();
		v2.setFontA();
		v2.setFontUnderlineA();
		v2.setFontShadowA();
		v2.setFontB();
		v2.setFontUnderlineB();
		v2.setFontShadowB();
		v2.setEllipseSize();

		v2.vCalc.vennType = vennXML.vennType;
		v2.vCalc.headerA = vennXML.HeaderA;
		v2.vCalc.headerB = vennXML.HeaderB;
		v2.vCalc.setA(vennXML.datasetA.replace("|~|", "\n"));
		v2.vCalc.setB(vennXML.datasetB.replace("|~|", "\n"));
		v2.vCalc.countVenn();
		v2.vCalc.vennOverlap = vennXML.overlaps;
		int[] overlap = v2.vCalc.getOverlap();
		for (int i=1; i<4; i++) {
			v2.vennValue[i].setText(String.valueOf(overlap[i]));
		}
	}

	public void loadVennXML(Venn3 v3) {
		v3.vennTP.tabPane.getSelectionModel().select(0);
		v3.vennTP.colorPickerBG.setValue(Color.valueOf(vennXML.bgcolor));
		v3.vennTP.valuesColorPicker.setValue(Color.valueOf(vennXML.valuecolor));
		v3.vennTP.fontSize.setValue(vennXML.fontSize);
		v3.vennTP.fontFamily.setValue(vennXML.fontFamily);
		String[] fs = vennXML.fontStyle.split(",");
		v3.vennTP.fontWeight.setSelected(Boolean.parseBoolean(fs[0]));
		v3.vennTP.fontPosture.setSelected(Boolean.parseBoolean(fs[1]));
		v3.vennTP.fontUnderline.setSelected(Boolean.parseBoolean(fs[2]));
		v3.vennTP.fontShadow.setSelected(Boolean.parseBoolean(fs[3]));

		v3.vennTP.tabPane.getSelectionModel().select(1);
		v3.headerA.setLayoutX(vennXML.lxHeaderA);
		v3.headerA.setLayoutY(vennXML.lyHeaderA);
		v3.vennTP.titleColorPickerA.setValue(Color.valueOf(vennXML.textcolorHeaderA));
		v3.headerA.setFill(Color.valueOf(vennXML.textcolorHeaderA));
		v3.vennTP.fontSizeA.setValue(vennXML.fontSizeHeaderA);
		v3.vennTP.fontFamilyA.setValue(vennXML.fontFamilyHeaderA);
		fs = vennXML.fontStyleHeaderA.split(",");
		v3.vennTP.fontWeightA.setSelected(Boolean.parseBoolean(fs[0]));
		v3.vennTP.fontPostureA.setSelected(Boolean.parseBoolean(fs[1]));
		v3.vennTP.fontUnderlineA.setSelected(Boolean.parseBoolean(fs[2]));
		v3.vennTP.fontShadowA.setSelected(Boolean.parseBoolean(fs[3]));
		v3.vennTP.sliderTitleAngleA.setValue(vennXML.angleHeaderA);
		v3.vennTP.fillColorPickerA.setValue(Color.valueOf(vennXML.fillEllipseA));
		v3.circleA.setFill(Color.valueOf(vennXML.fillEllipseA));
		v3.vennTP.strokeColorPickerA.setValue(Color.valueOf(vennXML.strokeEllipseA));
		v3.circleA.setStroke(Color.valueOf(vennXML.strokeEllipseA));
		v3.vennTP.sliderStrokeWidthA.setValue(vennXML.widthEllipseA);
		v3.circleA.setStrokeWidth(vennXML.widthEllipseA);
		v3.vennTP.sliderSizeCircleA.setValue(vennXML.sizeEllipseA);
		v3.vennTP.datasetHeaderA.setText(vennXML.HeaderA);
		v3.headerA.setText(vennXML.HeaderA);

		v3.vennTP.tabPane.getSelectionModel().select(2);
		v3.headerB.setLayoutX(vennXML.lxHeaderB);
		v3.headerB.setLayoutY(vennXML.lyHeaderB);
		v3.vennTP.titleColorPickerB.setValue(Color.valueOf(vennXML.textcolorHeaderB));
		v3.headerB.setFill(Color.valueOf(vennXML.textcolorHeaderB));
		v3.vennTP.fontSizeB.setValue(vennXML.fontSizeHeaderB);
		v3.vennTP.fontFamilyB.setValue(vennXML.fontFamilyHeaderB);
		fs = vennXML.fontStyleHeaderB.split(",");
		v3.vennTP.fontWeightB.setSelected(Boolean.parseBoolean(fs[0]));
		v3.vennTP.fontPostureB.setSelected(Boolean.parseBoolean(fs[1]));
		v3.vennTP.fontUnderlineB.setSelected(Boolean.parseBoolean(fs[2]));
		v3.vennTP.fontShadowB.setSelected(Boolean.parseBoolean(fs[3]));
		v3.vennTP.sliderTitleAngleB.setValue(vennXML.angleHeaderB);
		v3.vennTP.fillColorPickerB.setValue(Color.valueOf(vennXML.fillEllipseB));
		v3.circleB.setFill(Color.valueOf(vennXML.fillEllipseB));
		v3.vennTP.strokeColorPickerB.setValue(Color.valueOf(vennXML.strokeEllipseB));
		v3.circleB.setStroke(Color.valueOf(vennXML.strokeEllipseB));
		v3.vennTP.sliderStrokeWidthB.setValue(vennXML.widthEllipseB);
		v3.circleB.setStrokeWidth(vennXML.widthEllipseB);
		v3.vennTP.sliderSizeCircleB.setValue(vennXML.sizeEllipseB);
		v3.vennTP.datasetHeaderB.setText(vennXML.HeaderB);
		v3.headerB.setText(vennXML.HeaderB);

		v3.vennTP.tabPane.getSelectionModel().select(3);
		v3.headerC.setLayoutX(vennXML.lxHeaderC);
		v3.headerC.setLayoutY(vennXML.lyHeaderC);
		v3.vennTP.titleColorPickerC.setValue(Color.valueOf(vennXML.textcolorHeaderC));
		v3.headerC.setFill(Color.valueOf(vennXML.textcolorHeaderC));
		v3.vennTP.fontSizeC.setValue(vennXML.fontSizeHeaderC);
		v3.vennTP.fontFamilyC.setValue(vennXML.fontFamilyHeaderC);
		fs = vennXML.fontStyleHeaderC.split(",");
		v3.vennTP.fontWeightC.setSelected(Boolean.parseBoolean(fs[0]));
		v3.vennTP.fontPostureC.setSelected(Boolean.parseBoolean(fs[1]));
		v3.vennTP.fontUnderlineC.setSelected(Boolean.parseBoolean(fs[2]));
		v3.vennTP.fontShadowC.setSelected(Boolean.parseBoolean(fs[3]));
		v3.vennTP.sliderTitleAngleC.setValue(vennXML.angleHeaderC);
		v3.vennTP.fillColorPickerC.setValue(Color.valueOf(vennXML.fillEllipseC));
		v3.circleC.setFill(Color.valueOf(vennXML.fillEllipseC));
		v3.vennTP.strokeColorPickerC.setValue(Color.valueOf(vennXML.strokeEllipseC));
		v3.circleC.setStroke(Color.valueOf(vennXML.strokeEllipseC));
		v3.vennTP.sliderStrokeWidthC.setValue(vennXML.widthEllipseC);
		v3.circleC.setStrokeWidth(vennXML.widthEllipseC);
		v3.vennTP.sliderSizeCircleC.setValue(vennXML.sizeEllipseC);
		v3.vennTP.datasetHeaderC.setText(vennXML.HeaderC);
		v3.headerC.setText(vennXML.HeaderC);

		v3.setBGColor();
		v3.setValuesColor();
		v3.setFont();
		v3.setFontUnderline();
		v3.setFontShadow();
		v3.setFontA();
		v3.setFontUnderlineA();
		v3.setFontShadowA();
		v3.setFontB();
		v3.setFontUnderlineB();
		v3.setFontShadowB();
		v3.setFontC();
		v3.setFontUnderlineC();
		v3.setFontShadowC();
		v3.setEllipseSize();

		v3.vCalc.vennType = vennXML.vennType;
		v3.vCalc.headerA = vennXML.HeaderA;
		v3.vCalc.headerB = vennXML.HeaderB;
		v3.vCalc.headerC = vennXML.HeaderC;
		v3.vCalc.setA(vennXML.datasetA.replace("|~|", "\n"));
		v3.vCalc.setB(vennXML.datasetB.replace("|~|", "\n"));
		v3.vCalc.setC(vennXML.datasetC.replace("|~|", "\n"));
		v3.vCalc.countVenn();
		v3.vCalc.vennOverlap = vennXML.overlaps;
		int[] overlap = v3.vCalc.getOverlap();
		for (int i=1; i<8; i++) {
			v3.vennValue[i].setText(String.valueOf(overlap[i]));
		}
	}

	public void loadVennXML(Venn4 v4) {
		v4.vennTP.tabPane.getSelectionModel().select(0);
		v4.vennTP.colorPickerBG.setValue(Color.valueOf(vennXML.bgcolor));
		v4.vennTP.valuesColorPicker.setValue(Color.valueOf(vennXML.valuecolor));
		v4.vennTP.fontSize.setValue(vennXML.fontSize);
		v4.vennTP.fontFamily.setValue(vennXML.fontFamily);
		String[] fs = vennXML.fontStyle.split(",");
		v4.vennTP.fontWeight.setSelected(Boolean.parseBoolean(fs[0]));
		v4.vennTP.fontPosture.setSelected(Boolean.parseBoolean(fs[1]));
		v4.vennTP.fontUnderline.setSelected(Boolean.parseBoolean(fs[2]));
		v4.vennTP.fontShadow.setSelected(Boolean.parseBoolean(fs[3]));

		v4.vennTP.tabPane.getSelectionModel().select(1);
		v4.headerA.setLayoutX(vennXML.lxHeaderA);
		v4.headerA.setLayoutY(vennXML.lyHeaderA);
		v4.vennTP.titleColorPickerA.setValue(Color.valueOf(vennXML.textcolorHeaderA));
		v4.headerA.setFill(Color.valueOf(vennXML.textcolorHeaderA));
		v4.vennTP.fontSizeA.setValue(vennXML.fontSizeHeaderA);
		v4.vennTP.fontFamilyA.setValue(vennXML.fontFamilyHeaderA);
		fs = vennXML.fontStyleHeaderA.split(",");
		v4.vennTP.fontWeightA.setSelected(Boolean.parseBoolean(fs[0]));
		v4.vennTP.fontPostureA.setSelected(Boolean.parseBoolean(fs[1]));
		v4.vennTP.fontUnderlineA.setSelected(Boolean.parseBoolean(fs[2]));
		v4.vennTP.fontShadowA.setSelected(Boolean.parseBoolean(fs[3]));
		v4.vennTP.sliderTitleAngleA.setValue(vennXML.angleHeaderA);
		v4.vennTP.fillColorPickerA.setValue(Color.valueOf(vennXML.fillEllipseA));
		v4.circleA.setFill(Color.valueOf(vennXML.fillEllipseA));
		v4.vennTP.strokeColorPickerA.setValue(Color.valueOf(vennXML.strokeEllipseA));
		v4.circleA.setStroke(Color.valueOf(vennXML.strokeEllipseA));
		v4.vennTP.sliderStrokeWidthA.setValue(vennXML.widthEllipseA);
		v4.circleA.setStrokeWidth(vennXML.widthEllipseA);
		v4.vennTP.sliderSizeCircleA.setValue(vennXML.sizeEllipseA);
		v4.vennTP.datasetHeaderA.setText(vennXML.HeaderA);
		v4.headerA.setText(vennXML.HeaderA);

		v4.vennTP.tabPane.getSelectionModel().select(2);
		v4.headerB.setLayoutX(vennXML.lxHeaderB);
		v4.headerB.setLayoutY(vennXML.lyHeaderB);
		v4.vennTP.titleColorPickerB.setValue(Color.valueOf(vennXML.textcolorHeaderB));
		v4.headerB.setFill(Color.valueOf(vennXML.textcolorHeaderB));
		v4.vennTP.fontSizeB.setValue(vennXML.fontSizeHeaderB);
		v4.vennTP.fontFamilyB.setValue(vennXML.fontFamilyHeaderB);
		fs = vennXML.fontStyleHeaderB.split(",");
		v4.vennTP.fontWeightB.setSelected(Boolean.parseBoolean(fs[0]));
		v4.vennTP.fontPostureB.setSelected(Boolean.parseBoolean(fs[1]));
		v4.vennTP.fontUnderlineB.setSelected(Boolean.parseBoolean(fs[2]));
		v4.vennTP.fontShadowB.setSelected(Boolean.parseBoolean(fs[3]));
		v4.vennTP.sliderTitleAngleB.setValue(vennXML.angleHeaderB);
		v4.vennTP.fillColorPickerB.setValue(Color.valueOf(vennXML.fillEllipseB));
		v4.circleB.setFill(Color.valueOf(vennXML.fillEllipseB));
		v4.vennTP.strokeColorPickerB.setValue(Color.valueOf(vennXML.strokeEllipseB));
		v4.circleB.setStroke(Color.valueOf(vennXML.strokeEllipseB));
		v4.vennTP.sliderStrokeWidthB.setValue(vennXML.widthEllipseB);
		v4.circleB.setStrokeWidth(vennXML.widthEllipseB);
		v4.vennTP.sliderSizeCircleB.setValue(vennXML.sizeEllipseB);
		v4.vennTP.datasetHeaderB.setText(vennXML.HeaderB);
		v4.headerB.setText(vennXML.HeaderB);

		v4.vennTP.tabPane.getSelectionModel().select(3);
		v4.headerC.setLayoutX(vennXML.lxHeaderC);
		v4.headerC.setLayoutY(vennXML.lyHeaderC);
		v4.vennTP.titleColorPickerC.setValue(Color.valueOf(vennXML.textcolorHeaderC));
		v4.headerC.setFill(Color.valueOf(vennXML.textcolorHeaderC));
		v4.vennTP.fontSizeC.setValue(vennXML.fontSizeHeaderC);
		v4.vennTP.fontFamilyC.setValue(vennXML.fontFamilyHeaderC);
		fs = vennXML.fontStyleHeaderC.split(",");
		v4.vennTP.fontWeightC.setSelected(Boolean.parseBoolean(fs[0]));
		v4.vennTP.fontPostureC.setSelected(Boolean.parseBoolean(fs[1]));
		v4.vennTP.fontUnderlineC.setSelected(Boolean.parseBoolean(fs[2]));
		v4.vennTP.fontShadowC.setSelected(Boolean.parseBoolean(fs[3]));
		v4.vennTP.sliderTitleAngleC.setValue(vennXML.angleHeaderC);
		v4.vennTP.fillColorPickerC.setValue(Color.valueOf(vennXML.fillEllipseC));
		v4.circleC.setFill(Color.valueOf(vennXML.fillEllipseC));
		v4.vennTP.strokeColorPickerC.setValue(Color.valueOf(vennXML.strokeEllipseC));
		v4.circleC.setStroke(Color.valueOf(vennXML.strokeEllipseC));
		v4.vennTP.sliderStrokeWidthC.setValue(vennXML.widthEllipseC);
		v4.circleC.setStrokeWidth(vennXML.widthEllipseC);
		v4.vennTP.sliderSizeCircleC.setValue(vennXML.sizeEllipseC);
		v4.vennTP.datasetHeaderC.setText(vennXML.HeaderC);
		v4.headerC.setText(vennXML.HeaderC);

		v4.vennTP.tabPane.getSelectionModel().select(4);
		v4.headerD.setLayoutX(vennXML.lxHeaderD);
		v4.headerD.setLayoutY(vennXML.lyHeaderD);
		v4.vennTP.titleColorPickerD.setValue(Color.valueOf(vennXML.textcolorHeaderD));
		v4.headerD.setFill(Color.valueOf(vennXML.textcolorHeaderD));
		v4.vennTP.fontSizeD.setValue(vennXML.fontSizeHeaderD);
		v4.vennTP.fontFamilyD.setValue(vennXML.fontFamilyHeaderD);
		fs = vennXML.fontStyleHeaderD.split(",");
		v4.vennTP.fontWeightD.setSelected(Boolean.parseBoolean(fs[0]));
		v4.vennTP.fontPostureD.setSelected(Boolean.parseBoolean(fs[1]));
		v4.vennTP.fontUnderlineD.setSelected(Boolean.parseBoolean(fs[2]));
		v4.vennTP.fontShadowD.setSelected(Boolean.parseBoolean(fs[3]));
		v4.vennTP.sliderTitleAngleD.setValue(vennXML.angleHeaderD);
		v4.vennTP.fillColorPickerD.setValue(Color.valueOf(vennXML.fillEllipseD));
		v4.circleD.setFill(Color.valueOf(vennXML.fillEllipseD));
		v4.vennTP.strokeColorPickerD.setValue(Color.valueOf(vennXML.strokeEllipseD));
		v4.circleD.setStroke(Color.valueOf(vennXML.strokeEllipseD));
		v4.vennTP.sliderStrokeWidthD.setValue(vennXML.widthEllipseD);
		v4.circleD.setStrokeWidth(vennXML.widthEllipseD);
		v4.vennTP.sliderSizeCircleD.setValue(vennXML.sizeEllipseD);
		v4.vennTP.datasetHeaderD.setText(vennXML.HeaderD);
		v4.headerD.setText(vennXML.HeaderD);

		v4.setBGColor();
		v4.setValuesColor();
		v4.setFont();
		v4.setFontUnderline();
		v4.setFontShadow();
		v4.setFontA();
		v4.setFontUnderlineA();
		v4.setFontShadowA();
		v4.setFontB();
		v4.setFontUnderlineB();
		v4.setFontShadowB();
		v4.setFontC();
		v4.setFontUnderlineC();
		v4.setFontShadowC();
		v4.setFontD();
		v4.setFontUnderlineD();
		v4.setFontShadowD();

		v4.vCalc.vennType = vennXML.vennType;
		v4.vCalc.headerA = vennXML.HeaderA;
		v4.vCalc.headerB = vennXML.HeaderB;
		v4.vCalc.headerC = vennXML.HeaderC;
		v4.vCalc.headerD = vennXML.HeaderD;
		v4.vCalc.setA(vennXML.datasetA.replace("|~|", "\n"));
		v4.vCalc.setB(vennXML.datasetB.replace("|~|", "\n"));
		v4.vCalc.setC(vennXML.datasetC.replace("|~|", "\n"));
		v4.vCalc.setD(vennXML.datasetD.replace("|~|", "\n"));
		v4.vCalc.countVenn();
		v4.vCalc.vennOverlap = vennXML.overlaps;
		int[] overlap = v4.vCalc.getOverlap();
		for (int i=1; i<16; i++) {
			v4.vennValue[i].setText(String.valueOf(overlap[i]));
		}
	}

	public void loadVennXML(Venn5 v5) {
		v5.vennTP.tabPane.getSelectionModel().select(0);
		v5.vennTP.colorPickerBG.setValue(Color.valueOf(vennXML.bgcolor));
		v5.vennTP.valuesColorPicker.setValue(Color.valueOf(vennXML.valuecolor));
		v5.vennTP.fontSize.setValue(vennXML.fontSize);
		v5.vennTP.fontFamily.setValue(vennXML.fontFamily);
		String[] fs = vennXML.fontStyle.split(",");
		v5.vennTP.fontWeight.setSelected(Boolean.parseBoolean(fs[0]));
		v5.vennTP.fontPosture.setSelected(Boolean.parseBoolean(fs[1]));
		v5.vennTP.fontUnderline.setSelected(Boolean.parseBoolean(fs[2]));
		v5.vennTP.fontShadow.setSelected(Boolean.parseBoolean(fs[3]));

		v5.vennTP.tabPane.getSelectionModel().select(1);
		v5.headerA.setLayoutX(vennXML.lxHeaderA);
		v5.headerA.setLayoutY(vennXML.lyHeaderA);
		v5.vennTP.titleColorPickerA.setValue(Color.valueOf(vennXML.textcolorHeaderA));
		v5.headerA.setFill(Color.valueOf(vennXML.textcolorHeaderA));
		v5.vennTP.fontSizeA.setValue(vennXML.fontSizeHeaderA);
		v5.vennTP.fontFamilyA.setValue(vennXML.fontFamilyHeaderA);
		fs = vennXML.fontStyleHeaderA.split(",");
		v5.vennTP.fontWeightA.setSelected(Boolean.parseBoolean(fs[0]));
		v5.vennTP.fontPostureA.setSelected(Boolean.parseBoolean(fs[1]));
		v5.vennTP.fontUnderlineA.setSelected(Boolean.parseBoolean(fs[2]));
		v5.vennTP.fontShadowA.setSelected(Boolean.parseBoolean(fs[3]));
		v5.vennTP.sliderTitleAngleA.setValue(vennXML.angleHeaderA);
		v5.vennTP.fillColorPickerA.setValue(Color.valueOf(vennXML.fillEllipseA));
		v5.circleA.setFill(Color.valueOf(vennXML.fillEllipseA));
		v5.vennTP.strokeColorPickerA.setValue(Color.valueOf(vennXML.strokeEllipseA));
		v5.circleA.setStroke(Color.valueOf(vennXML.strokeEllipseA));
		v5.vennTP.sliderStrokeWidthA.setValue(vennXML.widthEllipseA);
		v5.circleA.setStrokeWidth(vennXML.widthEllipseA);
		v5.vennTP.sliderSizeCircleA.setValue(vennXML.sizeEllipseA);
		v5.vennTP.datasetHeaderA.setText(vennXML.HeaderA);
		v5.headerA.setText(vennXML.HeaderA);

		v5.vennTP.tabPane.getSelectionModel().select(2);
		v5.headerB.setLayoutX(vennXML.lxHeaderB);
		v5.headerB.setLayoutY(vennXML.lyHeaderB);
		v5.vennTP.titleColorPickerB.setValue(Color.valueOf(vennXML.textcolorHeaderB));
		v5.headerB.setFill(Color.valueOf(vennXML.textcolorHeaderB));
		v5.vennTP.fontSizeB.setValue(vennXML.fontSizeHeaderB);
		v5.vennTP.fontFamilyB.setValue(vennXML.fontFamilyHeaderB);
		fs = vennXML.fontStyleHeaderB.split(",");
		v5.vennTP.fontWeightB.setSelected(Boolean.parseBoolean(fs[0]));
		v5.vennTP.fontPostureB.setSelected(Boolean.parseBoolean(fs[1]));
		v5.vennTP.fontUnderlineB.setSelected(Boolean.parseBoolean(fs[2]));
		v5.vennTP.fontShadowB.setSelected(Boolean.parseBoolean(fs[3]));
		v5.vennTP.sliderTitleAngleB.setValue(vennXML.angleHeaderB);
		v5.vennTP.fillColorPickerB.setValue(Color.valueOf(vennXML.fillEllipseB));
		v5.circleB.setFill(Color.valueOf(vennXML.fillEllipseB));
		v5.vennTP.strokeColorPickerB.setValue(Color.valueOf(vennXML.strokeEllipseB));
		v5.circleB.setStroke(Color.valueOf(vennXML.strokeEllipseB));
		v5.vennTP.sliderStrokeWidthB.setValue(vennXML.widthEllipseB);
		v5.circleB.setStrokeWidth(vennXML.widthEllipseB);
		v5.vennTP.sliderSizeCircleB.setValue(vennXML.sizeEllipseB);
		v5.vennTP.datasetHeaderB.setText(vennXML.HeaderB);
		v5.headerB.setText(vennXML.HeaderB);

		v5.vennTP.tabPane.getSelectionModel().select(3);
		v5.headerC.setLayoutX(vennXML.lxHeaderC);
		v5.headerC.setLayoutY(vennXML.lyHeaderC);
		v5.vennTP.titleColorPickerC.setValue(Color.valueOf(vennXML.textcolorHeaderC));
		v5.headerC.setFill(Color.valueOf(vennXML.textcolorHeaderC));
		v5.vennTP.fontSizeC.setValue(vennXML.fontSizeHeaderC);
		v5.vennTP.fontFamilyC.setValue(vennXML.fontFamilyHeaderC);
		fs = vennXML.fontStyleHeaderC.split(",");
		v5.vennTP.fontWeightC.setSelected(Boolean.parseBoolean(fs[0]));
		v5.vennTP.fontPostureC.setSelected(Boolean.parseBoolean(fs[1]));
		v5.vennTP.fontUnderlineC.setSelected(Boolean.parseBoolean(fs[2]));
		v5.vennTP.fontShadowC.setSelected(Boolean.parseBoolean(fs[3]));
		v5.vennTP.sliderTitleAngleC.setValue(vennXML.angleHeaderC);
		v5.vennTP.fillColorPickerC.setValue(Color.valueOf(vennXML.fillEllipseC));
		v5.circleC.setFill(Color.valueOf(vennXML.fillEllipseC));
		v5.vennTP.strokeColorPickerC.setValue(Color.valueOf(vennXML.strokeEllipseC));
		v5.circleC.setStroke(Color.valueOf(vennXML.strokeEllipseC));
		v5.vennTP.sliderStrokeWidthC.setValue(vennXML.widthEllipseC);
		v5.circleC.setStrokeWidth(vennXML.widthEllipseC);
		v5.vennTP.sliderSizeCircleC.setValue(vennXML.sizeEllipseC);
		v5.vennTP.datasetHeaderC.setText(vennXML.HeaderC);
		v5.headerC.setText(vennXML.HeaderC);

		v5.vennTP.tabPane.getSelectionModel().select(4);
		v5.headerD.setLayoutX(vennXML.lxHeaderD);
		v5.headerD.setLayoutY(vennXML.lyHeaderD);
		v5.vennTP.titleColorPickerD.setValue(Color.valueOf(vennXML.textcolorHeaderD));
		v5.headerD.setFill(Color.valueOf(vennXML.textcolorHeaderD));
		v5.vennTP.fontSizeD.setValue(vennXML.fontSizeHeaderD);
		v5.vennTP.fontFamilyD.setValue(vennXML.fontFamilyHeaderD);
		fs = vennXML.fontStyleHeaderD.split(",");
		v5.vennTP.fontWeightD.setSelected(Boolean.parseBoolean(fs[0]));
		v5.vennTP.fontPostureD.setSelected(Boolean.parseBoolean(fs[1]));
		v5.vennTP.fontUnderlineD.setSelected(Boolean.parseBoolean(fs[2]));
		v5.vennTP.fontShadowD.setSelected(Boolean.parseBoolean(fs[3]));
		v5.vennTP.sliderTitleAngleD.setValue(vennXML.angleHeaderD);
		v5.vennTP.fillColorPickerD.setValue(Color.valueOf(vennXML.fillEllipseD));
		v5.circleD.setFill(Color.valueOf(vennXML.fillEllipseD));
		v5.vennTP.strokeColorPickerD.setValue(Color.valueOf(vennXML.strokeEllipseD));
		v5.circleD.setStroke(Color.valueOf(vennXML.strokeEllipseD));
		v5.vennTP.sliderStrokeWidthD.setValue(vennXML.widthEllipseD);
		v5.circleD.setStrokeWidth(vennXML.widthEllipseD);
		v5.vennTP.sliderSizeCircleD.setValue(vennXML.sizeEllipseD);
		v5.vennTP.datasetHeaderD.setText(vennXML.HeaderD);
		v5.headerD.setText(vennXML.HeaderD);

		v5.vennTP.tabPane.getSelectionModel().select(5);
		v5.headerE.setLayoutX(vennXML.lxHeaderE);
		v5.headerE.setLayoutY(vennXML.lyHeaderE);
		v5.vennTP.titleColorPickerE.setValue(Color.valueOf(vennXML.textcolorHeaderE));
		v5.headerE.setFill(Color.valueOf(vennXML.textcolorHeaderE));
		v5.vennTP.fontSizeE.setValue(vennXML.fontSizeHeaderE);
		v5.vennTP.fontFamilyE.setValue(vennXML.fontFamilyHeaderE);
		fs = vennXML.fontStyleHeaderE.split(",");
		v5.vennTP.fontWeightE.setSelected(Boolean.parseBoolean(fs[0]));
		v5.vennTP.fontPostureE.setSelected(Boolean.parseBoolean(fs[1]));
		v5.vennTP.fontUnderlineE.setSelected(Boolean.parseBoolean(fs[2]));
		v5.vennTP.fontShadowE.setSelected(Boolean.parseBoolean(fs[3]));
		v5.vennTP.sliderTitleAngleE.setValue(vennXML.angleHeaderE);
		v5.vennTP.fillColorPickerE.setValue(Color.valueOf(vennXML.fillEllipseE));
		v5.circleE.setFill(Color.valueOf(vennXML.fillEllipseE));
		v5.vennTP.strokeColorPickerE.setValue(Color.valueOf(vennXML.strokeEllipseE));
		v5.circleE.setStroke(Color.valueOf(vennXML.strokeEllipseE));
		v5.vennTP.sliderStrokeWidthE.setValue(vennXML.widthEllipseE);
		v5.circleE.setStrokeWidth(vennXML.widthEllipseE);
		v5.vennTP.sliderSizeCircleE.setValue(vennXML.sizeEllipseE);
		v5.vennTP.datasetHeaderE.setText(vennXML.HeaderE);
		v5.headerE.setText(vennXML.HeaderE);

		v5.setBGColor();
		v5.setValuesColor();
		v5.setFont();
		v5.setFontUnderline();
		v5.setFontShadow();
		v5.setFontA();
		v5.setFontUnderlineA();
		v5.setFontShadowA();
		v5.setFontB();
		v5.setFontUnderlineB();
		v5.setFontShadowB();
		v5.setFontC();
		v5.setFontUnderlineC();
		v5.setFontShadowC();
		v5.setFontD();
		v5.setFontUnderlineD();
		v5.setFontShadowD();
		v5.setFontE();
		v5.setFontUnderlineE();
		v5.setFontShadowE();

		v5.vCalc.vennType = vennXML.vennType;
		v5.vCalc.headerA = vennXML.HeaderA;
		v5.vCalc.headerB = vennXML.HeaderB;
		v5.vCalc.headerC = vennXML.HeaderC;
		v5.vCalc.headerD = vennXML.HeaderD;
		v5.vCalc.headerE = vennXML.HeaderE;
		v5.vCalc.setA(vennXML.datasetA.replace("|~|", "\n"));
		v5.vCalc.setB(vennXML.datasetB.replace("|~|", "\n"));
		v5.vCalc.setC(vennXML.datasetC.replace("|~|", "\n"));
		v5.vCalc.setD(vennXML.datasetD.replace("|~|", "\n"));
		v5.vCalc.setE(vennXML.datasetE.replace("|~|", "\n"));
		v5.vCalc.countVenn();
		v5.vCalc.vennOverlap = vennXML.overlaps;
		int[] overlap = v5.vCalc.getOverlap();
		for (int i=1; i<32; i++) {
			v5.vennValue[i].setText(String.valueOf(overlap[i]));
		}
	}

	public void loadVennXML(Euler1 e1) {
		e1.vennTP.tabPane.getSelectionModel().select(0);
		e1.vennTP.colorPickerBG.setValue(Color.valueOf(vennXML.bgcolor));
		e1.vennTP.valuesColorPicker.setValue(Color.valueOf(vennXML.valuecolor));
		e1.vennTP.fontSize.setValue(vennXML.fontSize);
		e1.vennTP.fontFamily.setValue(vennXML.fontFamily);
		String[] fs = vennXML.fontStyle.split(",");
		e1.vennTP.fontWeight.setSelected(Boolean.parseBoolean(fs[0]));
		e1.vennTP.fontPosture.setSelected(Boolean.parseBoolean(fs[1]));
		e1.vennTP.fontUnderline.setSelected(Boolean.parseBoolean(fs[2]));
		e1.vennTP.fontShadow.setSelected(Boolean.parseBoolean(fs[3]));

		e1.vennTP.tabPane.getSelectionModel().select(1);
		e1.vennTP.titleColorPickerA.setValue(Color.valueOf(vennXML.textcolorHeaderA));
		e1.vennTP.fontSizeA.setValue(vennXML.fontSizeHeaderA);
		e1.vennTP.fontFamilyA.setValue(vennXML.fontFamilyHeaderA);
		fs = vennXML.fontStyleHeaderA.split(",");
		e1.vennTP.fontWeightA.setSelected(Boolean.parseBoolean(fs[0]));
		e1.vennTP.fontPostureA.setSelected(Boolean.parseBoolean(fs[1]));
		e1.vennTP.fontUnderlineA.setSelected(Boolean.parseBoolean(fs[2]));
		e1.vennTP.fontShadowA.setSelected(Boolean.parseBoolean(fs[3]));
		e1.vennTP.sliderTitleAngleA.setValue(vennXML.angleHeaderA);
		e1.vennTP.fillColorPickerA.setValue(Color.valueOf(vennXML.fillEllipseA));
		e1.vennTP.strokeColorPickerA.setValue(Color.valueOf(vennXML.strokeEllipseA));
		e1.vennTP.sliderStrokeWidthA.setValue(vennXML.widthEllipseA);
		e1.vennTP.sliderSizeCircleA.setValue(vennXML.sizeEllipseA);
		e1.vennTP.datasetHeaderA.setText(vennXML.HeaderA);

		e1.headerA.setLayoutX(vennXML.lxHeaderA);
		e1.headerA.setLayoutY(vennXML.lyHeaderA);
		e1.headerA.setFill(Color.valueOf(vennXML.textcolorHeaderA));
		e1.circleA.setFill(Color.valueOf(vennXML.fillEllipseA));
		e1.circleA.setStroke(Color.valueOf(vennXML.strokeEllipseA));
		e1.circleA.setStrokeWidth(vennXML.widthEllipseA);
		e1.headerA.setText(vennXML.HeaderA);

		e1.setBGColor();
		e1.setValuesColor();
		e1.setFont();
		e1.setFontUnderline();
		e1.setFontShadow();
		e1.setFontA();
		e1.setFontUnderlineA();
		e1.setFontShadowA();

		e1.vCalc.vennType = vennXML.vennType;
		e1.vCalc.headerA = vennXML.HeaderA;
		e1.vCalc.setA(vennXML.datasetA.replace("|~|", "\n"));
		e1.vCalc.countVenn();
		e1.vCalc.vennOverlap = vennXML.overlaps;
		int[] overlap = e1.vCalc.getOverlap();
		e1.vennValue[1].setText(String.valueOf(overlap[1]));
	}

	public void loadVennXML(Euler2 e2) {
		e2.vennTP.tabPane.getSelectionModel().select(0);
		e2.vennTP.colorPickerBG.setValue(Color.valueOf(vennXML.bgcolor));
		e2.vennTP.valuesColorPicker.setValue(Color.valueOf(vennXML.valuecolor));
		e2.vennTP.fontSize.setValue(vennXML.fontSize);
		e2.vennTP.fontFamily.setValue(vennXML.fontFamily);
		String[] fs = vennXML.fontStyle.split(",");
		e2.vennTP.fontWeight.setSelected(Boolean.parseBoolean(fs[0]));
		e2.vennTP.fontPosture.setSelected(Boolean.parseBoolean(fs[1]));
		e2.vennTP.fontUnderline.setSelected(Boolean.parseBoolean(fs[2]));
		e2.vennTP.fontShadow.setSelected(Boolean.parseBoolean(fs[3]));
		e2.vennTP.tabPane.getSelectionModel().select(1);
		e2.vennTP.titleColorPickerA.setValue(Color.valueOf(vennXML.textcolorHeaderA));
		e2.vennTP.fontSizeA.setValue(vennXML.fontSizeHeaderA);
		e2.vennTP.fontFamilyA.setValue(vennXML.fontFamilyHeaderA);
		fs = vennXML.fontStyleHeaderA.split(",");
		e2.vennTP.fontWeightA.setSelected(Boolean.parseBoolean(fs[0]));
		e2.vennTP.fontPostureA.setSelected(Boolean.parseBoolean(fs[1]));
		e2.vennTP.fontUnderlineA.setSelected(Boolean.parseBoolean(fs[2]));
		e2.vennTP.fontShadowA.setSelected(Boolean.parseBoolean(fs[3]));
		e2.vennTP.sliderTitleAngleA.setValue(vennXML.angleHeaderA);
		e2.vennTP.fillColorPickerA.setValue(Color.valueOf(vennXML.fillEllipseA));
		e2.vennTP.strokeColorPickerA.setValue(Color.valueOf(vennXML.strokeEllipseA));
		e2.vennTP.sliderStrokeWidthA.setValue(vennXML.widthEllipseA);
		e2.vennTP.sliderSizeCircleA.setValue(vennXML.sizeEllipseA);
		e2.vennTP.datasetHeaderA.setText(vennXML.HeaderA);
		e2.headerA.setLayoutX(vennXML.lxHeaderA);
		e2.headerA.setLayoutY(vennXML.lyHeaderA);
		e2.headerA.setFill(Color.valueOf(vennXML.textcolorHeaderA));
		e2.circleA.setFill(Color.valueOf(vennXML.fillEllipseA));
		e2.circleA.setStroke(Color.valueOf(vennXML.strokeEllipseA));
		e2.circleA.setStrokeWidth(vennXML.widthEllipseA);
		e2.headerA.setText(vennXML.HeaderA);


		e2.vennTP.tabPane.getSelectionModel().select(2);
		e2.vennTP.titleColorPickerB.setValue(Color.valueOf(vennXML.textcolorHeaderB));
		e2.vennTP.fontSizeB.setValue(vennXML.fontSizeHeaderB);
		e2.vennTP.fontFamilyB.setValue(vennXML.fontFamilyHeaderB);
		fs = vennXML.fontStyleHeaderB.split(",");
		e2.vennTP.fontWeightB.setSelected(Boolean.parseBoolean(fs[0]));
		e2.vennTP.fontPostureB.setSelected(Boolean.parseBoolean(fs[1]));
		e2.vennTP.fontUnderlineB.setSelected(Boolean.parseBoolean(fs[2]));
		e2.vennTP.fontShadowB.setSelected(Boolean.parseBoolean(fs[3]));
		e2.vennTP.sliderTitleAngleB.setValue(vennXML.angleHeaderB);
		e2.vennTP.fillColorPickerB.setValue(Color.valueOf(vennXML.fillEllipseB));
		e2.vennTP.strokeColorPickerB.setValue(Color.valueOf(vennXML.strokeEllipseB));
		e2.vennTP.sliderStrokeWidthB.setValue(vennXML.widthEllipseB);
		e2.vennTP.sliderSizeCircleB.setValue(vennXML.sizeEllipseB);
		e2.vennTP.datasetHeaderB.setText(vennXML.HeaderB);
		e2.headerB.setLayoutX(vennXML.lxHeaderB);
		e2.headerB.setLayoutY(vennXML.lyHeaderB);
		e2.headerB.setText(vennXML.HeaderB);
		e2.headerB.setFill(Color.valueOf(vennXML.textcolorHeaderB));
		e2.circleB.setFill(Color.valueOf(vennXML.fillEllipseB));
		e2.circleB.setStroke(Color.valueOf(vennXML.strokeEllipseB));
		e2.circleB.setStrokeWidth(vennXML.widthEllipseB);

		e2.setBGColor();
		e2.setValuesColor();
		e2.setFont();
		e2.setFontUnderline();
		e2.setFontShadow();
		e2.setFontA();
		e2.setFontUnderlineA();
		e2.setFontShadowA();
		e2.setFontB();
		e2.setFontUnderlineB();
		e2.setFontShadowB();
		//e2.setEllipseSize();

		e2.vCalc.vennType = vennXML.vennType;
		e2.vCalc.headerA = vennXML.HeaderA;
		e2.vCalc.headerB = vennXML.HeaderB;
		e2.vCalc.setA(vennXML.datasetA.replace("|~|", "\n"));
		e2.vCalc.setB(vennXML.datasetB.replace("|~|", "\n"));
		e2.vCalc.countVenn();
		e2.vCalc.vennOverlap = vennXML.overlaps;
		e2.setValues();
	}

	public void loadVennXML(Euler3 e3) {
		e3.vennTP.tabPane.getSelectionModel().select(0);
		e3.vennTP.colorPickerBG.setValue(Color.valueOf(vennXML.bgcolor));
		e3.vennTP.valuesColorPicker.setValue(Color.valueOf(vennXML.valuecolor));
		e3.vennTP.fontSize.setValue(vennXML.fontSize);
		e3.vennTP.fontFamily.setValue(vennXML.fontFamily);
		String[] fs = vennXML.fontStyle.split(",");
		e3.vennTP.fontWeight.setSelected(Boolean.parseBoolean(fs[0]));
		e3.vennTP.fontPosture.setSelected(Boolean.parseBoolean(fs[1]));
		e3.vennTP.fontUnderline.setSelected(Boolean.parseBoolean(fs[2]));
		e3.vennTP.fontShadow.setSelected(Boolean.parseBoolean(fs[3]));

		e3.vennTP.tabPane.getSelectionModel().select(1);
		e3.headerA.setLayoutX(vennXML.lxHeaderA);
		e3.headerA.setLayoutY(vennXML.lyHeaderA);
		e3.vennTP.titleColorPickerA.setValue(Color.valueOf(vennXML.textcolorHeaderA));
		e3.headerA.setFill(Color.valueOf(vennXML.textcolorHeaderA));
		e3.vennTP.fontSizeA.setValue(vennXML.fontSizeHeaderA);
		e3.vennTP.fontFamilyA.setValue(vennXML.fontFamilyHeaderA);
		fs = vennXML.fontStyleHeaderA.split(",");
		e3.vennTP.fontWeightA.setSelected(Boolean.parseBoolean(fs[0]));
		e3.vennTP.fontPostureA.setSelected(Boolean.parseBoolean(fs[1]));
		e3.vennTP.fontUnderlineA.setSelected(Boolean.parseBoolean(fs[2]));
		e3.vennTP.fontShadowA.setSelected(Boolean.parseBoolean(fs[3]));
		e3.vennTP.sliderTitleAngleA.setValue(vennXML.angleHeaderA);
		e3.vennTP.fillColorPickerA.setValue(Color.valueOf(vennXML.fillEllipseA));
		e3.circleA.setFill(Color.valueOf(vennXML.fillEllipseA));
		e3.vennTP.strokeColorPickerA.setValue(Color.valueOf(vennXML.strokeEllipseA));
		e3.circleA.setStroke(Color.valueOf(vennXML.strokeEllipseA));
		e3.vennTP.sliderStrokeWidthA.setValue(vennXML.widthEllipseA);
		e3.circleA.setStrokeWidth(vennXML.widthEllipseA);
		e3.vennTP.sliderSizeCircleA.setValue(vennXML.sizeEllipseA);
		e3.vennTP.datasetHeaderA.setText(vennXML.HeaderA);
		e3.headerA.setText(vennXML.HeaderA);

		e3.vennTP.tabPane.getSelectionModel().select(2);
		e3.headerB.setLayoutX(vennXML.lxHeaderB);
		e3.headerB.setLayoutY(vennXML.lyHeaderB);
		e3.vennTP.titleColorPickerB.setValue(Color.valueOf(vennXML.textcolorHeaderB));
		e3.headerB.setFill(Color.valueOf(vennXML.textcolorHeaderB));
		e3.vennTP.fontSizeB.setValue(vennXML.fontSizeHeaderB);
		e3.vennTP.fontFamilyB.setValue(vennXML.fontFamilyHeaderB);
		fs = vennXML.fontStyleHeaderB.split(",");
		e3.vennTP.fontWeightB.setSelected(Boolean.parseBoolean(fs[0]));
		e3.vennTP.fontPostureB.setSelected(Boolean.parseBoolean(fs[1]));
		e3.vennTP.fontUnderlineB.setSelected(Boolean.parseBoolean(fs[2]));
		e3.vennTP.fontShadowB.setSelected(Boolean.parseBoolean(fs[3]));
		e3.vennTP.sliderTitleAngleB.setValue(vennXML.angleHeaderB);
		e3.vennTP.fillColorPickerB.setValue(Color.valueOf(vennXML.fillEllipseB));
		e3.circleB.setFill(Color.valueOf(vennXML.fillEllipseB));
		e3.vennTP.strokeColorPickerB.setValue(Color.valueOf(vennXML.strokeEllipseB));
		e3.circleB.setStroke(Color.valueOf(vennXML.strokeEllipseB));
		e3.vennTP.sliderStrokeWidthB.setValue(vennXML.widthEllipseB);
		e3.circleB.setStrokeWidth(vennXML.widthEllipseB);
		e3.vennTP.sliderSizeCircleB.setValue(vennXML.sizeEllipseB);
		e3.vennTP.datasetHeaderB.setText(vennXML.HeaderB);
		e3.headerB.setText(vennXML.HeaderB);

		e3.vennTP.tabPane.getSelectionModel().select(3);
		e3.headerC.setLayoutX(vennXML.lxHeaderC);
		e3.headerC.setLayoutY(vennXML.lyHeaderC);
		e3.vennTP.titleColorPickerC.setValue(Color.valueOf(vennXML.textcolorHeaderC));
		e3.headerC.setFill(Color.valueOf(vennXML.textcolorHeaderC));
		e3.vennTP.fontSizeC.setValue(vennXML.fontSizeHeaderC);
		e3.vennTP.fontFamilyC.setValue(vennXML.fontFamilyHeaderC);
		fs = vennXML.fontStyleHeaderC.split(",");
		e3.vennTP.fontWeightC.setSelected(Boolean.parseBoolean(fs[0]));
		e3.vennTP.fontPostureC.setSelected(Boolean.parseBoolean(fs[1]));
		e3.vennTP.fontUnderlineC.setSelected(Boolean.parseBoolean(fs[2]));
		e3.vennTP.fontShadowC.setSelected(Boolean.parseBoolean(fs[3]));
		e3.vennTP.sliderTitleAngleC.setValue(vennXML.angleHeaderC);
		e3.vennTP.fillColorPickerC.setValue(Color.valueOf(vennXML.fillEllipseC));
		e3.circleC.setFill(Color.valueOf(vennXML.fillEllipseC));
		e3.vennTP.strokeColorPickerC.setValue(Color.valueOf(vennXML.strokeEllipseC));
		e3.circleC.setStroke(Color.valueOf(vennXML.strokeEllipseC));
		e3.vennTP.sliderStrokeWidthC.setValue(vennXML.widthEllipseC);
		e3.circleC.setStrokeWidth(vennXML.widthEllipseC);
		e3.vennTP.sliderSizeCircleC.setValue(vennXML.sizeEllipseC);
		e3.vennTP.datasetHeaderC.setText(vennXML.HeaderC);
		e3.headerC.setText(vennXML.HeaderC);

		e3.setBGColor();
		e3.setValuesColor();
		e3.setFont();
		e3.setFontUnderline();
		e3.setFontShadow();
		e3.setFontA();
		e3.setFontUnderlineA();
		e3.setFontShadowA();
		e3.setFontB();
		e3.setFontUnderlineB();
		e3.setFontShadowB();
		e3.setFontC();
		e3.setFontUnderlineC();
		e3.setFontShadowC();
		//e3.setEllipseSize();

		e3.vCalc.vennType = vennXML.vennType;
		e3.vCalc.headerA = vennXML.HeaderA;
		e3.vCalc.headerB = vennXML.HeaderB;
		e3.vCalc.headerC = vennXML.HeaderC;
		e3.vCalc.setA(vennXML.datasetA.replace("|~|", "\n"));
		e3.vCalc.setB(vennXML.datasetB.replace("|~|", "\n"));
		e3.vCalc.setC(vennXML.datasetC.replace("|~|", "\n"));
		e3.vCalc.countVenn();
		e3.vCalc.vennOverlap = vennXML.overlaps;
		e3.setVennValues();
	}

	public void saveVennXML(Venn2 v2, File file) {
		try {
			ArrayList<String> dataset;
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<venn type=\""+v2.vCalc.vennType+"\">\n");
			fw.write("\t<main bgcolor=\""+v2.vennTP.colorPickerBG.getValue()+"\" valuecolor=\""+v2.vennTP.valuesColorPicker.getValue()+"\" fontSize=\""+v2.vennTP.fontSize.getValue()+"\" fontFamily=\""+v2.vennTP.fontFamily.getValue()+"\" fontStyle=\""+v2.vennTP.fontWeight.isSelected()+","+v2.vennTP.fontPosture.isSelected()+","+v2.vennTP.fontUnderline.isSelected()+","+v2.vennTP.fontShadow.isSelected()+"\"></main>\n");
			fw.write("\t<titleA lx=\""+v2.headerA.getLayoutX()+"\" ly=\""+v2.headerA.getLayoutY()+"\" angle=\""+v2.vennTP.sliderTitleAngleA.getValue()+"\" textcolor=\""+v2.vennTP.titleColorPickerA.getValue()+"\" fontSize=\""+v2.vennTP.fontSizeA.getValue()+"\" fontFamily=\""+v2.vennTP.fontFamilyA.getValue()+"\" fontStyle=\""+v2.vennTP.fontWeightA.isSelected()+","+v2.vennTP.fontPostureA.isSelected()+","+v2.vennTP.fontUnderlineA.isSelected()+","+v2.vennTP.fontShadowA.isSelected()+"\">"+v2.headerA.getText()+"</titleA>\n");
			fw.write("\t<ellipseA fill=\""+v2.circleA.getFill()+"\" stroke=\""+v2.circleA.getStroke()+"\" stroke-width=\""+v2.circleA.getStrokeWidth()+"\" size=\""+v2.vennTP.sliderSizeCircleA.getValue()+"\"> </ellipseA>\n");
			dataset =  v2.vCalc.getDatasetA();
			fw.write("\t<datasetA>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetA>\n");
			fw.write("\t<titleB lx=\""+v2.headerB.getLayoutX()+"\" ly=\""+v2.headerB.getLayoutY()+"\" angle=\""+v2.vennTP.sliderTitleAngleB.getValue()+"\" textcolor=\""+v2.vennTP.titleColorPickerB.getValue()+"\" fontSize=\""+v2.vennTP.fontSizeB.getValue()+"\" fontFamily=\""+v2.vennTP.fontFamilyB.getValue()+"\" fontStyle=\""+v2.vennTP.fontWeightB.isSelected()+","+v2.vennTP.fontPostureB.isSelected()+","+v2.vennTP.fontUnderlineB.isSelected()+","+v2.vennTP.fontShadowB.isSelected()+"\">"+v2.headerB.getText()+"</titleB>\n");
			fw.write("\t<ellipseB fill=\""+v2.circleB.getFill()+"\" stroke=\""+v2.circleB.getStroke()+"\" stroke-width=\""+v2.circleB.getStrokeWidth()+"\" size=\""+v2.vennTP.sliderSizeCircleB.getValue()+"\"> </ellipseB>\n");
			dataset =  v2.vCalc.getDatasetB();
			fw.write("\t<datasetB>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetB>\n");
			fw.write("<overlapEllipses distance=\""+v2.vennTP.sliderOverlapCirclesAB.getValue()+"\"></overlapEllipses>\n");
			fw.write("\t<overlapValues A=\""+v2.vCalc.vennOverlap[1]+"\" B=\""+v2.vCalc.vennOverlap[2]+"\" AB=\""+v2.vCalc.vennOverlap[3]+"\" C=\""+v2.vCalc.vennOverlap[4]+"\" AC=\""+v2.vCalc.vennOverlap[5]+"\" BC=\""+v2.vCalc.vennOverlap[6]+"\" ABC=\""+v2.vCalc.vennOverlap[7]+"\" D=\""+v2.vCalc.vennOverlap[8]+"\" AD=\""+v2.vCalc.vennOverlap[9]+"\" BD=\""+v2.vCalc.vennOverlap[10]+"\" ABD=\""+v2.vCalc.vennOverlap[11]+"\" CD=\""+v2.vCalc.vennOverlap[12]+"\" ACD=\""+v2.vCalc.vennOverlap[13]+"\" BCD=\""+v2.vCalc.vennOverlap[14]+"\" ABCD=\""+v2.vCalc.vennOverlap[15]+"\" E=\""+v2.vCalc.vennOverlap[16]+"\" AE=\""+v2.vCalc.vennOverlap[17]+"\" BE=\""+v2.vCalc.vennOverlap[18]+"\" ABE=\""+v2.vCalc.vennOverlap[19]+"\" CE=\""+v2.vCalc.vennOverlap[20]+"\" ACE=\""+v2.vCalc.vennOverlap[21]+"\" BCE=\""+v2.vCalc.vennOverlap[22]+"\" ABCE=\""+v2.vCalc.vennOverlap[23]+"\" DE=\""+v2.vCalc.vennOverlap[24]+"\" ADE=\""+v2.vCalc.vennOverlap[25]+"\" BDE=\""+v2.vCalc.vennOverlap[26]+"\" ABDE=\""+v2.vCalc.vennOverlap[27]+"\" CDE=\""+v2.vCalc.vennOverlap[28]+"\" ACDE=\""+v2.vCalc.vennOverlap[29]+"\" BCDE=\""+v2.vCalc.vennOverlap[30]+"\" ABCDE=\""+v2.vCalc.vennOverlap[31]+"\"> </overlapValues>\n");
			fw.write("</venn>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveVennXML(Venn3 v3, File file) {
		try {
			ArrayList<String> dataset;
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<venn type=\""+v3.vCalc.vennType+"\">\n");
			fw.write("\t<main bgcolor=\""+v3.vennTP.colorPickerBG.getValue()+"\" valuecolor=\""+v3.vennTP.valuesColorPicker.getValue()+"\" fontSize=\""+v3.vennTP.fontSize.getValue()+"\" fontFamily=\""+v3.vennTP.fontFamily.getValue()+"\" fontStyle=\""+v3.vennTP.fontWeight.isSelected()+","+v3.vennTP.fontPosture.isSelected()+","+v3.vennTP.fontUnderline.isSelected()+","+v3.vennTP.fontShadow.isSelected()+"\"></main>\n");
			fw.write("\t<titleA lx=\""+v3.headerA.getLayoutX()+"\" ly=\""+v3.headerA.getLayoutY()+"\" angle=\""+v3.vennTP.sliderTitleAngleA.getValue()+"\" textcolor=\""+v3.vennTP.titleColorPickerA.getValue()+"\" fontSize=\""+v3.vennTP.fontSizeA.getValue()+"\" fontFamily=\""+v3.vennTP.fontFamilyA.getValue()+"\" fontStyle=\""+v3.vennTP.fontWeightA.isSelected()+","+v3.vennTP.fontPostureA.isSelected()+","+v3.vennTP.fontUnderlineA.isSelected()+","+v3.vennTP.fontShadowA.isSelected()+"\">"+v3.headerA.getText()+"</titleA>\n");
			fw.write("\t<ellipseA fill=\""+v3.circleA.getFill()+"\" stroke=\""+v3.circleA.getStroke()+"\" stroke-width=\""+v3.circleA.getStrokeWidth()+"\" size=\""+v3.vennTP.sliderSizeCircleA.getValue()+"\"> </ellipseA>\n");
			dataset =  v3.vCalc.getDatasetA();
			fw.write("\t<datasetA>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetA>\n");
			fw.write("\t<titleB lx=\""+v3.headerB.getLayoutX()+"\" ly=\""+v3.headerB.getLayoutY()+"\" angle=\""+v3.vennTP.sliderTitleAngleB.getValue()+"\" textcolor=\""+v3.vennTP.titleColorPickerB.getValue()+"\" fontSize=\""+v3.vennTP.fontSizeB.getValue()+"\" fontFamily=\""+v3.vennTP.fontFamilyB.getValue()+"\" fontStyle=\""+v3.vennTP.fontWeightB.isSelected()+","+v3.vennTP.fontPostureB.isSelected()+","+v3.vennTP.fontUnderlineB.isSelected()+","+v3.vennTP.fontShadowB.isSelected()+"\">"+v3.headerB.getText()+"</titleB>\n");
			fw.write("\t<ellipseB fill=\""+v3.circleB.getFill()+"\" stroke=\""+v3.circleB.getStroke()+"\" stroke-width=\""+v3.circleB.getStrokeWidth()+"\" size=\""+v3.vennTP.sliderSizeCircleB.getValue()+"\"> </ellipseB>\n");
			dataset =  v3.vCalc.getDatasetB();
			fw.write("\t<datasetB>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetB>\n");
			fw.write("\t<titleC lx=\""+v3.headerC.getLayoutX()+"\" ly=\""+v3.headerC.getLayoutY()+"\" angle=\""+v3.vennTP.sliderTitleAngleC.getValue()+"\" textcolor=\""+v3.vennTP.titleColorPickerC.getValue()+"\" fontSize=\""+v3.vennTP.fontSizeC.getValue()+"\" fontFamily=\""+v3.vennTP.fontFamilyC.getValue()+"\" fontStyle=\""+v3.vennTP.fontWeightC.isSelected()+","+v3.vennTP.fontPostureC.isSelected()+","+v3.vennTP.fontUnderlineC.isSelected()+","+v3.vennTP.fontShadowC.isSelected()+"\">"+v3.headerC.getText()+"</titleC>\n");
			fw.write("\t<ellipseC fill=\""+v3.circleC.getFill()+"\" stroke=\""+v3.circleC.getStroke()+"\" stroke-width=\""+v3.circleC.getStrokeWidth()+"\" size=\""+v3.vennTP.sliderSizeCircleC.getValue()+"\"> </ellipseC>\n");
			dataset =  v3.vCalc.getDatasetC();
			fw.write("\t<datasetC>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetC>\n");
			fw.write("\t<overlapValues A=\""+v3.vCalc.vennOverlap[1]+"\" B=\""+v3.vCalc.vennOverlap[2]+"\" AB=\""+v3.vCalc.vennOverlap[3]+"\" C=\""+v3.vCalc.vennOverlap[4]+"\" AC=\""+v3.vCalc.vennOverlap[5]+"\" BC=\""+v3.vCalc.vennOverlap[6]+"\" ABC=\""+v3.vCalc.vennOverlap[7]+"\" D=\""+v3.vCalc.vennOverlap[8]+"\" AD=\""+v3.vCalc.vennOverlap[9]+"\" BD=\""+v3.vCalc.vennOverlap[10]+"\" ABD=\""+v3.vCalc.vennOverlap[11]+"\" CD=\""+v3.vCalc.vennOverlap[12]+"\" ACD=\""+v3.vCalc.vennOverlap[13]+"\" BCD=\""+v3.vCalc.vennOverlap[14]+"\" ABCD=\""+v3.vCalc.vennOverlap[15]+"\" E=\""+v3.vCalc.vennOverlap[16]+"\" AE=\""+v3.vCalc.vennOverlap[17]+"\" BE=\""+v3.vCalc.vennOverlap[18]+"\" ABE=\""+v3.vCalc.vennOverlap[19]+"\" CE=\""+v3.vCalc.vennOverlap[20]+"\" ACE=\""+v3.vCalc.vennOverlap[21]+"\" BCE=\""+v3.vCalc.vennOverlap[22]+"\" ABCE=\""+v3.vCalc.vennOverlap[23]+"\" DE=\""+v3.vCalc.vennOverlap[24]+"\" ADE=\""+v3.vCalc.vennOverlap[25]+"\" BDE=\""+v3.vCalc.vennOverlap[26]+"\" ABDE=\""+v3.vCalc.vennOverlap[27]+"\" CDE=\""+v3.vCalc.vennOverlap[28]+"\" ACDE=\""+v3.vCalc.vennOverlap[29]+"\" BCDE=\""+v3.vCalc.vennOverlap[30]+"\" ABCDE=\""+v3.vCalc.vennOverlap[31]+"\"> </overlapValues>\n");
			fw.write("</venn>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveVennXML(Venn4 v4, File file) {
		try {
			ArrayList<String> dataset;
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<venn type=\""+v4.vCalc.vennType+"\">\n");
			fw.write("\t<main bgcolor=\""+v4.vennTP.colorPickerBG.getValue()+"\" valuecolor=\""+v4.vennTP.valuesColorPicker.getValue()+"\" fontSize=\""+v4.vennTP.fontSize.getValue()+"\" fontFamily=\""+v4.vennTP.fontFamily.getValue()+"\" fontStyle=\""+v4.vennTP.fontWeight.isSelected()+","+v4.vennTP.fontPosture.isSelected()+","+v4.vennTP.fontUnderline.isSelected()+","+v4.vennTP.fontShadow.isSelected()+"\"></main>\n");
			fw.write("\t<titleA lx=\""+v4.headerA.getLayoutX()+"\" ly=\""+v4.headerA.getLayoutY()+"\" angle=\""+v4.vennTP.sliderTitleAngleA.getValue()+"\" textcolor=\""+v4.vennTP.titleColorPickerA.getValue()+"\" fontSize=\""+v4.vennTP.fontSizeA.getValue()+"\" fontFamily=\""+v4.vennTP.fontFamilyA.getValue()+"\" fontStyle=\""+v4.vennTP.fontWeightA.isSelected()+","+v4.vennTP.fontPostureA.isSelected()+","+v4.vennTP.fontUnderlineA.isSelected()+","+v4.vennTP.fontShadowA.isSelected()+"\">"+v4.headerA.getText()+"</titleA>\n");
			fw.write("\t<ellipseA fill=\""+v4.circleA.getFill()+"\" stroke=\""+v4.circleA.getStroke()+"\" stroke-width=\""+v4.circleA.getStrokeWidth()+"\" size=\""+v4.vennTP.sliderSizeCircleA.getValue()+"\"> </ellipseA>\n");
			dataset =  v4.vCalc.getDatasetA();
			fw.write("\t<datasetA>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetA>\n");
			fw.write("\t<titleB lx=\""+v4.headerB.getLayoutX()+"\" ly=\""+v4.headerB.getLayoutY()+"\" angle=\""+v4.vennTP.sliderTitleAngleB.getValue()+"\" textcolor=\""+v4.vennTP.titleColorPickerB.getValue()+"\" fontSize=\""+v4.vennTP.fontSizeB.getValue()+"\" fontFamily=\""+v4.vennTP.fontFamilyB.getValue()+"\" fontStyle=\""+v4.vennTP.fontWeightB.isSelected()+","+v4.vennTP.fontPostureB.isSelected()+","+v4.vennTP.fontUnderlineB.isSelected()+","+v4.vennTP.fontShadowB.isSelected()+"\">"+v4.headerB.getText()+"</titleB>\n");
			fw.write("\t<ellipseB fill=\""+v4.circleB.getFill()+"\" stroke=\""+v4.circleB.getStroke()+"\" stroke-width=\""+v4.circleB.getStrokeWidth()+"\" size=\""+v4.vennTP.sliderSizeCircleB.getValue()+"\"> </ellipseB>\n");
			dataset =  v4.vCalc.getDatasetB();
			fw.write("\t<datasetB>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetB>\n");
			fw.write("\t<titleC lx=\""+v4.headerC.getLayoutX()+"\" ly=\""+v4.headerC.getLayoutY()+"\" angle=\""+v4.vennTP.sliderTitleAngleC.getValue()+"\" textcolor=\""+v4.vennTP.titleColorPickerC.getValue()+"\" fontSize=\""+v4.vennTP.fontSizeC.getValue()+"\" fontFamily=\""+v4.vennTP.fontFamilyC.getValue()+"\" fontStyle=\""+v4.vennTP.fontWeightC.isSelected()+","+v4.vennTP.fontPostureC.isSelected()+","+v4.vennTP.fontUnderlineC.isSelected()+","+v4.vennTP.fontShadowC.isSelected()+"\">"+v4.headerC.getText()+"</titleC>\n");
			fw.write("\t<ellipseC fill=\""+v4.circleC.getFill()+"\" stroke=\""+v4.circleC.getStroke()+"\" stroke-width=\""+v4.circleC.getStrokeWidth()+"\" size=\""+v4.vennTP.sliderSizeCircleC.getValue()+"\"> </ellipseC>\n");
			dataset =  v4.vCalc.getDatasetC();
			fw.write("\t<datasetC>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetC>\n");
			fw.write("\t<titleD lx=\""+v4.headerD.getLayoutX()+"\" ly=\""+v4.headerD.getLayoutY()+"\" angle=\""+v4.vennTP.sliderTitleAngleD.getValue()+"\" textcolor=\""+v4.vennTP.titleColorPickerD.getValue()+"\" fontSize=\""+v4.vennTP.fontSizeD.getValue()+"\" fontFamily=\""+v4.vennTP.fontFamilyD.getValue()+"\" fontStyle=\""+v4.vennTP.fontWeightD.isSelected()+","+v4.vennTP.fontPostureD.isSelected()+","+v4.vennTP.fontUnderlineD.isSelected()+","+v4.vennTP.fontShadowD.isSelected()+"\">"+v4.headerD.getText()+"</titleD>\n");
			fw.write("\t<ellipseD fill=\""+v4.circleD.getFill()+"\" stroke=\""+v4.circleD.getStroke()+"\" stroke-width=\""+v4.circleD.getStrokeWidth()+"\" size=\""+v4.vennTP.sliderSizeCircleD.getValue()+"\"> </ellipseD>\n");
			dataset =  v4.vCalc.getDatasetD();
			fw.write("\t<datasetD>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetD>\n");
			fw.write("\t<overlapValues A=\""+v4.vCalc.vennOverlap[1]+"\" B=\""+v4.vCalc.vennOverlap[2]+"\" AB=\""+v4.vCalc.vennOverlap[3]+"\" C=\""+v4.vCalc.vennOverlap[4]+"\" AC=\""+v4.vCalc.vennOverlap[5]+"\" BC=\""+v4.vCalc.vennOverlap[6]+"\" ABC=\""+v4.vCalc.vennOverlap[7]+"\" D=\""+v4.vCalc.vennOverlap[8]+"\" AD=\""+v4.vCalc.vennOverlap[9]+"\" BD=\""+v4.vCalc.vennOverlap[10]+"\" ABD=\""+v4.vCalc.vennOverlap[11]+"\" CD=\""+v4.vCalc.vennOverlap[12]+"\" ACD=\""+v4.vCalc.vennOverlap[13]+"\" BCD=\""+v4.vCalc.vennOverlap[14]+"\" ABCD=\""+v4.vCalc.vennOverlap[15]+"\" E=\""+v4.vCalc.vennOverlap[16]+"\" AE=\""+v4.vCalc.vennOverlap[17]+"\" BE=\""+v4.vCalc.vennOverlap[18]+"\" ABE=\""+v4.vCalc.vennOverlap[19]+"\" CE=\""+v4.vCalc.vennOverlap[20]+"\" ACE=\""+v4.vCalc.vennOverlap[21]+"\" BCE=\""+v4.vCalc.vennOverlap[22]+"\" ABCE=\""+v4.vCalc.vennOverlap[23]+"\" DE=\""+v4.vCalc.vennOverlap[24]+"\" ADE=\""+v4.vCalc.vennOverlap[25]+"\" BDE=\""+v4.vCalc.vennOverlap[26]+"\" ABDE=\""+v4.vCalc.vennOverlap[27]+"\" CDE=\""+v4.vCalc.vennOverlap[28]+"\" ACDE=\""+v4.vCalc.vennOverlap[29]+"\" BCDE=\""+v4.vCalc.vennOverlap[30]+"\" ABCDE=\""+v4.vCalc.vennOverlap[31]+"\"> </overlapValues>\n");
			fw.write("</venn>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveVennXML(Venn5 v5, File file) {
		try {
			ArrayList<String> dataset;
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<venn type=\""+v5.vCalc.vennType+"\">\n");
			fw.write("\t<main bgcolor=\""+v5.vennTP.colorPickerBG.getValue()+"\" valuecolor=\""+v5.vennTP.valuesColorPicker.getValue()+"\" fontSize=\""+v5.vennTP.fontSize.getValue()+"\" fontFamily=\""+v5.vennTP.fontFamily.getValue()+"\" fontStyle=\""+v5.vennTP.fontWeight.isSelected()+","+v5.vennTP.fontPosture.isSelected()+","+v5.vennTP.fontUnderline.isSelected()+","+v5.vennTP.fontShadow.isSelected()+"\"></main>\n");
			fw.write("\t<titleA lx=\""+v5.headerA.getLayoutX()+"\" ly=\""+v5.headerA.getLayoutY()+"\" angle=\""+v5.vennTP.sliderTitleAngleA.getValue()+"\" textcolor=\""+v5.vennTP.titleColorPickerA.getValue()+"\" fontSize=\""+v5.vennTP.fontSizeA.getValue()+"\" fontFamily=\""+v5.vennTP.fontFamilyA.getValue()+"\" fontStyle=\""+v5.vennTP.fontWeightA.isSelected()+","+v5.vennTP.fontPostureA.isSelected()+","+v5.vennTP.fontUnderlineA.isSelected()+","+v5.vennTP.fontShadowA.isSelected()+"\">"+v5.headerA.getText()+"</titleA>\n");
			fw.write("\t<ellipseA fill=\""+v5.circleA.getFill()+"\" stroke=\""+v5.circleA.getStroke()+"\" stroke-width=\""+v5.circleA.getStrokeWidth()+"\" size=\""+v5.vennTP.sliderSizeCircleA.getValue()+"\"> </ellipseA>\n");
			dataset =  v5.vCalc.getDatasetA();
			fw.write("\t<datasetA>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetA>\n");
			fw.write("\t<titleB lx=\""+v5.headerB.getLayoutX()+"\" ly=\""+v5.headerB.getLayoutY()+"\" angle=\""+v5.vennTP.sliderTitleAngleB.getValue()+"\" textcolor=\""+v5.vennTP.titleColorPickerB.getValue()+"\" fontSize=\""+v5.vennTP.fontSizeB.getValue()+"\" fontFamily=\""+v5.vennTP.fontFamilyB.getValue()+"\" fontStyle=\""+v5.vennTP.fontWeightB.isSelected()+","+v5.vennTP.fontPostureB.isSelected()+","+v5.vennTP.fontUnderlineB.isSelected()+","+v5.vennTP.fontShadowB.isSelected()+"\">"+v5.headerB.getText()+"</titleB>\n");
			fw.write("\t<ellipseB fill=\""+v5.circleB.getFill()+"\" stroke=\""+v5.circleB.getStroke()+"\" stroke-width=\""+v5.circleB.getStrokeWidth()+"\" size=\""+v5.vennTP.sliderSizeCircleB.getValue()+"\"> </ellipseB>\n");
			dataset =  v5.vCalc.getDatasetB();
			fw.write("\t<datasetB>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetB>\n");
			fw.write("\t<titleC lx=\""+v5.headerC.getLayoutX()+"\" ly=\""+v5.headerC.getLayoutY()+"\" angle=\""+v5.vennTP.sliderTitleAngleC.getValue()+"\" textcolor=\""+v5.vennTP.titleColorPickerC.getValue()+"\" fontSize=\""+v5.vennTP.fontSizeC.getValue()+"\" fontFamily=\""+v5.vennTP.fontFamilyC.getValue()+"\" fontStyle=\""+v5.vennTP.fontWeightC.isSelected()+","+v5.vennTP.fontPostureC.isSelected()+","+v5.vennTP.fontUnderlineC.isSelected()+","+v5.vennTP.fontShadowC.isSelected()+"\">"+v5.headerC.getText()+"</titleC>\n");
			fw.write("\t<ellipseC fill=\""+v5.circleC.getFill()+"\" stroke=\""+v5.circleC.getStroke()+"\" stroke-width=\""+v5.circleC.getStrokeWidth()+"\" size=\""+v5.vennTP.sliderSizeCircleC.getValue()+"\"> </ellipseC>\n");
			dataset =  v5.vCalc.getDatasetC();
			fw.write("\t<datasetC>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetC>\n");
			fw.write("\t<titleD lx=\""+v5.headerD.getLayoutX()+"\" ly=\""+v5.headerD.getLayoutY()+"\" angle=\""+v5.vennTP.sliderTitleAngleD.getValue()+"\" textcolor=\""+v5.vennTP.titleColorPickerD.getValue()+"\" fontSize=\""+v5.vennTP.fontSizeD.getValue()+"\" fontFamily=\""+v5.vennTP.fontFamilyD.getValue()+"\" fontStyle=\""+v5.vennTP.fontWeightD.isSelected()+","+v5.vennTP.fontPostureD.isSelected()+","+v5.vennTP.fontUnderlineD.isSelected()+","+v5.vennTP.fontShadowD.isSelected()+"\">"+v5.headerD.getText()+"</titleD>\n");
			fw.write("\t<ellipseD fill=\""+v5.circleD.getFill()+"\" stroke=\""+v5.circleD.getStroke()+"\" stroke-width=\""+v5.circleD.getStrokeWidth()+"\" size=\""+v5.vennTP.sliderSizeCircleD.getValue()+"\"> </ellipseD>\n");
			dataset =  v5.vCalc.getDatasetD();
			fw.write("\t<datasetD>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetD>\n");
			fw.write("\t<titleE lx=\""+v5.headerE.getLayoutX()+"\" ly=\""+v5.headerE.getLayoutY()+"\" angle=\""+v5.vennTP.sliderTitleAngleE.getValue()+"\" textcolor=\""+v5.vennTP.titleColorPickerE.getValue()+"\" fontSize=\""+v5.vennTP.fontSizeE.getValue()+"\" fontFamily=\""+v5.vennTP.fontFamilyE.getValue()+"\" fontStyle=\""+v5.vennTP.fontWeightE.isSelected()+","+v5.vennTP.fontPostureE.isSelected()+","+v5.vennTP.fontUnderlineE.isSelected()+","+v5.vennTP.fontShadowE.isSelected()+"\">"+v5.headerE.getText()+"</titleE>\n");
			fw.write("\t<ellipseE fill=\""+v5.circleE.getFill()+"\" stroke=\""+v5.circleE.getStroke()+"\" stroke-width=\""+v5.circleE.getStrokeWidth()+"\" size=\""+v5.vennTP.sliderSizeCircleE.getValue()+"\"> </ellipseE>\n");
			dataset =  v5.vCalc.getDatasetE();
			fw.write("\t<datasetE>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetE>\n");
			fw.write("\t<overlapValues A=\""+v5.vCalc.vennOverlap[1]+"\" B=\""+v5.vCalc.vennOverlap[2]+"\" AB=\""+v5.vCalc.vennOverlap[3]+"\" C=\""+v5.vCalc.vennOverlap[4]+"\" AC=\""+v5.vCalc.vennOverlap[5]+"\" BC=\""+v5.vCalc.vennOverlap[6]+"\" ABC=\""+v5.vCalc.vennOverlap[7]+"\" D=\""+v5.vCalc.vennOverlap[8]+"\" AD=\""+v5.vCalc.vennOverlap[9]+"\" BD=\""+v5.vCalc.vennOverlap[10]+"\" ABD=\""+v5.vCalc.vennOverlap[11]+"\" CD=\""+v5.vCalc.vennOverlap[12]+"\" ACD=\""+v5.vCalc.vennOverlap[13]+"\" BCD=\""+v5.vCalc.vennOverlap[14]+"\" ABCD=\""+v5.vCalc.vennOverlap[15]+"\" E=\""+v5.vCalc.vennOverlap[16]+"\" AE=\""+v5.vCalc.vennOverlap[17]+"\" BE=\""+v5.vCalc.vennOverlap[18]+"\" ABE=\""+v5.vCalc.vennOverlap[19]+"\" CE=\""+v5.vCalc.vennOverlap[20]+"\" ACE=\""+v5.vCalc.vennOverlap[21]+"\" BCE=\""+v5.vCalc.vennOverlap[22]+"\" ABCE=\""+v5.vCalc.vennOverlap[23]+"\" DE=\""+v5.vCalc.vennOverlap[24]+"\" ADE=\""+v5.vCalc.vennOverlap[25]+"\" BDE=\""+v5.vCalc.vennOverlap[26]+"\" ABDE=\""+v5.vCalc.vennOverlap[27]+"\" CDE=\""+v5.vCalc.vennOverlap[28]+"\" ACDE=\""+v5.vCalc.vennOverlap[29]+"\" BCDE=\""+v5.vCalc.vennOverlap[30]+"\" ABCDE=\""+v5.vCalc.vennOverlap[31]+"\"> </overlapValues>\n");
			fw.write("</venn>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveVennXML(Euler1 e1, File file) {
		try {
			ArrayList<String> dataset;
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<venn type=\""+e1.vCalc.vennType+"\">\n");
			fw.write("\t<main bgcolor=\""+e1.vennTP.colorPickerBG.getValue()+"\" valuecolor=\""+e1.vennTP.valuesColorPicker.getValue()+"\" fontSize=\""+e1.vennTP.fontSize.getValue()+"\" fontFamily=\""+e1.vennTP.fontFamily.getValue()+"\" fontStyle=\""+e1.vennTP.fontWeight.isSelected()+","+e1.vennTP.fontPosture.isSelected()+","+e1.vennTP.fontUnderline.isSelected()+","+e1.vennTP.fontShadow.isSelected()+"\"></main>\n");
			fw.write("\t<titleA lx=\""+e1.headerA.getLayoutX()+"\" ly=\""+e1.headerA.getLayoutY()+"\" angle=\""+e1.vennTP.sliderTitleAngleA.getValue()+"\" textcolor=\""+e1.vennTP.titleColorPickerA.getValue()+"\" fontSize=\""+e1.vennTP.fontSizeA.getValue()+"\" fontFamily=\""+e1.vennTP.fontFamilyA.getValue()+"\" fontStyle=\""+e1.vennTP.fontWeightA.isSelected()+","+e1.vennTP.fontPostureA.isSelected()+","+e1.vennTP.fontUnderlineA.isSelected()+","+e1.vennTP.fontShadowA.isSelected()+"\">"+e1.headerA.getText()+"</titleA>\n");
			fw.write("\t<ellipseA fill=\""+e1.circleA.getFill()+"\" stroke=\""+e1.circleA.getStroke()+"\" stroke-width=\""+e1.circleA.getStrokeWidth()+"\" size=\""+e1.vennTP.sliderSizeCircleA.getValue()+"\"> </ellipseA>\n");
			dataset =  e1.vCalc.getDatasetA();
			fw.write("\t<datasetA>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetA>\n");
			fw.write("\t<overlapValues A=\""+e1.vCalc.vennOverlap[1]+"\" B=\""+e1.vCalc.vennOverlap[2]+"\" AB=\""+e1.vCalc.vennOverlap[3]+"\" C=\""+e1.vCalc.vennOverlap[4]+"\" AC=\""+e1.vCalc.vennOverlap[5]+"\" BC=\""+e1.vCalc.vennOverlap[6]+"\" ABC=\""+e1.vCalc.vennOverlap[7]+"\" D=\""+e1.vCalc.vennOverlap[8]+"\" AD=\""+e1.vCalc.vennOverlap[9]+"\" BD=\""+e1.vCalc.vennOverlap[10]+"\" ABD=\""+e1.vCalc.vennOverlap[11]+"\" CD=\""+e1.vCalc.vennOverlap[12]+"\" ACD=\""+e1.vCalc.vennOverlap[13]+"\" BCD=\""+e1.vCalc.vennOverlap[14]+"\" ABCD=\""+e1.vCalc.vennOverlap[15]+"\" E=\""+e1.vCalc.vennOverlap[16]+"\" AE=\""+e1.vCalc.vennOverlap[17]+"\" BE=\""+e1.vCalc.vennOverlap[18]+"\" ABE=\""+e1.vCalc.vennOverlap[19]+"\" CE=\""+e1.vCalc.vennOverlap[20]+"\" ACE=\""+e1.vCalc.vennOverlap[21]+"\" BCE=\""+e1.vCalc.vennOverlap[22]+"\" ABCE=\""+e1.vCalc.vennOverlap[23]+"\" DE=\""+e1.vCalc.vennOverlap[24]+"\" ADE=\""+e1.vCalc.vennOverlap[25]+"\" BDE=\""+e1.vCalc.vennOverlap[26]+"\" ABDE=\""+e1.vCalc.vennOverlap[27]+"\" CDE=\""+e1.vCalc.vennOverlap[28]+"\" ACDE=\""+e1.vCalc.vennOverlap[29]+"\" BCDE=\""+e1.vCalc.vennOverlap[30]+"\" ABCDE=\""+e1.vCalc.vennOverlap[31]+"\"> </overlapValues>\n");
		fw.write("</venn>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveVennXML(Euler2 e2, File file) {
		try {
			ArrayList<String> dataset;
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<venn type=\""+e2.vCalc.vennType+"\">\n");
			fw.write("\t<main bgcolor=\""+e2.vennTP.colorPickerBG.getValue()+"\" valuecolor=\""+e2.vennTP.valuesColorPicker.getValue()+"\" fontSize=\""+e2.vennTP.fontSize.getValue()+"\" fontFamily=\""+e2.vennTP.fontFamily.getValue()+"\" fontStyle=\""+e2.vennTP.fontWeight.isSelected()+","+e2.vennTP.fontPosture.isSelected()+","+e2.vennTP.fontUnderline.isSelected()+","+e2.vennTP.fontShadow.isSelected()+"\"></main>\n");
			fw.write("\t<titleA lx=\""+e2.headerA.getLayoutX()+"\" ly=\""+e2.headerA.getLayoutY()+"\" angle=\""+e2.vennTP.sliderTitleAngleA.getValue()+"\" textcolor=\""+e2.vennTP.titleColorPickerA.getValue()+"\" fontSize=\""+e2.vennTP.fontSizeA.getValue()+"\" fontFamily=\""+e2.vennTP.fontFamilyA.getValue()+"\" fontStyle=\""+e2.vennTP.fontWeightA.isSelected()+","+e2.vennTP.fontPostureA.isSelected()+","+e2.vennTP.fontUnderlineA.isSelected()+","+e2.vennTP.fontShadowA.isSelected()+"\">"+e2.headerA.getText()+"</titleA>\n");
			fw.write("\t<ellipseA fill=\""+e2.circleA.getFill()+"\" stroke=\""+e2.circleA.getStroke()+"\" stroke-width=\""+e2.circleA.getStrokeWidth()+"\" size=\""+e2.vennTP.sliderSizeCircleA.getValue()+"\"> </ellipseA>\n");
			dataset =  e2.vCalc.getDatasetA();
			fw.write("\t<datasetA>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetA>\n");
			fw.write("\t<titleB lx=\""+e2.headerB.getLayoutX()+"\" ly=\""+e2.headerB.getLayoutY()+"\" angle=\""+e2.vennTP.sliderTitleAngleB.getValue()+"\" textcolor=\""+e2.vennTP.titleColorPickerB.getValue()+"\" fontSize=\""+e2.vennTP.fontSizeB.getValue()+"\" fontFamily=\""+e2.vennTP.fontFamilyB.getValue()+"\" fontStyle=\""+e2.vennTP.fontWeightB.isSelected()+","+e2.vennTP.fontPostureB.isSelected()+","+e2.vennTP.fontUnderlineB.isSelected()+","+e2.vennTP.fontShadowB.isSelected()+"\">"+e2.headerB.getText()+"</titleB>\n");
			fw.write("\t<ellipseB fill=\""+e2.circleB.getFill()+"\" stroke=\""+e2.circleB.getStroke()+"\" stroke-width=\""+e2.circleB.getStrokeWidth()+"\" size=\""+e2.vennTP.sliderSizeCircleB.getValue()+"\"> </ellipseB>\n");
			dataset =  e2.vCalc.getDatasetB();
			fw.write("\t<datasetB>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetB>\n");
			fw.write("\t<overlapValues A=\""+e2.vCalc.vennOverlap[1]+"\" B=\""+e2.vCalc.vennOverlap[2]+"\" AB=\""+e2.vCalc.vennOverlap[3]+"\" C=\""+e2.vCalc.vennOverlap[4]+"\" AC=\""+e2.vCalc.vennOverlap[5]+"\" BC=\""+e2.vCalc.vennOverlap[6]+"\" ABC=\""+e2.vCalc.vennOverlap[7]+"\" D=\""+e2.vCalc.vennOverlap[8]+"\" AD=\""+e2.vCalc.vennOverlap[9]+"\" BD=\""+e2.vCalc.vennOverlap[10]+"\" ABD=\""+e2.vCalc.vennOverlap[11]+"\" CD=\""+e2.vCalc.vennOverlap[12]+"\" ACD=\""+e2.vCalc.vennOverlap[13]+"\" BCD=\""+e2.vCalc.vennOverlap[14]+"\" ABCD=\""+e2.vCalc.vennOverlap[15]+"\" E=\""+e2.vCalc.vennOverlap[16]+"\" AE=\""+e2.vCalc.vennOverlap[17]+"\" BE=\""+e2.vCalc.vennOverlap[18]+"\" ABE=\""+e2.vCalc.vennOverlap[19]+"\" CE=\""+e2.vCalc.vennOverlap[20]+"\" ACE=\""+e2.vCalc.vennOverlap[21]+"\" BCE=\""+e2.vCalc.vennOverlap[22]+"\" ABCE=\""+e2.vCalc.vennOverlap[23]+"\" DE=\""+e2.vCalc.vennOverlap[24]+"\" ADE=\""+e2.vCalc.vennOverlap[25]+"\" BDE=\""+e2.vCalc.vennOverlap[26]+"\" ABDE=\""+e2.vCalc.vennOverlap[27]+"\" CDE=\""+e2.vCalc.vennOverlap[28]+"\" ACDE=\""+e2.vCalc.vennOverlap[29]+"\" BCDE=\""+e2.vCalc.vennOverlap[30]+"\" ABCDE=\""+e2.vCalc.vennOverlap[31]+"\"> </overlapValues>\n");
			fw.write("</venn>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveVennXML(Euler3 e3, File file) {
		try {
			ArrayList<String> dataset;
			FileWriter fw = new FileWriter(file);
			fw.write("<?xml version=\"1.0\"?>\n");
			fw.write("<venn type=\""+e3.vCalc.vennType+"\">\n");
			fw.write("\t<main bgcolor=\""+e3.vennTP.colorPickerBG.getValue()+"\" valuecolor=\""+e3.vennTP.valuesColorPicker.getValue()+"\" fontSize=\""+e3.vennTP.fontSize.getValue()+"\" fontFamily=\""+e3.vennTP.fontFamily.getValue()+"\" fontStyle=\""+e3.vennTP.fontWeight.isSelected()+","+e3.vennTP.fontPosture.isSelected()+","+e3.vennTP.fontUnderline.isSelected()+","+e3.vennTP.fontShadow.isSelected()+"\"></main>\n");
			fw.write("\t<titleA lx=\""+e3.headerA.getLayoutX()+"\" ly=\""+e3.headerA.getLayoutY()+"\" angle=\""+e3.vennTP.sliderTitleAngleA.getValue()+"\" textcolor=\""+e3.vennTP.titleColorPickerA.getValue()+"\" fontSize=\""+e3.vennTP.fontSizeA.getValue()+"\" fontFamily=\""+e3.vennTP.fontFamilyA.getValue()+"\" fontStyle=\""+e3.vennTP.fontWeightA.isSelected()+","+e3.vennTP.fontPostureA.isSelected()+","+e3.vennTP.fontUnderlineA.isSelected()+","+e3.vennTP.fontShadowA.isSelected()+"\">"+e3.headerA.getText()+"</titleA>\n");
			fw.write("\t<ellipseA fill=\""+e3.circleA.getFill()+"\" stroke=\""+e3.circleA.getStroke()+"\" stroke-width=\""+e3.circleA.getStrokeWidth()+"\" size=\""+e3.vennTP.sliderSizeCircleA.getValue()+"\"> </ellipseA>\n");
			dataset =  e3.vCalc.getDatasetA();
			fw.write("\t<datasetA>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetA>\n");
			fw.write("\t<titleB lx=\""+e3.headerB.getLayoutX()+"\" ly=\""+e3.headerB.getLayoutY()+"\" angle=\""+e3.vennTP.sliderTitleAngleB.getValue()+"\" textcolor=\""+e3.vennTP.titleColorPickerB.getValue()+"\" fontSize=\""+e3.vennTP.fontSizeB.getValue()+"\" fontFamily=\""+e3.vennTP.fontFamilyB.getValue()+"\" fontStyle=\""+e3.vennTP.fontWeightB.isSelected()+","+e3.vennTP.fontPostureB.isSelected()+","+e3.vennTP.fontUnderlineB.isSelected()+","+e3.vennTP.fontShadowB.isSelected()+"\">"+e3.headerB.getText()+"</titleB>\n");
			fw.write("\t<ellipseB fill=\""+e3.circleB.getFill()+"\" stroke=\""+e3.circleB.getStroke()+"\" stroke-width=\""+e3.circleB.getStrokeWidth()+"\" size=\""+e3.vennTP.sliderSizeCircleB.getValue()+"\"> </ellipseB>\n");
			dataset =  e3.vCalc.getDatasetB();
			fw.write("\t<datasetB>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetB>\n");
			fw.write("\t<titleC lx=\""+e3.headerC.getLayoutX()+"\" ly=\""+e3.headerC.getLayoutY()+"\" angle=\""+e3.vennTP.sliderTitleAngleC.getValue()+"\" textcolor=\""+e3.vennTP.titleColorPickerC.getValue()+"\" fontSize=\""+e3.vennTP.fontSizeC.getValue()+"\" fontFamily=\""+e3.vennTP.fontFamilyC.getValue()+"\" fontStyle=\""+e3.vennTP.fontWeightC.isSelected()+","+e3.vennTP.fontPostureC.isSelected()+","+e3.vennTP.fontUnderlineC.isSelected()+","+e3.vennTP.fontShadowC.isSelected()+"\">"+e3.headerC.getText()+"</titleC>\n");
			fw.write("\t<ellipseC fill=\""+e3.circleC.getFill()+"\" stroke=\""+e3.circleC.getStroke()+"\" stroke-width=\""+e3.circleC.getStrokeWidth()+"\" size=\""+e3.vennTP.sliderSizeCircleC.getValue()+"\"> </ellipseC>\n");
			dataset =  e3.vCalc.getDatasetC();
			fw.write("\t<datasetC>");
			for (int i=0; i<dataset.size(); i++){
				fw.write(dataset.get(i)+"|~|");
			}
			fw.write("</datasetC>\n");
			fw.write("\t<overlapValues A=\""+e3.vCalc.vennOverlap[1]+"\" B=\""+e3.vCalc.vennOverlap[2]+"\" AB=\""+e3.vCalc.vennOverlap[3]+"\" C=\""+e3.vCalc.vennOverlap[4]+"\" AC=\""+e3.vCalc.vennOverlap[5]+"\" BC=\""+e3.vCalc.vennOverlap[6]+"\" ABC=\""+e3.vCalc.vennOverlap[7]+"\" D=\""+e3.vCalc.vennOverlap[8]+"\" AD=\""+e3.vCalc.vennOverlap[9]+"\" BD=\""+e3.vCalc.vennOverlap[10]+"\" ABD=\""+e3.vCalc.vennOverlap[11]+"\" CD=\""+e3.vCalc.vennOverlap[12]+"\" ACD=\""+e3.vCalc.vennOverlap[13]+"\" BCD=\""+e3.vCalc.vennOverlap[14]+"\" ABCD=\""+e3.vCalc.vennOverlap[15]+"\" E=\""+e3.vCalc.vennOverlap[16]+"\" AE=\""+e3.vCalc.vennOverlap[17]+"\" BE=\""+e3.vCalc.vennOverlap[18]+"\" ABE=\""+e3.vCalc.vennOverlap[19]+"\" CE=\""+e3.vCalc.vennOverlap[20]+"\" ACE=\""+e3.vCalc.vennOverlap[21]+"\" BCE=\""+e3.vCalc.vennOverlap[22]+"\" ABCE=\""+e3.vCalc.vennOverlap[23]+"\" DE=\""+e3.vCalc.vennOverlap[24]+"\" ADE=\""+e3.vCalc.vennOverlap[25]+"\" BDE=\""+e3.vCalc.vennOverlap[26]+"\" ABDE=\""+e3.vCalc.vennOverlap[27]+"\" CDE=\""+e3.vCalc.vennOverlap[28]+"\" ACDE=\""+e3.vCalc.vennOverlap[29]+"\" BCDE=\""+e3.vCalc.vennOverlap[30]+"\" ABCDE=\""+e3.vCalc.vennOverlap[31]+"\"> </overlapValues>\n");
			fw.write("</venn>\n");
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveImage(File file, BorderPane vennPlotBP) {
		String[] tmp = file.toString().split("\\.");
		String ext = tmp[tmp.length-1];
		int iw = (int)(vennPlotBP.getWidth() * res);
		int ih = (int)(vennPlotBP.getHeight() * res);
		if (ext.equalsIgnoreCase("png")) {
			vennPlotBP.setScaleX(res);
			vennPlotBP.setScaleY(res);
			WritableImage wim = new WritableImage(iw, ih);
			final SnapshotParameters snapshotParameters = new SnapshotParameters();
			snapshotParameters.setFill(Color.TRANSPARENT);
			vennPlotBP.snapshot(snapshotParameters, wim);
			try {
				ImageIO.write(SwingFXUtils.fromFXImage(wim, null), "png", file);
			} catch (Exception ex) {
				System.out.println(ex.getMessage());
			}
			vennPlotBP.setScaleX(1);
			vennPlotBP.setScaleY(1);
		}
		if (ext.equalsIgnoreCase("jpg")) {
			vennPlotBP.setScaleX(res);
			vennPlotBP.setScaleY(res);
			WritableImage wim = new WritableImage(iw, ih);
			final SnapshotParameters snapshotParameters = new SnapshotParameters();
			snapshotParameters.setFill(null);
			vennPlotBP.snapshot(snapshotParameters, wim);
			BufferedImage image = SwingFXUtils.fromFXImage(wim, null);
			BufferedImage imageRGB = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.OPAQUE);
			Graphics2D graphics = imageRGB.createGraphics();
			graphics.drawImage(image, 0, 0, null);
			try {
				ImageIO.write(imageRGB, "jpg", file);
			} catch (Exception ex) {
				System.out.println(ex.getMessage());
			}
			graphics.dispose();
		}
		if (ext.equalsIgnoreCase("bmp")) {
			vennPlotBP.setScaleX(res);
			vennPlotBP.setScaleY(res);
			WritableImage wim = new WritableImage(iw, ih);
			final SnapshotParameters snapshotParameters = new SnapshotParameters();
			snapshotParameters.setFill(null);
			vennPlotBP.snapshot(snapshotParameters, wim);
			BufferedImage image = SwingFXUtils.fromFXImage(wim, null);
			BufferedImage imageRGB = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.OPAQUE);
			Graphics2D graphics = imageRGB.createGraphics();
			graphics.drawImage(image, 0, 0, null);
			try {
				ImageIO.write(imageRGB, "bmp", file);
			} catch (Exception ex) {
				System.out.println(ex.getMessage());
			}
			graphics.dispose();
		}
		if (ext.equalsIgnoreCase("svg")) {
			FX2SVG.saveSvg(file.toString(), vennPlotBP);
		}
	}
}



