package vennDIS;

// Copyright (C) 2014 Vladimir Ignatchenko (vladimirsign@gmail.com)
// Dr. Thomas Kislinger laboratory (http://kislingerlab.uhnres.utoronto.ca/)
//
// This file is part of VennDIS software.
// VennDIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VennDIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VennDIS. If not, see <http://www.gnu.org/licenses/>.

import javafx.scene.control.Button;
import javafx.scene.control.Separator;
import javafx.scene.control.Toggle;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.control.ToolBar;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.geometry.Insets;

public class VennToolBar {

	public BorderPane toolBar;
	public ToggleGroup resGroup;
	public Button btnNew, btnEdit, btnOpen, btnSave, btnSaveImage, btnHelp, btnWFS;
	public ToggleButton btnResN, btnResH, btnResUH;
	private Image imNew, imEdit, imOpen, imSave, imSaveImage, imHelp, imWFS;

	public VennToolBar(BorderPane borderPane) {
		imNew = new Image(getClass().getResourceAsStream("icons/new32.png"));
		imEdit = new Image(getClass().getResourceAsStream("icons/edit32.png"));
		imOpen = new Image(getClass().getResourceAsStream("icons/open32.png"));
		imSave = new Image(getClass().getResourceAsStream("icons/save32.png"));
		imSaveImage = new Image(getClass().getResourceAsStream("icons/save_image32.png"));
		imHelp = new Image(getClass().getResourceAsStream("icons/help32.png"));
		imWFS = new Image(getClass().getResourceAsStream("icons/wfs32.png"));
		makeToolBar();
		borderPane.setTop(toolBar);
	}

	private void makeToolBar() {
		toolBar = new BorderPane();
		toolBar.setId("mainToolBar");
			HBox leftBar = new HBox();
			leftBar.setPadding(new Insets(5, 5, 5, 5));
			leftBar.setSpacing(5);
				btnNew = new Button("New  ");
				btnNew.setId("new");
				btnNew.setTooltip(new Tooltip("Create new Venn/Euler diagram"));
				btnNew.setGraphic(new ImageView(imNew));
				btnNew.setStyle("-fx-padding: 5px;");

				btnEdit = new Button("Edit  ");
				btnEdit.setId("edit");
				btnEdit.setTooltip(new Tooltip("Edit input data"));
				btnEdit.setGraphic(new ImageView(imEdit));
				btnEdit.setStyle("-fx-padding: 5px;");

				btnOpen = new Button("Open ");
				btnOpen.setId("open");
				btnOpen.setTooltip(new Tooltip("Open diagram from XML file"));
				btnOpen.setGraphic(new ImageView(imOpen));
				btnSave = new Button("Save As");
				btnSave.setId("save");
				btnSave.setTooltip(new Tooltip("Save diagram as XML file"));
				btnSave.setGraphic(new ImageView(imSave));
				btnSaveImage = new Button("Export");
				btnSaveImage.setId("saveimage");
				btnSaveImage.setTooltip(new Tooltip("Export diagram as bitmap image"));
				btnSaveImage.setGraphic(new ImageView(imSaveImage));
			leftBar.getChildren().addAll(btnNew, btnEdit, btnOpen, btnSave, btnSaveImage);

			HBox rightBar = new HBox();
			rightBar.setPadding(new Insets(5, 5, 5, 5));
			rightBar.setSpacing(5);
				btnHelp = new Button("Help");
				btnHelp.setId("help");
				btnHelp.setTooltip(new Tooltip("Help"));
				btnHelp.setGraphic(new ImageView(imHelp));

				btnWFS = new Button("Full-screen");
				btnWFS.setId("wfs");
				btnWFS.setTooltip(new Tooltip("Full-screen mode"));
				btnWFS.setGraphic(new ImageView(imWFS));
			rightBar.getChildren().addAll(btnHelp, btnWFS);
		toolBar.setLeft(leftBar);
		toolBar.setRight(rightBar);
	}
}



