package vennDIS;

import java.io.*;
import java.util.regex.*;
import java.util.ArrayList;

public class Fasta {

	public ArrayList<fastaStructure> fastaArray = new  ArrayList<fastaStructure>();
	public class fastaStructure {
		public String LBL = "";
		public String SEQ = "";
	}

	public void readFasta(String filename) {
		System.out.println("reading "+filename+" ...");
		fastaArray = new  ArrayList<fastaStructure>();
		fastaStructure fastaElement = new fastaStructure();
		try {
			FileReader fr=new FileReader(filename);
			BufferedReader fbr = new BufferedReader(fr);
			String strLine;
			Pattern p;
			Matcher m;
			int firstLine = 1;
			while ((strLine = fbr.readLine()) != null) {
				p = Pattern.compile("^>");
				m = p.matcher(strLine);
				if (m.find()) {
					if (firstLine==1) { firstLine = 0; } else { fastaArray.add(fastaElement); }
					fastaElement = new fastaStructure();
					fastaElement.LBL = strLine;
				} else {
					fastaElement.SEQ += strLine;
				}
			}
			fastaArray.add(fastaElement);
		} catch (Exception e) {
			System.err.println("Error: " + e.getMessage());
		}
	}

	public void appendFasta(String filename) {
		System.out.println("reading "+filename+" ...");
		fastaStructure fastaElement = new fastaStructure();
		try {
			FileReader fr=new FileReader(filename);
			BufferedReader fbr = new BufferedReader(fr);
			String strLine;
			Pattern p;
			Matcher m;
			int firstLine = 1;
			while ((strLine = fbr.readLine()) != null) {
				p = Pattern.compile("^>");
				m = p.matcher(strLine);
				if (m.find()) {
					if (firstLine==1) { firstLine = 0; } else { fastaArray.add(fastaElement); }
					fastaElement = new fastaStructure();
					fastaElement.LBL = strLine;
				} else {
					fastaElement.SEQ += strLine;
				}
			}
			fastaArray.add(fastaElement);
		} catch (Exception e) {
			System.err.println("Error: " + e.getMessage());
		}
	}

	public void writeFasta(String filename) {
		System.out.println("writing "+filename+" ...");
		try {
			FileWriter fw = new FileWriter(filename);
			for (int i=0; i<fastaArray.size(); i++) {
				fw.write(fastaArray.get(i).LBL.toString()+"\n");
				fw.write(fastaArray.get(i).SEQ.toString()+"\n");
			}
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void replaceLItoJ() {
		for (int i=0; i<fastaArray.size(); i++) {
			fastaArray.get(i).SEQ = fastaArray.get(i).SEQ.replace("L", "J");
			fastaArray.get(i).SEQ = fastaArray.get(i).SEQ.replace("I", "J");
		}
	}

	public ArrayList<Integer> findPeptideJ(String peptide) {
		peptide = peptide.replace("L", "J");
		peptide = peptide.replace("I", "J");
		ArrayList<Integer> fastaIndex = new  ArrayList<Integer>();
		for (int i=0; i<fastaArray.size(); i++) {
			if (fastaArray.get(i).SEQ.contains(peptide)) {
				fastaIndex.add(i);
			}
		}
		return(fastaIndex);
	}

	public ArrayList<Integer> findPeptide(String peptide) {
		ArrayList<Integer> fastaIndex = new  ArrayList<Integer>();
		for (int i=0; i<fastaArray.size(); i++) {
			if (fastaArray.get(i).SEQ.contains(peptide)) {
				fastaIndex.add(i);
			}
		}
		return(fastaIndex);
	}

	public String getSwpAC(int index) {
		String swpAC = "";
		Pattern p = Pattern.compile(">sp\\|([A-Z0-9]+)\\|");
		Matcher m = p.matcher(fastaArray.get(index).LBL.toString());
		if (m.find()) {
			swpAC = m.group(1);
		}
		return(swpAC);
	}

	public String getSwpID(int index) {
		String swpID = "";
		Pattern p = Pattern.compile(">sp\\|[A-Z0-9]+\\|([A-Z0-9_]+) ");
		Matcher m = p.matcher(fastaArray.get(index).LBL.toString());
		if (m.find()) {
			swpID = m.group(1);
		}
		return(swpID);
	}

	public String getSwpGN(int index) {
		String swpGN = "";
		Pattern p = Pattern.compile("GN=([A-Za-z0-9_;\\-]+) ");
		Matcher m = p.matcher(fastaArray.get(index).LBL.toString());
		if (m.find()) {
			swpGN = m.group(1);
		}
		return(swpGN);
	}

	public String getSwpOS(int index) {
		String swpOS = "";
		Pattern p = Pattern.compile("OS=([A-Za-z0-9 ]+)( GN| PE)");
		Matcher m = p.matcher(fastaArray.get(index).LBL.toString());
		if (m.find()) {
			swpOS = m.group(1);
		}
		return(swpOS);
	}

	public String getSwpDSC(int index) {
		String swpDSC = "";
		Pattern p = Pattern.compile(" (.+) OS=");
		Matcher m = p.matcher(fastaArray.get(index).LBL.toString());
		if (m.find()) {
			swpDSC = m.group(1);
		}
		return(swpDSC);
	}

	public ArrayList<String> getSwpAC(ArrayList<Integer> indexList) {
		ArrayList<String> acList = new ArrayList<String>();
		Pattern p = Pattern.compile(">sp\\|([A-Z0-9]+)\\|");
		Matcher m;
		for (int i=0; i<indexList.size(); i++) {
			m = p.matcher(fastaArray.get(indexList.get(i)).LBL.toString());
			if (m.find()) {
				acList.add(m.group(1));
			} else {
				acList.add("");
			}
		}
		return(acList);
	}

	public ArrayList<String> getSwpID(ArrayList<Integer> indexList) {
		ArrayList<String> idList = new ArrayList<String>();
		Pattern p = Pattern.compile(">sp\\|[A-Z0-9]+\\|([A-Z0-9_]+) ");
		Matcher m;
		for (int i=0; i<indexList.size(); i++) {
			m = p.matcher(fastaArray.get(indexList.get(i)).LBL.toString());
			if (m.find()) {
				idList.add(m.group(1));
			} else {
				idList.add("");
			}
		}
		return(idList);
	}

	public ArrayList<String> getSwpGN(ArrayList<Integer> indexList) {
		ArrayList<String> gnList = new ArrayList<String>();
		Pattern p = Pattern.compile("GN=([A-Za-z0-9_;\\-]+) ");
		Matcher m;
		for (int i=0; i<indexList.size(); i++) {
			m = p.matcher(fastaArray.get(indexList.get(i)).LBL.toString());
			if (m.find()) {
				gnList.add(m.group(1));
			} else {
				gnList.add("");
			}
		}
		return(gnList);
	}

	public ArrayList<String> getSwpOS(ArrayList<Integer> indexList) {
		ArrayList<String> osList = new ArrayList<String>();
		Pattern p = Pattern.compile("OS=([A-Za-z0-9 ]+)( GN| PE)");
		Matcher m;
		for (int i=0; i<indexList.size(); i++) {
			m = p.matcher(fastaArray.get(indexList.get(i)).LBL.toString());
			if (m.find()) {
				osList.add(m.group(1));
			} else {
				osList.add("");
			}
		}
		return(osList);
	}

	public ArrayList<String> getSwpDSC(ArrayList<Integer> indexList) {
		ArrayList<String> dscList = new ArrayList<String>();
		Pattern p = Pattern.compile(" (.+) OS=");
		Matcher m;
		for (int i=0; i<indexList.size(); i++) {
			m = p.matcher(fastaArray.get(indexList.get(i)).LBL.toString());
			if (m.find()) {
				dscList.add(m.group(1));
			} else {
				dscList.add("");
			}
		}
		return(dscList);
	}
}

